@@ -1,5 +1,5 @@
 /*
- * Copyright 2017-present MongoDB, Inc.
+ * Copyright 2017 MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,20 +26,17 @@
 zend_class_entry* php_phongo_binary_interface_ce;
 
 /* {{{ MongoDB\BSON\BinaryInterface function entries */
-/* clang-format off */
-ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_BinaryInterface___toString, 0, 0, IS_STRING, 0)
-ZEND_END_ARG_INFO()
-
 ZEND_BEGIN_ARG_INFO_EX(ai_BinaryInterface_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_binary_interface_me[] = {
+	/* clang-format off */
 	ZEND_ABSTRACT_ME(BinaryInterface, getData, ai_BinaryInterface_void)
 	ZEND_ABSTRACT_ME(BinaryInterface, getType, ai_BinaryInterface_void)
-	ZEND_ABSTRACT_ME(BinaryInterface, __toString, ai_BinaryInterface___toString)
+	ZEND_ABSTRACT_ME(BinaryInterface, __toString, ai_BinaryInterface_void)
 	PHP_FE_END
+	/* clang-format on */
 };
-/* clang-format on */
 /* }}} */
 
 void php_phongo_binary_interface_init_ce(INIT_FUNC_ARGS) /* {{{ */
