@@ -14,24 +14,23 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+
 #include <php.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/php_var.h>
 #include <zend_smart_str.h>
+#include <ext/standard/php_var.h>
+#include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
-#include "php_bson.h"
+#include "phongo_bson_encode.h"
+#include "phongo_error.h"
+#include "Javascript_arginfo.h"
 
 zend_class_entry* php_phongo_javascript_ce;
 
 /* Initialize the object and return whether it was successful. An exception will
  * be thrown on error. */
-static bool php_phongo_javascript_init(php_phongo_javascript_t* intern, const char* code, size_t code_len, zval* scope) /* {{{ */
+static bool php_phongo_javascript_init(php_phongo_javascript_t* intern, const char* code, size_t code_len, zval* scope)
 {
 	if (scope && Z_TYPE_P(scope) != IS_OBJECT && Z_TYPE_P(scope) != IS_ARRAY && Z_TYPE_P(scope) != IS_NULL) {
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected scope to be array or object, %s given", zend_get_type_by_const(Z_TYPE_P(scope)));
@@ -54,11 +53,11 @@ static bool php_phongo_javascript_init(php_phongo_javascript_t* intern, const ch
 	}
 
 	return true;
-} /* }}} */
+}
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_javascript_init_from_hash(php_phongo_javascript_t* intern, HashTable* props) /* {{{ */
+static bool php_phongo_javascript_init_from_hash(php_phongo_javascript_t* intern, HashTable* props)
 {
 	zval *code, *scope;
 
@@ -70,9 +69,9 @@ static bool php_phongo_javascript_init_from_hash(php_phongo_javascript_t* intern
 
 	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "%s initialization requires \"code\" string field", ZSTR_VAL(php_phongo_javascript_ce->name));
 	return false;
-} /* }}} */
+}
 
-HashTable* php_phongo_javascript_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp) /* {{{ */
+HashTable* php_phongo_javascript_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp)
 {
 	php_phongo_javascript_t* intern;
 	HashTable*               props;
@@ -95,7 +94,7 @@ HashTable* php_phongo_javascript_get_properties_hash(phongo_compat_object_handle
 			php_phongo_bson_state state;
 
 			PHONGO_BSON_INIT_STATE(state);
-			if (!php_phongo_bson_to_zval_ex(bson_get_data(intern->scope), intern->scope->len, &state)) {
+			if (!php_phongo_bson_to_zval_ex(intern->scope, &state)) {
 				zval_ptr_dtor(&state.zchild);
 				goto failure;
 			}
@@ -114,15 +113,13 @@ HashTable* php_phongo_javascript_get_properties_hash(phongo_compat_object_handle
 failure:
 	PHONGO_GET_PROPERTY_HASH_FREE_PROPS(is_temp, props);
 	return NULL;
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\Javascript::__construct(string $code[, array|object $scope])
-   Construct a new BSON Javascript type. The scope is a document mapping
+/* Construct a new BSON Javascript type. The scope is a document mapping
    identifiers and values, representing the scope in which the code string will
    be evaluated. Note that this type cannot be represented as Extended JSON. */
-static PHP_METHOD(Javascript, __construct)
+static PHP_METHOD(MongoDB_BSON_Javascript, __construct)
 {
-	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 	char*                    code;
 	size_t                   code_len;
@@ -130,31 +127,24 @@ static PHP_METHOD(Javascript, __construct)
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|A!", &code, &code_len, &scope) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 2)
+	Z_PARAM_STRING(code, code_len)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_OBJECT_EX(scope, 1, 0)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_javascript_init(intern, code, code_len, scope);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\BSON\Javascript MongoDB\BSON\Javascript::__set_state(array $properties)
-*/
-static PHP_METHOD(Javascript, __set_state)
+static PHP_METHOD(MongoDB_BSON_Javascript, __set_state)
 {
-	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 	HashTable*               props;
 	zval*                    array;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(array)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	object_init_ex(return_value, php_phongo_javascript_ce);
 
@@ -162,59 +152,36 @@ static PHP_METHOD(Javascript, __set_state)
 	props  = Z_ARRVAL_P(array);
 
 	php_phongo_javascript_init_from_hash(intern, props);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\Javascript::__toString()
-   Return the Javascript's code string. */
-static PHP_METHOD(Javascript, __toString)
+/* Return the Javascript's code string. */
+static PHP_METHOD(MongoDB_BSON_Javascript, __toString)
 {
-	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
 	RETURN_STRINGL(intern->code, intern->code_len);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\Javascript::getCode()
-*/
-static PHP_METHOD(Javascript, getCode)
+static PHP_METHOD(MongoDB_BSON_Javascript, getCode)
 {
-	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
 	RETURN_STRINGL(intern->code, intern->code_len);
-} /* }}} */
+}
 
-/* {{{ proto object|null MongoDB\BSON\Javascript::getScope()
-*/
-static PHP_METHOD(Javascript, getScope)
+static PHP_METHOD(MongoDB_BSON_Javascript, getScope)
 {
-	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
@@ -227,7 +194,7 @@ static PHP_METHOD(Javascript, getScope)
 
 		PHONGO_BSON_INIT_STATE(state);
 
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(intern->scope), intern->scope->len, &state)) {
+		if (!php_phongo_bson_to_zval_ex(intern->scope, &state)) {
 			zval_ptr_dtor(&state.zchild);
 			return;
 		}
@@ -236,21 +203,13 @@ static PHP_METHOD(Javascript, getScope)
 	} else {
 		RETURN_NULL();
 	}
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\BSON\Javascript::jsonSerialize()
-*/
-static PHP_METHOD(Javascript, jsonSerialize)
+static PHP_METHOD(MongoDB_BSON_Javascript, jsonSerialize)
 {
-	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
@@ -261,20 +220,17 @@ static PHP_METHOD(Javascript, jsonSerialize)
 		php_phongo_bson_state state;
 
 		PHONGO_BSON_INIT_STATE(state);
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(intern->scope), intern->scope->len, &state)) {
+		if (!php_phongo_bson_to_zval_ex(intern->scope, &state)) {
 			zval_ptr_dtor(&state.zchild);
 			return;
 		}
 
 		ADD_ASSOC_ZVAL_EX(return_value, "$scope", &state.zchild);
 	}
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\BSON\Javascript::serialize()
-*/
-static PHP_METHOD(Javascript, serialize)
+static PHP_METHOD(MongoDB_BSON_Javascript, serialize)
 {
-	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 	zval                     retval;
 	php_phongo_bson_state    state;
@@ -285,15 +241,10 @@ static PHP_METHOD(Javascript, serialize)
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	if (intern->scope && intern->scope->len) {
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(intern->scope), intern->scope->len, &state)) {
+		if (!php_phongo_bson_to_zval_ex(intern->scope, &state)) {
 			zval_ptr_dtor(&state.zchild);
 			return;
 		}
@@ -314,13 +265,10 @@ static PHP_METHOD(Javascript, serialize)
 
 	smart_str_free(&buf);
 	zval_ptr_dtor(&retval);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\BSON\Javascript::unserialize(string $serialized)
-*/
-static PHP_METHOD(Javascript, unserialize)
+static PHP_METHOD(MongoDB_BSON_Javascript, unserialize)
 {
-	zend_error_handling      error_handling;
 	php_phongo_javascript_t* intern;
 	char*                    serialized;
 	size_t                   serialized_len;
@@ -329,12 +277,9 @@ static PHP_METHOD(Javascript, unserialize)
 
 	intern = Z_JAVASCRIPT_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_STRING(serialized, serialized_len)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	PHP_VAR_UNSERIALIZE_INIT(var_hash);
 	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash)) {
@@ -348,20 +293,16 @@ static PHP_METHOD(Javascript, unserialize)
 
 	php_phongo_javascript_init_from_hash(intern, HASH_OF(&props));
 	zval_ptr_dtor(&props);
-} /* }}} */
+}
 
-/* {{{ proto array MongoDB\Driver\Javascript::__serialize()
-*/
-static PHP_METHOD(Javascript, __serialize)
+static PHP_METHOD(MongoDB_BSON_Javascript, __serialize)
 {
 	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_ARR(php_phongo_javascript_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true));
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Javascript::__unserialize(array $data)
-*/
-static PHP_METHOD(Javascript, __unserialize)
+static PHP_METHOD(MongoDB_BSON_Javascript, __unserialize)
 {
 	zval* data;
 
@@ -370,56 +311,12 @@ static PHP_METHOD(Javascript, __unserialize)
 	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_javascript_init_from_hash(Z_JAVASCRIPT_OBJ_P(getThis()), Z_ARRVAL_P(data));
-} /* }}} */
+}
 
-/* {{{ MongoDB\BSON\Javascript function entries */
-/* clang-format off */
-ZEND_BEGIN_ARG_INFO_EX(ai_Javascript___construct, 0, 0, 1)
-	ZEND_ARG_INFO(0, javascript)
-	ZEND_ARG_INFO(0, scope)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Javascript___set_state, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, properties, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(ai_Javascript___toString, 0, 0, IS_STRING, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Javascript___unserialize, 0, 0, 1)
-	ZEND_ARG_ARRAY_INFO(0, data, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Javascript_jsonSerialize, 0, 0, IS_ARRAY, 0)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Javascript_unserialize, 0, 0, 1)
-	ZEND_ARG_INFO(0, serialized)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Javascript_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_javascript_me[] = {
-	PHP_ME(Javascript, __construct, ai_Javascript___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Javascript, __serialize, ai_Javascript_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Javascript, __set_state, ai_Javascript___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-	PHP_ME(Javascript, __toString, ai_Javascript___toString, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Javascript, __unserialize, ai_Javascript___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Javascript, jsonSerialize, ai_Javascript_jsonSerialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Javascript, serialize, ai_Javascript_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Javascript, unserialize, ai_Javascript_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Javascript, getCode, ai_Javascript_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Javascript, getScope, ai_Javascript_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-};
-/* clang-format on */
-/* }}} */
-
-/* {{{ MongoDB\BSON\Javascript object handlers */
+/* MongoDB\BSON\Javascript object handlers */
 static zend_object_handlers php_phongo_handler_javascript;
 
-static void php_phongo_javascript_free_object(zend_object* object) /* {{{ */
+static void php_phongo_javascript_free_object(zend_object* object)
 {
 	php_phongo_javascript_t* intern = Z_OBJ_JAVASCRIPT(object);
 
@@ -437,22 +334,21 @@ static void php_phongo_javascript_free_object(zend_object* object) /* {{{ */
 		zend_hash_destroy(intern->properties);
 		FREE_HASHTABLE(intern->properties);
 	}
-} /* }}} */
+}
 
-zend_object* php_phongo_javascript_create_object(zend_class_entry* class_type) /* {{{ */
+zend_object* php_phongo_javascript_create_object(zend_class_entry* class_type)
 {
-	php_phongo_javascript_t* intern = NULL;
+	php_phongo_javascript_t* intern = zend_object_alloc(sizeof(php_phongo_javascript_t), class_type);
 
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_javascript_t, class_type);
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
 
 	intern->std.handlers = &php_phongo_handler_javascript;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static zend_object* php_phongo_javascript_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
+static zend_object* php_phongo_javascript_clone_object(phongo_compat_object_handler_type* object)
 {
 	php_phongo_javascript_t* intern;
 	php_phongo_javascript_t* new_intern;
@@ -468,9 +364,9 @@ static zend_object* php_phongo_javascript_clone_object(phongo_compat_object_hand
 	new_intern->scope = bson_copy(intern->scope);
 
 	return new_object;
-} /* }}} */
+}
 
-static int php_phongo_javascript_compare_objects(zval* o1, zval* o2) /* {{{ */
+static int php_phongo_javascript_compare_objects(zval* o1, zval* o2)
 {
 	php_phongo_javascript_t *intern1, *intern2;
 
@@ -481,33 +377,23 @@ static int php_phongo_javascript_compare_objects(zval* o1, zval* o2) /* {{{ */
 
 	/* Do not consider the scope document for comparisons */
 	return strcmp(intern1->code, intern2->code);
-} /* }}} */
+}
 
-static HashTable* php_phongo_javascript_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_javascript_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	*is_temp = 1;
 	return php_phongo_javascript_get_properties_hash(object, true);
-} /* }}} */
+}
 
-static HashTable* php_phongo_javascript_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
+static HashTable* php_phongo_javascript_get_properties(phongo_compat_object_handler_type* object)
 {
 	return php_phongo_javascript_get_properties_hash(object, false);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_javascript_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_javascript_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "Javascript", php_phongo_javascript_me);
-	php_phongo_javascript_ce                = zend_register_internal_class(&ce);
+	php_phongo_javascript_ce                = register_class_MongoDB_BSON_Javascript(php_phongo_javascript_interface_ce, php_phongo_json_serializable_ce, php_phongo_type_ce, zend_ce_serializable);
 	php_phongo_javascript_ce->create_object = php_phongo_javascript_create_object;
-	PHONGO_CE_FINAL(php_phongo_javascript_ce);
-
-	zend_class_implements(php_phongo_javascript_ce, 1, php_phongo_javascript_interface_ce);
-	zend_class_implements(php_phongo_javascript_ce, 1, php_phongo_json_serializable_ce);
-	zend_class_implements(php_phongo_javascript_ce, 1, php_phongo_type_ce);
-	zend_class_implements(php_phongo_javascript_ce, 1, zend_ce_serializable);
 
 #if PHP_VERSION_ID >= 80000
 	zend_class_implements(php_phongo_javascript_ce, 1, zend_ce_stringable);
@@ -520,13 +406,4 @@ void php_phongo_javascript_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_javascript.get_properties = php_phongo_javascript_get_properties;
 	php_phongo_handler_javascript.free_obj       = php_phongo_javascript_free_object;
 	php_phongo_handler_javascript.offset         = XtOffsetOf(php_phongo_javascript_t, std);
-} /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+}
