@@ -26,19 +26,16 @@
 zend_class_entry* php_phongo_objectid_interface_ce;
 
 /* {{{ MongoDB\BSON\ObjectIdInterface function entries */
-/* clang-format off */
-ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_ObjectIdInterface___toString, 0, 0, IS_STRING, 0)
-ZEND_END_ARG_INFO()
-
 ZEND_BEGIN_ARG_INFO_EX(ai_ObjectIdInterface_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_objectid_interface_me[] = {
+	/* clang-format off */
 	ZEND_ABSTRACT_ME(ObjectIdInterface, getTimestamp, ai_ObjectIdInterface_void)
-	ZEND_ABSTRACT_ME(ObjectIdInterface, __toString, ai_ObjectIdInterface___toString)
+	ZEND_ABSTRACT_ME(ObjectIdInterface, __toString, ai_ObjectIdInterface_void)
 	PHP_FE_END
+	/* clang-format on */
 };
-/* clang-format on */
 /* }}} */
 
 void php_phongo_objectid_interface_init_ce(INIT_FUNC_ARGS) /* {{{ */
