@@ -17,26 +17,11 @@
 #include <php.h>
 
 #include "php_phongo.h"
+#include "Serializable_arginfo.h"
 
 zend_class_entry* php_phongo_serializable_ce;
 
-/* {{{ MongoDB\BSON\Serializable function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_Serializable_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_serializable_me[] = {
-	/* clang-format off */
-	ZEND_ABSTRACT_ME(Serializable, bsonSerialize, ai_Serializable_void)
-	PHP_FE_END
-	/* clang-format on */
-};
-/* }}} */
-
-void php_phongo_serializable_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_serializable_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "Serializable", php_phongo_serializable_me);
-	php_phongo_serializable_ce = zend_register_internal_interface(&ce);
-	zend_class_implements(php_phongo_serializable_ce, 1, php_phongo_type_ce);
-} /* }}} */
+	php_phongo_serializable_ce = register_class_MongoDB_BSON_Serializable(php_phongo_type_ce);
+}
