@@ -115,9 +115,6 @@ static PHP_METHOD(Undefined, __unserialize)
 
 /* {{{ MongoDB\BSON\Undefined function entries */
 /* clang-format off */
-ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(ai_Undefined___toString, 0, 0, IS_STRING, 0)
-ZEND_END_ARG_INFO()
-
 ZEND_BEGIN_ARG_INFO_EX(ai_Undefined___unserialize, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, data, 0)
 ZEND_END_ARG_INFO()
@@ -135,7 +132,7 @@ ZEND_END_ARG_INFO()
 static zend_function_entry php_phongo_undefined_me[] = {
 	/* __set_state intentionally missing */
 	PHP_ME(Undefined, __serialize, ai_Undefined_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Undefined, __toString, ai_Undefined___toString, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Undefined, __toString, ai_Undefined_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Undefined, __unserialize, ai_Undefined___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Undefined, jsonSerialize, ai_Undefined_jsonSerialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Undefined, serialize, ai_Undefined_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
@@ -183,10 +180,6 @@ void php_phongo_undefined_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_implements(php_phongo_undefined_ce, 1, php_phongo_type_ce);
 	zend_class_implements(php_phongo_undefined_ce, 1, zend_ce_serializable);
 
-#if PHP_VERSION_ID >= 80000
-	zend_class_implements(php_phongo_undefined_ce, 1, zend_ce_stringable);
-#endif
-
 	memcpy(&php_phongo_handler_undefined, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	/* Re-assign default handler previously removed in php_phongo.c */
 	php_phongo_handler_undefined.clone_obj = zend_objects_clone_obj;
