@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-present MongoDB, Inc.
+ * Copyright 2015-2017 MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,6 +26,11 @@
 #include "php_phongo.h"
 #include "php_bson.h"
 
+#if PHP_VERSION_ID < 70200
+#include <ext/spl/spl_iterators.h>
+#define zend_ce_countable spl_ce_Countable
+#endif /* PHP_VERSION_ID < 70200 */
+
 #define PHONGO_BULKWRITE_BYPASS_UNSET -1
 
 zend_class_entry* php_phongo_bulkwrite_ce;
@@ -526,14 +531,10 @@ static PHP_METHOD(BulkWrite, count)
 } /* }}} */
 
 /* {{{ MongoDB\Driver\BulkWrite function entries */
-/* clang-format off */
 ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite___construct, 0, 0, 0)
 	ZEND_ARG_ARRAY_INFO(0, options, 1)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_BulkWrite_count, 0, 0, IS_LONG, 0)
-ZEND_END_ARG_INFO()
-
 ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite_insert, 0, 0, 1)
 	ZEND_ARG_INFO(0, document)
 ZEND_END_ARG_INFO()
@@ -553,15 +554,16 @@ ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_bulkwrite_me[] = {
+	/* clang-format off */
 	PHP_ME(BulkWrite, __construct, ai_BulkWrite___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(BulkWrite, insert, ai_BulkWrite_insert, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(BulkWrite, update, ai_BulkWrite_update, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(BulkWrite, delete, ai_BulkWrite_delete, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(BulkWrite, count, ai_BulkWrite_count, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(BulkWrite, count, ai_BulkWrite_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_BulkWrite_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
+	/* clang-format on */
 };
-/* clang-format on */
 /* }}} */
 
 /* {{{ MongoDB\Driver\BulkWrite object handlers */
