@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -52,96 +52,6 @@ static void php_phongo_cursor_free_current(php_phongo_cursor_t* cursor) /* {{{ *
 	}
 } /* }}} */
 
-/* {{{ MongoDB\Driver\Cursor iterator handlers */
-static int php_phongo_cursor_valid(php_phongo_cursor_t* cursor) /* {{{ */
-{
-	if (!Z_ISUNDEF(cursor->visitor_data.zchild)) {
-		return SUCCESS;
-	}
-
-	return FAILURE;
-} /* }}} */
-
-static void php_phongo_cursor_get_current_key(php_phongo_cursor_t* cursor, zval* key) /* {{{ */
-{
-	ZVAL_LONG(key, cursor->current);
-} /* }}} */
-
-static zval* php_phongo_cursor_get_current_data(php_phongo_cursor_t* cursor) /* {{{ */
-{
-	return &cursor->visitor_data.zchild;
-} /* }}} */
-
-static void php_phongo_cursor_move_forward(php_phongo_cursor_t* cursor) /* {{{ */
-{
-	const bson_t* doc;
-
-	php_phongo_cursor_free_current(cursor);
-
-	/* If the cursor has already advanced, increment its position. Otherwise,
-	 * the first call to mongoc_cursor_next() will be made below and we should
-	 * leave its position at zero. */
-	if (cursor->advanced) {
-		cursor->current++;
-	} else {
-		cursor->advanced = true;
-	}
-
-	if (mongoc_cursor_next(cursor->cursor, &doc)) {
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data)) {
-			/* Free invalid result, but don't return as we want to free the
-			 * session if the cursor is exhausted. */
-			php_phongo_cursor_free_current(cursor);
-		}
-	} else {
-		bson_error_t  error = { 0 };
-		const bson_t* doc   = NULL;
-
-		if (mongoc_cursor_error_document(cursor->cursor, &error, &doc)) {
-			/* Intentionally not destroying the cursor as it will happen
-			 * naturally now that there are no more results */
-			phongo_throw_exception_from_bson_error_t_and_reply(&error, doc);
-		}
-	}
-
-	php_phongo_cursor_free_session_if_exhausted(cursor);
-} /* }}} */
-
-static void php_phongo_cursor_rewind(php_phongo_cursor_t* cursor) /* {{{ */
-{
-	const bson_t* doc;
-
-	/* If the cursor was never advanced (e.g. command cursor), do so now */
-	if (!cursor->advanced) {
-		cursor->advanced = true;
-
-		if (!phongo_cursor_advance_and_check_for_error(cursor->cursor)) {
-			/* Exception should already have been thrown */
-			return;
-		}
-	}
-
-	if (cursor->current > 0) {
-		phongo_throw_exception(PHONGO_ERROR_LOGIC, "Cursors cannot rewind after starting iteration");
-		return;
-	}
-
-	php_phongo_cursor_free_current(cursor);
-
-	doc = mongoc_cursor_current(cursor->cursor);
-
-	if (doc) {
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data)) {
-			/* Free invalid result, but don't return as we want to free the
-			 * session if the cursor is exhausted. */
-			php_phongo_cursor_free_current(cursor);
-		}
-	}
-
-	php_phongo_cursor_free_session_if_exhausted(cursor);
-} /* }}} */
-/* }}} */
-
 /* {{{ proto void MongoDB\Driver\Cursor::setTypeMap(array $typemap)
    Sets a type map to use for BSON unserialization */
 static PHP_METHOD(Cursor, setTypeMap)
@@ -214,8 +124,9 @@ static void php_phongo_cursor_id_new_from_id(zval* object, int64_t cursorid) /*
 
 	object_init_ex(object, php_phongo_cursorid_ce);
 
-	intern     = Z_CURSORID_OBJ_P(object);
-	intern->id = cursorid;
+	intern              = Z_CURSORID_OBJ_P(object);
+	intern->id          = cursorid;
+	intern->initialized = true;
 } /* }}} */
 
 /* {{{ proto array MongoDB\Driver\Cursor::toArray()
@@ -274,7 +185,7 @@ static PHP_METHOD(Cursor, getServer)
 	}
 	zend_restore_error_handling(&error_handling);
 
-	phongo_server_init(return_value, intern->client, intern->server_id);
+	phongo_server_init(return_value, &intern->manager, intern->server_id);
 } /* }}} */
 
 /* {{{ proto boolean MongoDB\Driver\Cursor::isDead()
@@ -309,9 +220,11 @@ PHP_METHOD(Cursor, current)
 	}
 	zend_restore_error_handling(&error_handling);
 
-	data = php_phongo_cursor_get_current_data(intern);
+	data = &intern->visitor_data.zchild;
 
-	if (data) {
+	if (Z_ISUNDEF_P(data)) {
+		RETURN_NULL();
+	} else {
 		ZVAL_COPY_DEREF(return_value, data);
 	}
 }
@@ -328,13 +241,18 @@ PHP_METHOD(Cursor, key)
 	}
 	zend_restore_error_handling(&error_handling);
 
-	php_phongo_cursor_get_current_key(intern, return_value);
+	if (Z_ISUNDEF(intern->visitor_data.zchild)) {
+		RETURN_NULL();
+	}
+
+	RETURN_LONG(intern->current);
 }
 
 PHP_METHOD(Cursor, next)
 {
 	zend_error_handling  error_handling;
 	php_phongo_cursor_t* intern = Z_CURSOR_OBJ_P(getThis());
+	const bson_t*        doc;
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
@@ -343,7 +261,35 @@ PHP_METHOD(Cursor, next)
 	}
 	zend_restore_error_handling(&error_handling);
 
-	php_phongo_cursor_move_forward(intern);
+	php_phongo_cursor_free_current(intern);
+
+	/* If the intern has already advanced, increment its position. Otherwise,
+	 * the first call to mongoc_cursor_next() will be made below and we should
+	 * leave its position at zero. */
+	if (intern->advanced) {
+		intern->current++;
+	} else {
+		intern->advanced = true;
+	}
+
+	if (mongoc_cursor_next(intern->cursor, &doc)) {
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &intern->visitor_data)) {
+			/* Free invalid result, but don't return as we want to free the
+			 * session if the intern is exhausted. */
+			php_phongo_cursor_free_current(intern);
+		}
+	} else {
+		bson_error_t  error = { 0 };
+		const bson_t* doc   = NULL;
+
+		if (mongoc_cursor_error_document(intern->cursor, &error, &doc)) {
+			/* Intentionally not destroying the intern as it will happen
+			 * naturally now that there are no more results */
+			phongo_throw_exception_from_bson_error_t_and_reply(&error, doc);
+		}
+	}
+
+	php_phongo_cursor_free_session_if_exhausted(intern);
 }
 
 PHP_METHOD(Cursor, valid)
@@ -358,13 +304,14 @@ PHP_METHOD(Cursor, valid)
 	}
 	zend_restore_error_handling(&error_handling);
 
-	RETURN_BOOL(php_phongo_cursor_valid(intern) == SUCCESS);
+	RETURN_BOOL(!Z_ISUNDEF(intern->visitor_data.zchild));
 }
 
 PHP_METHOD(Cursor, rewind)
 {
 	zend_error_handling  error_handling;
 	php_phongo_cursor_t* intern = Z_CURSOR_OBJ_P(getThis());
+	const bson_t*        doc;
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
@@ -373,10 +320,53 @@ PHP_METHOD(Cursor, rewind)
 	}
 	zend_restore_error_handling(&error_handling);
 
-	php_phongo_cursor_rewind(intern);
+	/* If the intern was never advanced (e.g. command intern), do so now */
+	if (!intern->advanced) {
+		intern->advanced = true;
+
+		if (!phongo_cursor_advance_and_check_for_error(intern->cursor)) {
+			/* Exception should already have been thrown */
+			return;
+		}
+	}
+
+	if (intern->current > 0) {
+		phongo_throw_exception(PHONGO_ERROR_LOGIC, "Cursors cannot rewind after starting iteration");
+		return;
+	}
+
+	php_phongo_cursor_free_current(intern);
+
+	doc = mongoc_cursor_current(intern->cursor);
+
+	if (doc) {
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &intern->visitor_data)) {
+			/* Free invalid result, but don't return as we want to free the
+			 * session if the intern is exhausted. */
+			php_phongo_cursor_free_current(intern);
+		}
+	}
+
+	php_phongo_cursor_free_session_if_exhausted(intern);
 }
 
 /* {{{ MongoDB\Driver\Cursor function entries */
+/* clang-format off */
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_current, 0, 0, IS_MIXED, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_key, 0, 0, IS_MIXED, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_next, 0, 0, IS_VOID, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_valid, 0, 0, _IS_BOOL, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_rewind, 0, 0, IS_VOID, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_setTypeMap, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, typemap, 0)
 ZEND_END_ARG_INFO()
@@ -385,24 +375,23 @@ ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_cursor_me[] = {
-	/* clang-format off */
 	PHP_ME(Cursor, setTypeMap, ai_Cursor_setTypeMap, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, toArray, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, getId, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, getServer, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, isDead, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 
-	PHP_ME(Cursor, current, ai_Cursor_void, ZEND_ACC_PUBLIC)
-	PHP_ME(Cursor, key, ai_Cursor_void, ZEND_ACC_PUBLIC)
-	PHP_ME(Cursor, next, ai_Cursor_void, ZEND_ACC_PUBLIC)
-	PHP_ME(Cursor, valid, ai_Cursor_void, ZEND_ACC_PUBLIC)
-	PHP_ME(Cursor, rewind, ai_Cursor_void, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, current, ai_Cursor_current, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, key, ai_Cursor_key, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, next, ai_Cursor_next, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, valid, ai_Cursor_valid, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, rewind, ai_Cursor_rewind, ZEND_ACC_PUBLIC)
 
 	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_Cursor_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
-	/* clang-format on */
 };
+/* clang-format on */
 /* }}} */
 
 /* {{{ MongoDB\Driver\Cursor object handlers */
@@ -417,7 +406,7 @@ static void php_phongo_cursor_free_object(zend_object* object) /* {{{ */
 	/* If this Cursor was created in a different process, reset the client so
 	 * that mongoc_cursor_destroy does not issue a killCursors command for an
 	 * active cursor owned by a parent process. */
-	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern, Z_MANAGER_OBJ_P(&intern->manager));
 
 	if (intern->cursor) {
 		mongoc_cursor_destroy(intern->cursor);
@@ -447,6 +436,10 @@ static void php_phongo_cursor_free_object(zend_object* object) /* {{{ */
 		zval_ptr_dtor(&intern->session);
 	}
 
+	if (!Z_ISUNDEF(intern->manager)) {
+		zval_ptr_dtor(&intern->manager);
+	}
+
 	php_phongo_bson_typemap_dtor(&intern->visitor_data.map);
 
 	php_phongo_cursor_free_current(intern);
@@ -532,7 +525,7 @@ static HashTable* php_phongo_cursor_get_debug_info(phongo_compat_object_handler_
 	{
 		zval server;
 
-		phongo_server_init(&server, intern->client, intern->server_id);
+		phongo_server_init(&server, &intern->manager, intern->server_id);
 		ADD_ASSOC_ZVAL_EX(&retval, "server", &server);
 	}
 
