@@ -16,19 +16,14 @@
 
 #include <php.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
-#include "src/phongo_apm.h"
+#include "phongo_apm.h"
+#include "phongo_error.h"
 
 ZEND_EXTERN_MODULE_GLOBALS(mongodb)
 
-/* {{{ proto void MongoDB\Driver\Monitoring\addSubscriber(MongoDB\Driver\Monitoring\Subscriber $subscriber)
-   Registers a global event subscriber */
-PHP_FUNCTION(MongoDB_Driver_Monitoring_addSubscriber)
+/* Registers a global event subscriber */
+PHP_FUNCTION(addSubscriber)
 {
 	zval* subscriber;
 
@@ -37,11 +32,10 @@ PHP_FUNCTION(MongoDB_Driver_Monitoring_addSubscriber)
 	PHONGO_PARSE_PARAMETERS_END();
 
 	phongo_apm_add_subscriber(MONGODB_G(subscribers), subscriber);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Monitoring\removeSubscriber(MongoDB\Driver\Monitoring\Subscriber $subscriber)
-   Unregisters a global event subscriber */
-PHP_FUNCTION(MongoDB_Driver_Monitoring_removeSubscriber)
+/* Unregisters a global event subscriber */
+PHP_FUNCTION(removeSubscriber)
 {
 	zval* subscriber;
 
@@ -50,13 +44,4 @@ PHP_FUNCTION(MongoDB_Driver_Monitoring_removeSubscriber)
 	PHONGO_PARSE_PARAMETERS_END();
 
 	phongo_apm_remove_subscriber(MONGODB_G(subscribers), subscriber);
-} /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+}
