@@ -14,27 +14,28 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+#include "mongoc/mongoc.h"
+
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
-#include "php_phongo.h"
-#include "php_bson.h"
 #include "php_array_api.h"
-#include "Session.h"
+
+#include "php_phongo.h"
+#include "phongo_bson_encode.h"
+#include "phongo_client.h"
+#include "phongo_error.h"
+
+#include "MongoDB/ReadConcern.h"
+#include "MongoDB/ReadPreference.h"
+#include "MongoDB/Server.h"
+#include "MongoDB/Session.h"
+#include "MongoDB/WriteConcern.h"
+#include "Session_arginfo.h"
 
 zend_class_entry* php_phongo_session_ce;
 
-#define PHONGO_TRANSACTION_NONE "none"
-#define PHONGO_TRANSACTION_STARTING "starting"
-#define PHONGO_TRANSACTION_IN_PROGRESS "in_progress"
-#define PHONGO_TRANSACTION_COMMITTED "committed"
-#define PHONGO_TRANSACTION_ABORTED "aborted"
-
 #define SESSION_CHECK_LIVELINESS(i, m)                                  \
 	if (!(i)->client_session) {                                         \
 		phongo_throw_exception(                                         \
@@ -153,11 +154,12 @@ static void php_phongo_transaction_options_to_zval(mongoc_client_session_t* cs,
 	mongoc_transaction_opts_destroy(opts);
 }
 
-/* {{{ proto void MongoDB\Driver\Session::advanceClusterTime(array|object $clusterTime)
-   Advances the cluster time for this Session */
-static PHP_METHOD(Session, advanceClusterTime)
+PHONGO_DISABLED_CONSTRUCTOR(MongoDB_Driver_Session)
+PHONGO_DISABLED_WAKEUP(MongoDB_Driver_Session)
+
+/* Advances the cluster time for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, advanceClusterTime)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	zval*                 zcluster_time;
 	bson_t                cluster_time = BSON_INITIALIZER;
@@ -165,12 +167,9 @@ static PHP_METHOD(Session, advanceClusterTime)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "advanceClusterTime")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "A", &zcluster_time) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	PHONGO_PARAM_ARRAY_OR_OBJECT(zcluster_time)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	php_phongo_zval_to_bson(zcluster_time, PHONGO_BSON_NONE, &cluster_time, NULL);
 
@@ -183,13 +182,11 @@ static PHP_METHOD(Session, advanceClusterTime)
 
 cleanup:
 	bson_destroy(&cluster_time);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::advanceOperationTime(MongoDB\BSON\TimestampInterface $timestamp)
-   Advances the operation time for this Session */
-static PHP_METHOD(Session, advanceOperationTime)
+/* Advances the operation time for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, advanceOperationTime)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	zval*                 ztimestamp;
 	uint32_t              timestamp = 0;
@@ -198,25 +195,20 @@ static PHP_METHOD(Session, advanceOperationTime)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "advanceOperationTime")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &ztimestamp, php_phongo_timestamp_interface_ce) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_OBJECT_OF_CLASS(ztimestamp, php_phongo_timestamp_interface_ce)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (!php_phongo_session_get_timestamp_parts(ztimestamp, &timestamp, &increment)) {
 		return;
 	}
 
 	mongoc_client_session_advance_operation_time(intern->client_session, timestamp, increment);
-} /* }}} */
+}
 
-/* {{{ proto object|null MongoDB\Driver\Session::getClusterTime()
-   Returns the cluster time for this Session */
-static PHP_METHOD(Session, getClusterTime)
+/* Returns the cluster time for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, getClusterTime)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	const bson_t*         cluster_time;
 	php_phongo_bson_state state;
@@ -226,12 +218,7 @@ static PHP_METHOD(Session, getClusterTime)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getClusterTime")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	cluster_time = mongoc_client_session_get_cluster_time(intern->client_session);
 
@@ -239,20 +226,18 @@ static PHP_METHOD(Session, getClusterTime)
 		RETURN_NULL();
 	}
 
-	if (!php_phongo_bson_to_zval_ex(bson_get_data(cluster_time), cluster_time->len, &state)) {
+	if (!php_phongo_bson_to_zval_ex(cluster_time, &state)) {
 		/* Exception should already have been thrown */
 		zval_ptr_dtor(&state.zchild);
 		return;
 	}
 
 	RETURN_ZVAL(&state.zchild, 0, 1);
-} /* }}} */
+}
 
-/* {{{ proto object MongoDB\Driver\Session::getLogicalSessionId()
-   Returns the logical session ID for this Session */
-static PHP_METHOD(Session, getLogicalSessionId)
+/* Returns the logical session ID for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, getLogicalSessionId)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	const bson_t*         lsid;
 	php_phongo_bson_state state;
@@ -262,41 +247,29 @@ static PHP_METHOD(Session, getLogicalSessionId)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getLogicalSessionId")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	lsid = mongoc_client_session_get_lsid(intern->client_session);
 
-	if (!php_phongo_bson_to_zval_ex(bson_get_data(lsid), lsid->len, &state)) {
+	if (!php_phongo_bson_to_zval_ex(lsid, &state)) {
 		/* Exception should already have been thrown */
 		zval_ptr_dtor(&state.zchild);
 		return;
 	}
 
 	RETURN_ZVAL(&state.zchild, 0, 1);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\BSON\Timestamp|null MongoDB\Driver\Session::getOperationTime()
-   Returns the operation time for this Session */
-static PHP_METHOD(Session, getOperationTime)
+/* Returns the operation time for this Session */
+static PHP_METHOD(MongoDB_Driver_Session, getOperationTime)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	uint32_t              timestamp, increment;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getOperationTime")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	mongoc_client_session_get_operation_time(intern->client_session, &timestamp, &increment);
 
@@ -308,25 +281,18 @@ static PHP_METHOD(Session, getOperationTime)
 	}
 
 	php_phongo_bson_new_timestamp_from_increment_and_timestamp(return_value, increment, timestamp);
-} /* }}} */
+}
 
-/* {{{ proto MongoDB\Driver\Server|null MongoDB\Driver\Session::getServer()
-   Returns the server this session is pinned to */
-static PHP_METHOD(Session, getServer)
+/* Returns the server this session is pinned to */
+static PHP_METHOD(MongoDB_Driver_Session, getServer)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	uint32_t              server_id = 0;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getServer")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	server_id = mongoc_client_session_get_server_id(intern->client_session);
 
@@ -336,45 +302,31 @@ static PHP_METHOD(Session, getServer)
 	}
 
 	phongo_server_init(return_value, &intern->manager, server_id);
-} /* }}} */
+}
 
-/* {{{ proto array|null MongoDB\Driver\Session::getTransactionOptions()
-   Returns options for the currently running transaction */
-static PHP_METHOD(Session, getTransactionOptions)
+/* Returns options for the currently running transaction */
+static PHP_METHOD(MongoDB_Driver_Session, getTransactionOptions)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getTransactionOptions")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	php_phongo_transaction_options_to_zval(intern->client_session, return_value);
-} /* }}} */
+}
 
-/* {{{ proto string MongoDB\Driver\Session::getTransactionState()
-   Returns the current transaction state for this session */
-static PHP_METHOD(Session, getTransactionState)
+/* Returns the current transaction state for this session */
+static PHP_METHOD(MongoDB_Driver_Session, getTransactionState)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	const char*           state;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "getTransactionState")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	state = php_phongo_get_transaction_state_string(mongoc_client_session_get_transaction_state(intern->client_session));
 	if (!state) {
@@ -383,7 +335,7 @@ static PHP_METHOD(Session, getTransactionState)
 	}
 
 	RETURN_STRING(state);
-} /* }}} */
+}
 
 /* Creates a opts structure from an array optionally containing an RP, RC,
  * WC object, and/or maxCommitTimeMS int. Returns NULL if no options were found,
@@ -482,11 +434,9 @@ mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* op
 	return opts;
 }
 
-/* {{{ proto void MongoDB\Driver\Session::startTransaction([array $options = null])
-   Starts a new transaction */
-static PHP_METHOD(Session, startTransaction)
+/* Starts a new transaction */
+static PHP_METHOD(MongoDB_Driver_Session, startTransaction)
 {
-	zend_error_handling       error_handling;
 	php_phongo_session_t*     intern;
 	zval*                     options     = NULL;
 	mongoc_transaction_opt_t* txn_options = NULL;
@@ -495,12 +445,10 @@ static PHP_METHOD(Session, startTransaction)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "startTransaction")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|a!", &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(0, 1)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_ARRAY_OR_NULL(options)
+	PHONGO_PARSE_PARAMETERS_END();
 
 	if (options) {
 		txn_options = php_mongodb_session_parse_transaction_options(options);
@@ -516,13 +464,11 @@ static PHP_METHOD(Session, startTransaction)
 	if (txn_options) {
 		mongoc_transaction_opts_destroy(txn_options);
 	}
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::commitTransaction(void)
-   Commits an existing transaction */
-static PHP_METHOD(Session, commitTransaction)
+/* Commits an existing transaction */
+static PHP_METHOD(MongoDB_Driver_Session, commitTransaction)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	bson_error_t          error;
 	bson_t                reply;
@@ -530,125 +476,75 @@ static PHP_METHOD(Session, commitTransaction)
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "commitTransaction")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	if (!mongoc_client_session_commit_transaction(intern->client_session, &reply, &error)) {
 		phongo_throw_exception_from_bson_error_t_and_reply(&error, &reply);
 	}
 
 	bson_destroy(&reply);
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::abortTransaction(void)
-   Aborts (rolls back) an existing transaction */
-static PHP_METHOD(Session, abortTransaction)
+/* Aborts (rolls back) an existing transaction */
+static PHP_METHOD(MongoDB_Driver_Session, abortTransaction)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 	bson_error_t          error;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "abortTransaction")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	if (!mongoc_client_session_abort_transaction(intern->client_session, &error)) {
 		phongo_throw_exception_from_bson_error_t(&error);
 	}
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::endSession(void)
-   Ends the session, and a running transaction if active */
-static PHP_METHOD(Session, endSession)
+/* Ends the session, and a running transaction if active */
+static PHP_METHOD(MongoDB_Driver_Session, endSession)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	mongoc_client_session_destroy(intern->client_session);
 	intern->client_session = NULL;
-} /* }}} */
+}
 
-/* {{{ proto void MongoDB\Driver\Session::isInTransaction(void)
-   Returns whether a multi-document transaction is in progress */
-static PHP_METHOD(Session, isInTransaction)
+/* Returns whether the session is dirty (i.e. was used with a command that
+   encountered a network error) and will be discarded when returned to the
+   server session pool. */
+static PHP_METHOD(MongoDB_Driver_Session, isDirty)
+{
+	php_phongo_session_t* intern = Z_SESSION_OBJ_P(getThis());
+
+	SESSION_CHECK_LIVELINESS(intern, "isDirty")
+
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	RETVAL_BOOL(mongoc_client_session_get_dirty(intern->client_session));
+}
+
+/* Returns whether a multi-document transaction is in progress */
+static PHP_METHOD(MongoDB_Driver_Session, isInTransaction)
 {
-	zend_error_handling   error_handling;
 	php_phongo_session_t* intern;
 
 	intern = Z_SESSION_OBJ_P(getThis());
 	SESSION_CHECK_LIVELINESS(intern, "isInTransaction")
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	RETURN_BOOL(mongoc_client_session_in_transaction(intern->client_session));
-} /* }}} */
+}
 
-/* {{{ MongoDB\Driver\Session function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_Session_advanceClusterTime, 0, 0, 1)
-	ZEND_ARG_INFO(0, clusterTime)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Session_advanceOperationTime, 0, 0, 1)
-	ZEND_ARG_INFO(0, timestamp)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Session_startTransaction, 0, 0, 0)
-	ZEND_ARG_ARRAY_INFO(0, options, 1)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Session_void, 0, 0, 0)
-ZEND_END_ARG_INFO()
-
-static zend_function_entry php_phongo_session_me[] = {
-	/* clang-format off */
-	PHP_ME(Session, abortTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, advanceClusterTime, ai_Session_advanceClusterTime, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, advanceOperationTime, ai_Session_advanceOperationTime, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, commitTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, endSession, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getClusterTime, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getLogicalSessionId, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getOperationTime, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getServer, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getTransactionOptions, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, getTransactionState, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, isInTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(Session, startTransaction, ai_Session_startTransaction, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_Session_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_FE_END
-	/* clang-format on */
-};
-/* }}} */
-
-/* {{{ MongoDB\Driver\Session object handlers */
+/* MongoDB\Driver\Session object handlers */
 static zend_object_handlers php_phongo_handler_session;
 
-static void php_phongo_session_free_object(zend_object* object) /* {{{ */
+static void php_phongo_session_free_object(zend_object* object)
 {
 	php_phongo_session_t* intern = Z_OBJ_SESSION(object);
 
@@ -668,13 +564,11 @@ static void php_phongo_session_free_object(zend_object* object) /* {{{ */
 	if (!Z_ISUNDEF(intern->manager)) {
 		zval_ptr_dtor(&intern->manager);
 	}
-} /* }}} */
+}
 
-static zend_object* php_phongo_session_create_object(zend_class_entry* class_type) /* {{{ */
+static zend_object* php_phongo_session_create_object(zend_class_entry* class_type)
 {
-	php_phongo_session_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_session_t, class_type);
+	php_phongo_session_t* intern = zend_object_alloc(sizeof(php_phongo_session_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -684,9 +578,9 @@ static zend_object* php_phongo_session_create_object(zend_class_entry* class_typ
 	intern->std.handlers = &php_phongo_handler_session;
 
 	return &intern->std;
-} /* }}} */
+}
 
-static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp)
 {
 	php_phongo_session_t* intern = NULL;
 	zval                  retval = ZVAL_STATIC_INIT;
@@ -696,35 +590,35 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 
 	array_init(&retval);
 
-	if (intern->client_session) {
-		const bson_t*         lsid;
+	if (!intern->client_session) {
+		ADD_ASSOC_BOOL_EX(&retval, "ended", true);
+
+		goto done;
+	}
+
+	{
+		const bson_t*         lsid = mongoc_client_session_get_lsid(intern->client_session);
 		php_phongo_bson_state state;
 
 		PHONGO_BSON_INIT_DEBUG_STATE(state);
 
-		lsid = mongoc_client_session_get_lsid(intern->client_session);
-
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(lsid), lsid->len, &state)) {
+		if (!php_phongo_bson_to_zval_ex(lsid, &state)) {
 			zval_ptr_dtor(&state.zchild);
 			goto done;
 		}
 
 		ADD_ASSOC_ZVAL_EX(&retval, "logicalSessionId", &state.zchild);
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "logicalSessionId");
 	}
 
-	if (intern->client_session) {
-		const bson_t* cluster_time;
-
-		cluster_time = mongoc_client_session_get_cluster_time(intern->client_session);
+	{
+		const bson_t* cluster_time = mongoc_client_session_get_cluster_time(intern->client_session);
 
 		if (cluster_time) {
 			php_phongo_bson_state state;
 
 			PHONGO_BSON_INIT_DEBUG_STATE(state);
 
-			if (!php_phongo_bson_to_zval_ex(bson_get_data(cluster_time), cluster_time->len, &state)) {
+			if (!php_phongo_bson_to_zval_ex(cluster_time, &state)) {
 				zval_ptr_dtor(&state.zchild);
 				goto done;
 			}
@@ -733,20 +627,15 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "clusterTime");
 		}
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "clusterTime");
 	}
 
-	if (intern->client_session) {
+	{
 		const mongoc_session_opt_t* cs_opts = mongoc_client_session_get_opts(intern->client_session);
 		ADD_ASSOC_BOOL_EX(&retval, "causalConsistency", mongoc_session_opts_get_causal_consistency(cs_opts));
 		ADD_ASSOC_BOOL_EX(&retval, "snapshot", mongoc_session_opts_get_snapshot(cs_opts));
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "causalConsistency");
-		ADD_ASSOC_NULL_EX(&retval, "snapshot");
 	}
 
-	if (intern->client_session) {
+	{
 		uint32_t timestamp, increment;
 
 		mongoc_client_session_get_operation_time(intern->client_session, &timestamp, &increment);
@@ -759,15 +648,12 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "operationTime");
 		}
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "operationTime");
 	}
 
-	if (intern->client_session) {
+	{
 		uint32_t server_id = mongoc_client_session_get_server_id(intern->client_session);
 
 		if (server_id) {
-
 			zval server;
 
 			phongo_server_init(&server, &intern->manager, server_id);
@@ -775,17 +661,12 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "server");
 		}
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "server");
 	}
 
-	if (intern->client_session) {
-		ADD_ASSOC_BOOL_EX(&retval, "inTransaction", mongoc_client_session_in_transaction(intern->client_session));
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "inTransaction");
-	}
+	ADD_ASSOC_BOOL_EX(&retval, "dirty", mongoc_client_session_get_dirty(intern->client_session));
+	ADD_ASSOC_BOOL_EX(&retval, "inTransaction", mongoc_client_session_in_transaction(intern->client_session));
 
-	if (intern->client_session) {
+	{
 		const char* state = php_phongo_get_transaction_state_string(mongoc_client_session_get_transaction_state(intern->client_session));
 
 		if (!state) {
@@ -794,50 +675,39 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 		}
 
 		ADD_ASSOC_STRING(&retval, "transactionState", state);
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "transactionState");
 	}
 
-	if (intern->client_session) {
+	{
 		zval txn_opts;
+
 		php_phongo_transaction_options_to_zval(intern->client_session, &txn_opts);
 		ADD_ASSOC_ZVAL_EX(&retval, "transactionOptions", &txn_opts);
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "transactionOptions");
 	}
 
 done:
 	return Z_ARRVAL(retval);
-} /* }}} */
-/* }}} */
+}
 
-void php_phongo_session_init_ce(INIT_FUNC_ARGS) /* {{{ */
+void php_phongo_session_init_ce(INIT_FUNC_ARGS)
 {
-	zend_class_entry ce;
-
-	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Session", php_phongo_session_me);
-	php_phongo_session_ce                = zend_register_internal_class(&ce);
+	php_phongo_session_ce                = register_class_MongoDB_Driver_Session();
 	php_phongo_session_ce->create_object = php_phongo_session_create_object;
-	PHONGO_CE_FINAL(php_phongo_session_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_session_ce);
 
 	memcpy(&php_phongo_handler_session, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_session.get_debug_info = php_phongo_session_get_debug_info;
 	php_phongo_handler_session.free_obj       = php_phongo_session_free_object;
 	php_phongo_handler_session.offset         = XtOffsetOf(php_phongo_session_t, std);
+}
 
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_NONE"), PHONGO_TRANSACTION_NONE);
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_STARTING"), PHONGO_TRANSACTION_STARTING);
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_IN_PROGRESS"), PHONGO_TRANSACTION_IN_PROGRESS);
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_COMMITTED"), PHONGO_TRANSACTION_COMMITTED);
-	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_ABORTED"), PHONGO_TRANSACTION_ABORTED);
-} /* }}} */
+void phongo_session_init(zval* return_value, zval* manager, mongoc_client_session_t* client_session)
+{
+	php_phongo_session_t* session;
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+	object_init_ex(return_value, php_phongo_session_ce);
+
+	session                 = Z_SESSION_OBJ_P(return_value);
+	session->client_session = client_session;
+
+	ZVAL_ZVAL(&session->manager, manager, 1, 0);
+}
