@@ -1,5 +1,5 @@
 /*
- * Copyright 2017 MongoDB, Inc.
+ * Copyright 2017-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,6 +17,6 @@
 #ifndef PHP_MONGODB_DRIVER_SESSION_H
 #define PHP_MONGODB_DRIVER_SESSION_H
 
-mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* txnOptions TSRMLS_DC);
+mongoc_transaction_opt_t* php_mongodb_session_parse_transaction_options(zval* txnOptions);
 
 #endif /* PHP_MONGODB_DRIVER_SESSION_H */
