@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-present MongoDB, Inc.
+ * Copyright 2014-2017 MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -503,7 +503,7 @@ void php_phongo_zval_to_bson_value(zval* data, php_phongo_bson_flags_t flags, bs
 	bson_iter_t iter;
 	bson_t      bson = BSON_INITIALIZER;
 
-	zval* data_object = ecalloc(1, sizeof(zval));
+	zval* data_object = ecalloc(sizeof(zval), 1);
 
 	array_init_size(data_object, 1);
 	add_assoc_zval(data_object, "data", data);
