@@ -113,20 +113,17 @@ static const char Pad64 = '=';
  *    characters followed by one "=" padding character.
  */
 
-int COMMON_PREFIX (bson_b64_ntop) (uint8_t const *src,
-                                   size_t srclength,
-                                   char *target,
-                                   size_t targsize)
+int
+bson_b64_ntop (uint8_t const *src,
+               size_t srclength,
+               char *target,
+               size_t targsize)
 {
    size_t datalength = 0;
    uint8_t input[3];
    uint8_t output[4];
    size_t i;
 
-   if (!target) {
-      return -1;
-   }
-
    while (2 < srclength) {
       input[0] = *src++;
       input[1] = *src++;
@@ -387,8 +384,8 @@ mongoc_b64_pton_do (char const *src, uint8_t *target, size_t targsize)
          if (ch != Pad64)
             return (-1);
          ch = *src++; /* Skip the = */
-         /* Fall through to "single trailing =" case. */
-         /* FALLTHROUGH */
+      /* Fall through to "single trailing =" case. */
+      /* FALLTHROUGH */
 
       case 3: /* Valid, means two bytes of info */
          /*
@@ -489,8 +486,8 @@ mongoc_b64_pton_len (char const *src)
          if (ch != Pad64)
             return (-1);
          ch = *src++; /* Skip the = */
-         /* Fall through to "single trailing =" case. */
-         /* FALLTHROUGH */
+      /* Fall through to "single trailing =" case. */
+      /* FALLTHROUGH */
 
       case 3: /* Valid, means two bytes of info */
          /*
@@ -517,43 +514,15 @@ mongoc_b64_pton_len (char const *src)
 }
 
 
-int COMMON_PREFIX (bson_b64_pton) (char const *src,
-                                   uint8_t *target,
-                                   size_t targsize)
+int
+bson_b64_pton (char const *src, uint8_t *target, size_t targsize)
 {
    static mongoc_common_once_t once = MONGOC_COMMON_ONCE_INIT;
 
    mongoc_common_once (&once, bson_b64_initialize_rmap);
 
-   if (!src) {
-      return -1;
-   }
-
    if (target)
       return mongoc_b64_pton_do (src, target, targsize);
    else
       return mongoc_b64_pton_len (src);
 }
-
-size_t COMMON_PREFIX (bson_b64_ntop_calculate_target_size) (size_t raw_size)
-{
-   size_t num_bits = raw_size * 8;
-   /* Calculate how many groups of six bits this contains, adding 5 to round up
-    * to the nearest group of 6. */
-   size_t num_b64_chars = (num_bits + 5) / 6;
-   /* Round to nearest set of four. */
-   size_t num_b64_chars_with_padding = 4 * ((num_b64_chars + 3) / 4);
-   /* Add one for NULL byte. */
-   return num_b64_chars_with_padding + 1;
-}
-
-size_t COMMON_PREFIX (bson_b64_pton_calculate_target_size) (
-   size_t base64_encoded_size)
-{
-   /* Without inspecting the data, we don't know how many padding characters
-    * there are. Assuming none, that means each character represents 6 bits of
-    * data. */
-   size_t num_bits = base64_encoded_size * 6;
-   /* Round down to the nearest group of eight. */
-   return num_bits / 8;
-}
