@@ -57,9 +57,10 @@ typedef uint32_t bson_unichar_t;
  *
  * %BSON_CONTEXT_NONE: Use default options.
  * %BSON_CONTEXT_THREAD_SAFE: Context will be called from multiple threads.
- * %BSON_CONTEXT_DISABLE_HOST_CACHE: Does nothing, is ignored.
  * %BSON_CONTEXT_DISABLE_PID_CACHE: Call getpid() instead of caching the
  *   result of getpid() when initializing the context.
+ * %BSON_CONTEXT_DISABLE_HOST_CACHE: Call gethostname() instead of caching the
+ *   result of gethostname() when initializing the context.
  */
 typedef enum {
    BSON_CONTEXT_NONE = 0,
@@ -97,17 +98,6 @@ typedef enum {
  */
 typedef struct _bson_context_t bson_context_t;
 
-/**
- * bson_json_opts_t:
- *
- * This structure is used to pass options for serializing BSON into extended
- * JSON to the respective serialization methods.
- *
- * max_len can be either a non-negative integer, or BSON_MAX_LEN_UNLIMITED to
- * set no limit for serialization length.
- */
-typedef struct _bson_json_opts_t bson_json_opts_t;
-
 
 /**
  * bson_t:
@@ -129,10 +119,10 @@ typedef struct _bson_json_opts_t bson_json_opts_t;
 #ifdef BSON_MEMCHECK
 BSON_ALIGNED_BEGIN (128)
 typedef struct _bson_t {
-   uint32_t flags; /* Internal flags for the bson_t. */
-   uint32_t len;   /* Length of BSON data. */
-   char *canary;   /* For valgrind check */
-   uint8_t padding[120 - sizeof (char *)];
+   uint32_t flags;       /* Internal flags for the bson_t. */
+   uint32_t len;         /* Length of BSON data. */
+   char *canary;         /* For valgrind check */
+   uint8_t padding[120 - sizeof (char*)];
 } bson_t BSON_ALIGNED_END (128);
 #else
 BSON_ALIGNED_BEGIN (128)
@@ -143,6 +133,7 @@ typedef struct _bson_t {
 } bson_t BSON_ALIGNED_END (128);
 #endif
 
+
 /**
  * BSON_INITIALIZER:
  *
@@ -154,9 +145,13 @@ typedef struct _bson_t {
  * ]|
  */
 #ifdef BSON_MEMCHECK
-#define BSON_INITIALIZER          \
-   {                              \
-      3, 5, bson_malloc (1), {5}, \
+#define BSON_INITIALIZER \
+   {                     \
+      3, 5,              \
+      bson_malloc (1),   \
+      {                  \
+         5               \
+      },                 \
    }
 #else
 #define BSON_INITIALIZER \
@@ -277,7 +272,6 @@ typedef enum {
    BSON_SUBTYPE_UUID = 0x04,
    BSON_SUBTYPE_MD5 = 0x05,
    BSON_SUBTYPE_ENCRYPTED = 0x06,
-   BSON_SUBTYPE_COLUMN = 0x07,
    BSON_SUBTYPE_USER = 0x80,
 } bson_subtype_t;
 
