@@ -17,9 +17,6 @@
 #include "mongoc-util-private.h"
 #include "mongoc-apm-private.h"
 #include "mongoc-cmd-private.h"
-#include "mongoc-handshake-private.h"
-
-static bson_oid_t kObjectIdZero = {{0}};
 
 /*
  * An Application Performance Management (APM) implementation, complying with
@@ -49,24 +46,6 @@ append_documents_from_cmd (const mongoc_cmd_t *cmd,
  * Private initializer / cleanup functions.
  */
 
-static void
-mongoc_apm_redact_command (bson_t *command);
-
-static void
-mongoc_apm_redact_reply (bson_t *reply);
-
-/*--------------------------------------------------------------------------
- *
- * mongoc_apm_command_started_init --
- *
- *       Initialises the command started event.
- *
- * Side effects:
- *       If provided, is_redacted indicates whether the command document was
- *       redacted to hide sensitive information.
- *
- *--------------------------------------------------------------------------
- */
 void
 mongoc_apm_command_started_init (mongoc_apm_command_started_t *event,
                                  const bson_t *command,
@@ -76,8 +55,6 @@ mongoc_apm_command_started_init (mongoc_apm_command_started_t *event,
                                  int64_t operation_id,
                                  const mongoc_host_list_t *host,
                                  uint32_t server_id,
-                                 const bson_oid_t *service_id,
-                                 bool *is_redacted, /* out */
                                  void *context)
 {
    bson_iter_t iter;
@@ -110,21 +87,6 @@ mongoc_apm_command_started_init (mongoc_apm_command_started_t *event,
       event->command_owned = false;
    }
 
-   if (mongoc_apm_is_sensitive_command_message (command_name, command)) {
-      if (!event->command_owned) {
-         event->command = bson_copy (event->command);
-         event->command_owned = true;
-      }
-
-      if (is_redacted) {
-         *is_redacted = true;
-      }
-
-      mongoc_apm_redact_command (event->command);
-   } else if (is_redacted) {
-      *is_redacted = false;
-   }
-
    event->database_name = database_name;
    event->command_name = command_name;
    event->request_id = request_id;
@@ -132,28 +94,13 @@ mongoc_apm_command_started_init (mongoc_apm_command_started_t *event,
    event->host = host;
    event->server_id = server_id;
    event->context = context;
-
-   bson_oid_copy_unsafe (service_id, &event->service_id);
 }
 
 
-/*--------------------------------------------------------------------------
- *
- * mongoc_apm_command_started_init_with_cmd --
- *
- *       Initialises the command started event from a mongoc_cmd_t.
- *
- * Side effects:
- *       If provided, is_redacted indicates whether the command document was
- *       redacted to hide sensitive information.
- *
- *--------------------------------------------------------------------------
- */
 void
 mongoc_apm_command_started_init_with_cmd (mongoc_apm_command_started_t *event,
                                           mongoc_cmd_t *cmd,
                                           int64_t request_id,
-                                          bool *is_redacted, /* out */
                                           void *context)
 {
    mongoc_apm_command_started_init (event,
@@ -164,8 +111,6 @@ mongoc_apm_command_started_init_with_cmd (mongoc_apm_command_started_t *event,
                                     cmd->operation_id,
                                     &cmd->server_stream->sd->host,
                                     cmd->server_stream->sd->id,
-                                    &cmd->server_stream->sd->service_id,
-                                    is_redacted,
                                     context);
 
    /* OP_MSG document sequence for insert, update, or delete? */
@@ -182,18 +127,6 @@ mongoc_apm_command_started_cleanup (mongoc_apm_command_started_t *event)
 }
 
 
-/*--------------------------------------------------------------------------
- *
- * mongoc_apm_command_succeeded_init --
- *
- *       Initialises the command succeeded event.
- *
- * Parameters:
- *       @force_redaction: If true, the reply document is always redacted,
- *       regardless of whether the command contains sensitive information.
- *
- *--------------------------------------------------------------------------
- */
 void
 mongoc_apm_command_succeeded_init (mongoc_apm_command_succeeded_t *event,
                                    int64_t duration,
@@ -203,57 +136,28 @@ mongoc_apm_command_succeeded_init (mongoc_apm_command_succeeded_t *event,
                                    int64_t operation_id,
                                    const mongoc_host_list_t *host,
                                    uint32_t server_id,
-                                   const bson_oid_t *service_id,
-                                   bool force_redaction,
                                    void *context)
 {
    BSON_ASSERT (reply);
 
-   if (force_redaction ||
-       mongoc_apm_is_sensitive_command_message (command_name, reply)) {
-      event->reply = bson_copy (reply);
-      event->reply_owned = true;
-
-      mongoc_apm_redact_reply (event->reply);
-   } else {
-      /* discard "const", we promise not to modify "reply" */
-      event->reply = (bson_t *) reply;
-      event->reply_owned = false;
-   }
-
    event->duration = duration;
+   event->reply = reply;
    event->command_name = command_name;
    event->request_id = request_id;
    event->operation_id = operation_id;
    event->host = host;
    event->server_id = server_id;
    event->context = context;
-
-   bson_oid_copy_unsafe (service_id, &event->service_id);
 }
 
 
 void
 mongoc_apm_command_succeeded_cleanup (mongoc_apm_command_succeeded_t *event)
 {
-   if (event->reply_owned) {
-      bson_destroy (event->reply);
-   }
+   /* no-op */
 }
 
 
-/*--------------------------------------------------------------------------
- *
- * mongoc_apm_command_failed_init --
- *
- *       Initialises the command failed event.
- *
- * Parameters:
- *       @force_redaction: If true, the reply document is always redacted,
- *       regardless of whether the command contains sensitive information.
- *
- *--------------------------------------------------------------------------
- */
 void
 mongoc_apm_command_failed_init (mongoc_apm_command_failed_t *event,
                                 int64_t duration,
@@ -264,43 +168,26 @@ mongoc_apm_command_failed_init (mongoc_apm_command_failed_t *event,
                                 int64_t operation_id,
                                 const mongoc_host_list_t *host,
                                 uint32_t server_id,
-                                const bson_oid_t *service_id,
-                                bool force_redaction,
                                 void *context)
 {
    BSON_ASSERT (reply);
 
-   if (force_redaction ||
-       mongoc_apm_is_sensitive_command_message (command_name, reply)) {
-      event->reply = bson_copy (reply);
-      event->reply_owned = true;
-
-      mongoc_apm_redact_reply (event->reply);
-   } else {
-      /* discard "const", we promise not to modify "reply" */
-      event->reply = (bson_t *) reply;
-      event->reply_owned = false;
-   }
-
    event->duration = duration;
    event->command_name = command_name;
    event->error = error;
+   event->reply = reply;
    event->request_id = request_id;
    event->operation_id = operation_id;
    event->host = host;
    event->server_id = server_id;
    event->context = context;
-
-   bson_oid_copy_unsafe (service_id, &event->service_id);
 }
 
 
 void
 mongoc_apm_command_failed_cleanup (mongoc_apm_command_failed_t *event)
 {
-   if (event->reply_owned) {
-      bson_destroy (event->reply);
-   }
+   /* no-op */
 }
 
 
@@ -365,19 +252,6 @@ mongoc_apm_command_started_get_server_id (
 }
 
 
-const bson_oid_t *
-mongoc_apm_command_started_get_service_id (
-   const mongoc_apm_command_started_t *event)
-{
-   if (0 == bson_oid_compare (&event->service_id, &kObjectIdZero)) {
-      /* serviceId is unset. */
-      return NULL;
-   }
-
-   return &event->service_id;
-}
-
-
 void *
 mongoc_apm_command_started_get_context (
    const mongoc_apm_command_started_t *event)
@@ -444,19 +318,6 @@ mongoc_apm_command_succeeded_get_server_id (
 }
 
 
-const bson_oid_t *
-mongoc_apm_command_succeeded_get_service_id (
-   const mongoc_apm_command_succeeded_t *event)
-{
-   if (0 == bson_oid_compare (&event->service_id, &kObjectIdZero)) {
-      /* serviceId is unset. */
-      return NULL;
-   }
-
-   return &event->service_id;
-}
-
-
 void *
 mongoc_apm_command_succeeded_get_context (
    const mongoc_apm_command_succeeded_t *event)
@@ -527,19 +388,6 @@ mongoc_apm_command_failed_get_server_id (
 }
 
 
-const bson_oid_t *
-mongoc_apm_command_failed_get_service_id (
-   const mongoc_apm_command_failed_t *event)
-{
-   if (0 == bson_oid_compare (&event->service_id, &kObjectIdZero)) {
-      /* serviceId is unset. */
-      return NULL;
-   }
-
-   return &event->service_id;
-}
-
-
 void *
 mongoc_apm_command_failed_get_context (const mongoc_apm_command_failed_t *event)
 {
@@ -722,13 +570,6 @@ mongoc_apm_server_heartbeat_started_get_context (
    return event->context;
 }
 
-bool
-mongoc_apm_server_heartbeat_started_get_awaited (
-   const mongoc_apm_server_heartbeat_started_t *event)
-{
-   return event->awaited;
-}
-
 
 /* heartbeat-succeeded event fields */
 
@@ -763,13 +604,6 @@ mongoc_apm_server_heartbeat_succeeded_get_context (
    return event->context;
 }
 
-bool
-mongoc_apm_server_heartbeat_succeeded_get_awaited (
-   const mongoc_apm_server_heartbeat_succeeded_t *event)
-{
-   return event->awaited;
-}
-
 
 /* heartbeat-failed event fields */
 
@@ -804,12 +638,6 @@ mongoc_apm_server_heartbeat_failed_get_context (
    return event->context;
 }
 
-bool
-mongoc_apm_server_heartbeat_failed_get_awaited (
-   const mongoc_apm_server_heartbeat_failed_t *event)
-{
-   return event->awaited;
-}
 
 /*
  * registering callbacks
@@ -927,70 +755,3 @@ mongoc_apm_set_server_heartbeat_failed_cb (
 {
    callbacks->server_heartbeat_failed = cb;
 }
-
-static bool
-_mongoc_apm_is_sensitive_command_name (const char *command_name)
-{
-   return 0 == strcasecmp (command_name, "authenticate") ||
-          0 == strcasecmp (command_name, "saslStart") ||
-          0 == strcasecmp (command_name, "saslContinue") ||
-          0 == strcasecmp (command_name, "getnonce") ||
-          0 == strcasecmp (command_name, "createUser") ||
-          0 == strcasecmp (command_name, "updateUser") ||
-          0 == strcasecmp (command_name, "copydbgetnonce") ||
-          0 == strcasecmp (command_name, "copydbsaslstart") ||
-          0 == strcasecmp (command_name, "copydb");
-}
-
-static bool
-_mongoc_apm_is_sensitive_hello_message (const char *command_name,
-                                        const bson_t *body)
-{
-   const bool is_hello =
-      (0 == strcasecmp (command_name, "hello") ||
-       0 == strcasecmp (command_name, HANDSHAKE_CMD_LEGACY_HELLO));
-
-   if (!is_hello) {
-      return false;
-   }
-   if (bson_empty (body)) {
-      /* An empty message body means that it has been redacted */
-      return true;
-   } else if (bson_has_field (body, "speculativeAuthenticate")) {
-      /* "hello" messages are only sensitive if they contain
-       * 'speculativeAuthenticate' */
-      return true;
-   } else {
-      /* Other "hello" messages are okay */
-      return false;
-   }
-}
-
-bool
-mongoc_apm_is_sensitive_command_message (const char *command_name,
-                                         const bson_t *body)
-{
-   BSON_ASSERT (body);
-
-   return _mongoc_apm_is_sensitive_command_name (command_name) ||
-          _mongoc_apm_is_sensitive_hello_message (command_name, body);
-}
-
-void
-mongoc_apm_redact_command (bson_t *command)
-{
-   BSON_ASSERT (command);
-
-   /* Reinit the command to have an empty document */
-   bson_reinit (command);
-}
-
-
-void
-mongoc_apm_redact_reply (bson_t *reply)
-{
-   BSON_ASSERT (reply);
-
-   /* Reinit the reply to have an empty document */
-   bson_reinit (reply);
-}
