@@ -50,8 +50,6 @@ struct _mongoc_client_pool_t {
    void *apm_context;
    int32_t error_api_version;
    bool error_api_set;
-   mongoc_server_api_t *api;
-   bool client_initialized;
 };
 
 
@@ -85,7 +83,6 @@ _mongoc_client_pool_set_internal_tls_opts (
 {
    bson_mutex_lock (&pool->mutex);
    if (!pool->ssl_opts_set) {
-      bson_mutex_unlock (&pool->mutex);
       return;
    }
    pool->ssl_opts.internal = bson_malloc (sizeof (_mongoc_internal_tls_opts_t));
@@ -185,7 +182,7 @@ mongoc_client_pool_destroy (mongoc_client_pool_t *pool)
       EXIT;
    }
 
-   if (!mongoc_server_session_pool_is_empty (pool->topology->session_pool)) {
+   if (pool->topology->session_pool) {
       client = mongoc_client_pool_pop (pool);
       _mongoc_client_end_sessions (client);
       mongoc_client_pool_push (pool, client);
@@ -202,8 +199,6 @@ mongoc_client_pool_destroy (mongoc_client_pool_t *pool)
    bson_mutex_destroy (&pool->mutex);
    mongoc_cond_destroy (&pool->cond);
 
-   mongoc_server_api_destroy (pool->api);
-
 #ifdef MONGOC_ENABLE_SSL
    _mongoc_ssl_opts_cleanup (&pool->ssl_opts, true);
 #endif
@@ -226,7 +221,9 @@ static void
 _start_scanner_if_needed (mongoc_client_pool_t *pool)
 {
    if (!pool->topology->single_threaded) {
+      bson_mutex_lock (&pool->topology->mutex);
       _mongoc_topology_background_monitoring_start (pool->topology);
+      bson_mutex_unlock (&pool->topology->mutex);
    }
 }
 
@@ -239,14 +236,10 @@ _initialize_new_client (mongoc_client_pool_t *pool, mongoc_client_t *client)
       pool->topology->scanner->initiator,
       pool->topology->scanner->initiator_context);
 
-   pool->client_initialized = true;
-   client->is_pooled = true;
    client->error_api_version = pool->error_api_version;
    _mongoc_client_set_apm_callbacks_private (
       client, &pool->apm_callbacks, pool->apm_context);
 
-   client->api = mongoc_server_api_copy (pool->api);
-
 #ifdef MONGOC_ENABLE_SSL
    if (pool->ssl_opts_set) {
       mongoc_client_set_ssl_opts (client, &pool->ssl_opts);
@@ -444,30 +437,30 @@ mongoc_client_pool_set_apm_callbacks (mongoc_client_pool_t *pool,
                                       mongoc_apm_callbacks_t *callbacks,
                                       void *context)
 {
-   mongoc_topology_t *const topology = BSON_ASSERT_PTR_INLINE (pool)->topology;
-   mc_tpld_modification tdmod;
+   mongoc_topology_t *topology;
+
+   topology = pool->topology;
 
    if (pool->apm_callbacks_set) {
       MONGOC_ERROR ("Can only set callbacks once");
       return false;
    }
 
-   tdmod = mc_tpld_modify_begin (topology);
+   bson_mutex_lock (&topology->mutex);
 
    if (callbacks) {
-      memcpy (&tdmod.new_td->apm_callbacks,
+      memcpy (&topology->description.apm_callbacks,
               callbacks,
               sizeof (mongoc_apm_callbacks_t));
       memcpy (&pool->apm_callbacks, callbacks, sizeof (mongoc_apm_callbacks_t));
    }
 
-   mongoc_topology_set_apm_callbacks (
-      topology, tdmod.new_td, callbacks, context);
-   tdmod.new_td->apm_context = context;
+   mongoc_topology_set_apm_callbacks (topology, callbacks, context);
+   topology->description.apm_context = context;
    pool->apm_context = context;
    pool->apm_callbacks_set = true;
 
-   mc_tpld_modify_commit (tdmod);
+   bson_mutex_unlock (&topology->mutex);
 
    return true;
 }
@@ -512,34 +505,3 @@ mongoc_client_pool_enable_auto_encryption (mongoc_client_pool_t *pool,
    return _mongoc_cse_client_pool_enable_auto_encryption (
       pool->topology, opts, error);
 }
-
-bool
-mongoc_client_pool_set_server_api (mongoc_client_pool_t *pool,
-                                   const mongoc_server_api_t *api,
-                                   bson_error_t *error)
-{
-   BSON_ASSERT_PARAM (pool);
-   BSON_ASSERT_PARAM (api);
-
-   if (pool->api) {
-      bson_set_error (error,
-                      MONGOC_ERROR_POOL,
-                      MONGOC_ERROR_POOL_API_ALREADY_SET,
-                      "Cannot set server api more than once per pool");
-      return false;
-   }
-
-   if (pool->client_initialized) {
-      bson_set_error (error,
-                      MONGOC_ERROR_POOL,
-                      MONGOC_ERROR_POOL_API_TOO_LATE,
-                      "Cannot set server api after a client has been created");
-      return false;
-   }
-
-   pool->api = mongoc_server_api_copy (api);
-
-   _mongoc_topology_scanner_set_server_api (pool->topology->scanner, api);
-
-   return true;
-}
