@@ -448,6 +448,7 @@ _mongoc_get_rr_search (const char *service,
    bool dns_success;
    bool callback_success = true;
    int num_matching_records;
+   uint32_t ttl;
 
    ENTRY;
 
@@ -536,13 +537,9 @@ _mongoc_get_rr_search (const char *service,
 
       num_matching_records++;
 
-      if (rr_data) {
-         uint32_t ttl;
-
-         ttl = ns_rr_ttl (resource_record);
-         if ((i == 0) || (ttl < rr_data->min_ttl)) {
-            rr_data->min_ttl = ttl;
-         }
+      ttl = ns_rr_ttl (resource_record);
+      if ((i == 0) || (ttl < rr_data->min_ttl)) {
+         rr_data->min_ttl = ttl;
       }
 
       if (!callback (service, &ns_answer, &resource_record, rr_data, error)) {
@@ -604,6 +601,8 @@ _mongoc_client_get_rr (const char *service,
                        size_t initial_buffer_size,
                        bson_error_t *error)
 {
+   BSON_ASSERT (rr_data);
+
 #ifdef MONGOC_HAVE_DNSAPI
    return _mongoc_get_rr_dnsapi (service, rr_type, rr_data, error);
 #elif (defined(MONGOC_HAVE_RES_NSEARCH) || defined(MONGOC_HAVE_RES_SEARCH))
@@ -1227,6 +1226,7 @@ mongoc_client_destroy (mongoc_client_t *client)
       mongoc_cluster_destroy (&client->cluster);
       mongoc_uri_destroy (client->uri);
       mongoc_set_destroy (client->client_sessions);
+      mongoc_server_api_destroy (client->api);
 
 #ifdef MONGOC_ENABLE_SSL
       _mongoc_ssl_opts_cleanup (&client->ssl_opts, true);
@@ -1304,6 +1304,17 @@ mongoc_client_start_session (mongoc_client_t *client,
       csid = (uint32_t) _mongoc_rand_simple (&client->csid_rand_seed);
    } while (mongoc_set_get (client->client_sessions, csid));
 
+   /* causal consistency and snapshot cannot both be set. */
+   if (opts && mongoc_session_opts_get_causal_consistency (opts) &&
+       mongoc_session_opts_get_snapshot (opts)) {
+      bson_set_error (
+         error,
+         MONGOC_ERROR_CLIENT,
+         MONGOC_ERROR_CLIENT_SESSION_FAILURE,
+         "Only one of causal consistency and snapshot can be enabled.");
+      _mongoc_client_push_server_session (client, ss);
+      RETURN (NULL);
+   }
    cs = _mongoc_client_session_new (client, ss, opts, csid);
 
    /* remember session so if we see its client_session_id in a command, we can
@@ -1730,9 +1741,8 @@ retry:
       retry_server_stream = mongoc_cluster_stream_for_writes (
          &client->cluster, parts->assembled.session, NULL, &ignored_error);
 
-      if (retry_server_stream &&
-          retry_server_stream->sd->max_wire_version >=
-             WIRE_VERSION_RETRY_WRITES) {
+      if (retry_server_stream && retry_server_stream->sd->max_wire_version >=
+                                    WIRE_VERSION_RETRY_WRITES) {
          parts->assembled.server_stream = retry_server_stream;
          bson_destroy (reply);
          GOTO (retry);
@@ -1781,9 +1791,8 @@ retry:
     * a new readable stream and retry. If server selection fails or the selected
     * server does not support retryable reads, fall through and allow the
     * original error to be reported. */
-   if (is_retryable &&
-       _mongoc_read_error_get_type (ret, error, reply) ==
-          MONGOC_READ_ERR_RETRY) {
+   if (is_retryable && _mongoc_read_error_get_type (ret, error, reply) ==
+                          MONGOC_READ_ERR_RETRY) {
       bson_error_t ignored_error;
 
       /* each read command may be retried at most once */
@@ -1800,9 +1809,8 @@ retry:
                                           NULL,
                                           &ignored_error);
 
-      if (retry_server_stream &&
-          retry_server_stream->sd->max_wire_version >=
-             WIRE_VERSION_RETRY_READS) {
+      if (retry_server_stream && retry_server_stream->sd->max_wire_version >=
+                                    WIRE_VERSION_RETRY_READS) {
          parts->assembled.server_stream = retry_server_stream;
          bson_destroy (reply);
          GOTO (retry);
@@ -2040,7 +2048,7 @@ _mongoc_client_command_with_opts (mongoc_client_t *client,
                                            error);
 
       if (server_stream && server_stream->sd->type != MONGOC_SERVER_MONGOS) {
-         parts.user_query_flags |= MONGOC_QUERY_SLAVE_OK;
+         parts.user_query_flags |= MONGOC_QUERY_SECONDARY_OK;
       }
    } else if (parts.is_write_command) {
       server_stream =
@@ -2356,6 +2364,7 @@ _mongoc_client_monitor_op_killcursors (mongoc_cluster_t *cluster,
                                     operation_id,
                                     &server_stream->sd->host,
                                     server_stream->sd->id,
+                                    &server_stream->sd->service_id,
                                     NULL,
                                     client->apm_context);
 
@@ -2403,6 +2412,7 @@ _mongoc_client_monitor_op_killcursors_succeeded (
                                       operation_id,
                                       &server_stream->sd->host,
                                       server_stream->sd->id,
+                                      &server_stream->sd->service_id,
                                       false,
                                       client->apm_context);
 
@@ -2446,6 +2456,7 @@ _mongoc_client_monitor_op_killcursors_failed (
                                    operation_id,
                                    &server_stream->sd->host,
                                    server_stream->sd->id,
+                                   &server_stream->sd->service_id,
                                    false,
                                    client->apm_context);
 
@@ -2527,7 +2538,7 @@ _mongoc_client_killcursors_command (mongoc_cluster_t *cluster,
 
    _mongoc_client_prepare_killcursors_command (cursor_id, collection, &command);
    mongoc_cmd_parts_init (
-      &parts, cluster->client, db, MONGOC_QUERY_SLAVE_OK, &command);
+      &parts, cluster->client, db, MONGOC_QUERY_SECONDARY_OK, &command);
    parts.assembled.operation_id = ++cluster->operation_id;
    mongoc_cmd_parts_set_session (&parts, cs);
 
@@ -3024,7 +3035,7 @@ _mongoc_client_end_sessions (mongoc_client_t *client)
       /* end sessions in chunks */
       while (_mongoc_topology_end_sessions_cmd (t, &cmd)) {
          mongoc_cmd_parts_init (
-            &parts, client, "admin", MONGOC_QUERY_SLAVE_OK, &cmd);
+            &parts, client, "admin", MONGOC_QUERY_SECONDARY_OK, &cmd);
          parts.assembled.operation_id = ++cluster->operation_id;
          parts.prohibit_lsid = true;
 
@@ -3102,3 +3113,59 @@ mongoc_client_enable_auto_encryption (mongoc_client_t *client,
    }
    return _mongoc_cse_client_enable_auto_encryption (client, opts, error);
 }
+
+bool
+mongoc_client_set_server_api (mongoc_client_t *client,
+                              const mongoc_server_api_t *api,
+                              bson_error_t *error)
+{
+   BSON_ASSERT_PARAM (client);
+   BSON_ASSERT_PARAM (api);
+
+   if (client->is_pooled) {
+      bson_set_error (
+         error,
+         MONGOC_ERROR_CLIENT,
+         MONGOC_ERROR_CLIENT_API_FROM_POOL,
+         "Cannot set server api on a client checked out from a pool");
+      return false;
+   }
+
+   if (client->api) {
+      bson_set_error (error,
+                      MONGOC_ERROR_CLIENT,
+                      MONGOC_ERROR_CLIENT_API_ALREADY_SET,
+                      "Cannot set server api more than once per client");
+      return false;
+   }
+
+   client->api = mongoc_server_api_copy (api);
+   bson_mutex_lock (&client->topology->mutex);
+   _mongoc_topology_scanner_set_server_api (client->topology->scanner, api);
+   bson_mutex_unlock (&client->topology->mutex);
+   return true;
+}
+
+mongoc_server_description_t *
+mongoc_client_get_handshake_description (mongoc_client_t *client,
+                                         uint32_t server_id,
+                                         bson_t *opts,
+                                         bson_error_t *error)
+{
+   mongoc_server_stream_t *server_stream;
+   mongoc_server_description_t *sd;
+
+   server_stream = mongoc_cluster_stream_for_server (&client->cluster,
+                                                     server_id,
+                                                     true /* reconnect */,
+                                                     NULL /* client session */,
+                                                     NULL /* reply */,
+                                                     error);
+   if (!server_stream) {
+      return NULL;
+   }
+
+   sd = mongoc_server_description_new_copy (server_stream->sd);
+   mongoc_server_stream_cleanup (server_stream);
+   return sd;
+}
