@@ -26,15 +26,9 @@
 #include "mongoc-collection-private.h"
 #include "mongoc-host-list-private.h"
 #include "mongoc-stream-private.h"
-#include "mongoc-ssl-private.h"
-#include "mongoc-util-private.h"
 
 struct __mongoc_crypt_t {
    mongocrypt_t *handle;
-   mongoc_ssl_opt_t kmip_tls_opt;
-   mongoc_ssl_opt_t aws_tls_opt;
-   mongoc_ssl_opt_t azure_tls_opt;
-   mongoc_ssl_opt_t gcp_tls_opt;
 };
 
 static void
@@ -199,15 +193,12 @@ typedef struct {
    mongoc_client_t *mongocryptd_client;
    mongoc_client_t *collinfo_client;
    const char *db_name;
-   _mongoc_crypt_t *crypt;
 } _state_machine_t;
 
 _state_machine_t *
-_state_machine_new (_mongoc_crypt_t *crypt)
+_state_machine_new (void)
 {
-   _state_machine_t *sm = bson_malloc0 (sizeof (_state_machine_t));
-   sm->crypt = crypt;
-   return sm;
+   return bson_malloc0 (sizeof (_state_machine_t));
 }
 
 void
@@ -413,16 +404,22 @@ fail:
 static mongoc_stream_t *
 _get_stream (const char *endpoint,
              int32_t connecttimeoutms,
-             const mongoc_ssl_opt_t *ssl_opt,
              bson_error_t *error)
 {
    mongoc_stream_t *base_stream = NULL;
    mongoc_stream_t *tls_stream = NULL;
    bool ret = false;
-   mongoc_ssl_opt_t ssl_opt_copy = {0};
+   mongoc_ssl_opt_t ssl_opts = {0};
    mongoc_host_list_t host;
+   char *host_and_port = NULL;
 
-   if (!_mongoc_host_list_from_string_with_err (&host, endpoint, error)) {
+   if (!strchr (endpoint, ':')) {
+      host_and_port = bson_strdup_printf ("%s:443", endpoint);
+   } else {
+      host_and_port = (char *) endpoint; /* we promise not to modify */
+   }
+
+   if (!_mongoc_host_list_from_string_with_err (&host, host_and_port, error)) {
       goto fail;
    }
 
@@ -432,9 +429,9 @@ _get_stream (const char *endpoint,
    }
 
    /* Wrap in a tls_stream. */
-   _mongoc_ssl_opts_copy_to (ssl_opt, &ssl_opt_copy, true /* copy_internal */);
+   memcpy (&ssl_opts, mongoc_ssl_opt_get_default (), sizeof ssl_opts);
    tls_stream = mongoc_stream_tls_new_with_hostname (
-      base_stream, host.host, &ssl_opt_copy, 1 /* client */);
+      base_stream, host.host, &ssl_opts, 1 /* client */);
 
    if (!mongoc_stream_tls_handshake_block (
           tls_stream, host.host, connecttimeoutms, error)) {
@@ -443,7 +440,9 @@ _get_stream (const char *endpoint,
 
    ret = true;
 fail:
-   _mongoc_ssl_opts_cleanup (&ssl_opt_copy, true /* free_internal */);
+   if (host_and_port != endpoint) {
+      bson_free (host_and_port);
+   }
    if (!ret) {
       if (tls_stream) {
          /* destroys base_stream too */
@@ -471,22 +470,6 @@ _state_need_kms (_state_machine_t *state_machine, bson_error_t *error)
    kms_ctx = mongocrypt_ctx_next_kms_ctx (state_machine->ctx);
    while (kms_ctx) {
       mongoc_iovec_t iov;
-      const mongoc_ssl_opt_t *ssl_opt;
-      const char *provider;
-
-      provider = mongocrypt_kms_ctx_get_kms_provider (kms_ctx, NULL);
-
-      if (0 == strcmp ("kmip", provider)) {
-         ssl_opt = &state_machine->crypt->kmip_tls_opt;
-      } else if (0 == strcmp ("aws", provider)) {
-         ssl_opt = &state_machine->crypt->aws_tls_opt;
-      } else if (0 == strcmp ("azure", provider)) {
-         ssl_opt = &state_machine->crypt->azure_tls_opt;
-      } else if (0 == strcmp ("gcp", provider)) {
-         ssl_opt = &state_machine->crypt->gcp_tls_opt;
-      } else {
-         ssl_opt = mongoc_ssl_opt_get_default ();
-      }
 
       mongocrypt_binary_destroy (http_req);
       http_req = mongocrypt_binary_new ();
@@ -501,11 +484,11 @@ _state_need_kms (_state_machine_t *state_machine, bson_error_t *error)
       }
 
       mongoc_stream_destroy (tls_stream);
-      tls_stream = _get_stream (endpoint, sockettimeout, ssl_opt, error);
+      tls_stream = _get_stream (endpoint, sockettimeout, error);
 #ifdef MONGOC_ENABLE_SSL_SECURE_CHANNEL
       /* Retry once with schannel as a workaround for CDRIVER-3566. */
       if (!tls_stream) {
-         tls_stream = _get_stream (endpoint, sockettimeout, ssl_opt, error);
+         tls_stream = _get_stream (endpoint, sockettimeout, error);
       }
 #endif
       if (!tls_stream) {
@@ -675,231 +658,6 @@ fail:
    return ret;
 }
 
-/* _parse_one_tls_opts parses one TLS document.
- * Pre-conditions:
- * - @iter is an iterator at the start of a KMS provider key/value pair.
- * - @out_opt must not be initialized.
- * Post-conditions:
- * - @out_opt is always initialized.
- * Returns false and sets @error on error. */
-static bool
-_parse_one_tls_opts (bson_iter_t *iter,
-                     mongoc_ssl_opt_t *out_opt,
-                     bson_error_t *error)
-{
-   bool ok = false;
-   const char *kms_provider;
-   bson_t tls_opts_doc;
-   const uint8_t *data;
-   uint32_t len;
-   bson_string_t *errmsg;
-   bson_iter_t permitted_iter;
-
-   errmsg = bson_string_new (NULL);
-   kms_provider = bson_iter_key (iter);
-   memset (out_opt, 0, sizeof (mongoc_ssl_opt_t));
-
-   if (!BSON_ITER_HOLDS_DOCUMENT (iter)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-                      MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-                      "Expected TLS options for %s to be a document, got: %s",
-                      kms_provider,
-                      _mongoc_bson_type_to_str (bson_iter_type (iter)));
-      goto fail;
-   }
-
-   bson_iter_document (iter, &len, &data);
-   if (!bson_init_static (&tls_opts_doc, data, len) ||
-       !bson_iter_init (&permitted_iter, &tls_opts_doc)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-                      MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-                      "Error iterating into TLS options document for %s",
-                      kms_provider);
-      goto fail;
-   }
-
-   while (bson_iter_next (&permitted_iter)) {
-      const char *key = bson_iter_key (&permitted_iter);
-
-      if (0 ==
-          bson_strcasecmp (key, MONGOC_URI_TLSCERTIFICATEKEYFILEPASSWORD)) {
-         continue;
-      }
-
-      if (0 == bson_strcasecmp (key, MONGOC_URI_TLSCERTIFICATEKEYFILE)) {
-         continue;
-      }
-
-      if (0 == bson_strcasecmp (key, MONGOC_URI_TLSCAFILE)) {
-         continue;
-      }
-
-      bson_set_error (
-         error,
-         MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-         MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-         "Error setting TLS option %s for %s. Insecure TLS options prohibited.",
-         key,
-         kms_provider);
-      goto fail;
-   }
-
-   if (!_mongoc_ssl_opts_from_bson (out_opt, &tls_opts_doc, errmsg)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-                      MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-                      "Error parsing TLS options for %s: %s",
-                      kms_provider,
-                      errmsg->str);
-      goto fail;
-   }
-
-   ok = true;
-fail:
-   bson_string_free (errmsg, true /* free_segment */);
-   return ok;
-}
-
-/* _parse_all_tls_opts initializes TLS options for all KMS providers.
- * @tls_opts is the BSON document passed through
- * mongoc_client_encryption_opts_set_tls_opts or
- * mongoc_auto_encryption_opts_set_tls_opts.
- * Defaults to using mongoc_ssl_opt_get_default() if options are not passed for
- * a provider. Returns false and sets @error on error. */
-static bool
-_parse_all_tls_opts (_mongoc_crypt_t *crypt,
-                     const bson_t *tls_opts,
-                     bson_error_t *error)
-{
-   bson_iter_t iter;
-   bool ok = false;
-   bool has_aws = false;
-   bool has_azure = false;
-   bool has_gcp = false;
-   bool has_kmip = false;
-
-   if (!tls_opts) {
-      return true;
-   }
-
-   if (!bson_iter_init (&iter, tls_opts)) {
-      bson_set_error (error,
-                      MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-                      MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-                      "Error starting iteration of TLS options");
-      goto fail;
-   }
-
-   while (bson_iter_next (&iter)) {
-      const char *key;
-
-      key = bson_iter_key (&iter);
-      if (0 == strcmp (key, "aws")) {
-         if (has_aws) {
-            bson_set_error (error,
-                           MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-                           MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-                           "Error parsing duplicate TLS options for %s", key);
-            goto fail;
-         }
-
-         has_aws = true;
-         if (!_parse_one_tls_opts (&iter, &crypt->aws_tls_opt, error)) {
-            goto fail;
-         }
-         continue;
-      }
-
-      if (0 == strcmp (key, "azure")) {
-         if (has_azure) {
-            bson_set_error (error,
-                           MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-                           MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-                           "Error parsing duplicate TLS options for %s", key);
-            goto fail;
-         }
-
-         has_azure = true;
-         if (!_parse_one_tls_opts (&iter, &crypt->azure_tls_opt, error)) {
-            goto fail;
-         }
-         continue;
-      }
-
-      if (0 == strcmp (key, "gcp")) {
-         if (has_gcp) {
-            bson_set_error (error,
-                           MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-                           MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-                           "Error parsing duplicate TLS options for %s", key);
-            goto fail;
-         }
-
-         has_gcp = true;
-         if (!_parse_one_tls_opts (&iter, &crypt->gcp_tls_opt, error)) {
-            goto fail;
-         }
-         continue;
-      }
-
-      if (0 == strcmp (key, "kmip")) {
-         if (has_kmip) {
-            bson_set_error (error,
-                           MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-                           MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-                           "Error parsing duplicate TLS options for %s", key);
-            goto fail;
-         }
-
-         has_kmip = true;
-         if (!_parse_one_tls_opts (&iter, &crypt->kmip_tls_opt, error)) {
-            goto fail;
-         }
-         continue;
-      }
-
-      bson_set_error (error,
-                      MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION,
-                      MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
-                      "Cannot configure TLS options for KMS provider: %s",
-                      key);
-      goto fail;
-   }
-
-   /* Configure with default TLS options. The mongoc_ssl_opt_t returned by
-    * mongoc_ssl_opt_get_default may contain non-NULL fields if
-    * MONGOC_SSL_DEFAULT_TRUST_FILE or MONGOC_SSL_DEFAULT_TRUST_DIR are defined.
-    */
-   if (!has_aws) {
-      _mongoc_ssl_opts_copy_to (mongoc_ssl_opt_get_default (),
-                                &crypt->aws_tls_opt,
-                                false /* copy internal */);
-   }
-
-   if (!has_azure) {
-      _mongoc_ssl_opts_copy_to (mongoc_ssl_opt_get_default (),
-                                &crypt->azure_tls_opt,
-                                false /* copy internal */);
-   }
-
-   if (!has_gcp) {
-      _mongoc_ssl_opts_copy_to (mongoc_ssl_opt_get_default (),
-                                &crypt->gcp_tls_opt,
-                                false /* copy internal */);
-   }
-
-   if (!has_kmip) {
-      _mongoc_ssl_opts_copy_to (mongoc_ssl_opt_get_default (),
-                                &crypt->kmip_tls_opt,
-                                false /* copy internal */);
-   }
-   ok = true;
-fail:
-   return ok;
-}
-
 /* Note, _mongoc_crypt_t only has one member, to the top-level handle of
    libmongocrypt, mongocrypt_t.
    The purpose of defining _mongoc_crypt_t is to limit all interaction with
@@ -909,7 +667,6 @@ fail:
 _mongoc_crypt_t *
 _mongoc_crypt_new (const bson_t *kms_providers,
                    const bson_t *schema_map,
-                   const bson_t *tls_opts,
                    bson_error_t *error)
 {
    _mongoc_crypt_t *crypt;
@@ -922,10 +679,6 @@ _mongoc_crypt_new (const bson_t *kms_providers,
    crypt = bson_malloc0 (sizeof (*crypt));
    crypt->handle = mongocrypt_new ();
 
-   if (!_parse_all_tls_opts (crypt, tls_opts, error)) {
-      goto fail;
-   }
-
    mongocrypt_setopt_log_handler (
       crypt->handle, _log_callback, NULL /* context */);
 
@@ -971,10 +724,6 @@ _mongoc_crypt_destroy (_mongoc_crypt_t *crypt)
       return;
    }
    mongocrypt_destroy (crypt->handle);
-   _mongoc_ssl_opts_cleanup (&crypt->kmip_tls_opt, true /* free_internal */);
-   _mongoc_ssl_opts_cleanup (&crypt->aws_tls_opt, true /* free_internal */);
-   _mongoc_ssl_opts_cleanup (&crypt->azure_tls_opt, true /* free_internal */);
-   _mongoc_ssl_opts_cleanup (&crypt->gcp_tls_opt, true /* free_internal */);
    bson_free (crypt);
 }
 
@@ -994,7 +743,7 @@ _mongoc_crypt_auto_encrypt (_mongoc_crypt_t *crypt,
 
    bson_init (cmd_out);
 
-   state_machine = _state_machine_new (crypt);
+   state_machine = _state_machine_new ();
    state_machine->keyvault_coll = keyvault_coll;
    state_machine->mongocryptd_client = mongocryptd_client;
    state_machine->collinfo_client = collinfo_client;
@@ -1038,7 +787,7 @@ _mongoc_crypt_auto_decrypt (_mongoc_crypt_t *crypt,
 
    bson_init (doc_out);
 
-   state_machine = _state_machine_new (crypt);
+   state_machine = _state_machine_new ();
    state_machine->keyvault_coll = keyvault_coll;
    state_machine->ctx = mongocrypt_ctx_new (crypt->handle);
    if (!state_machine->ctx) {
@@ -1085,7 +834,7 @@ _mongoc_crypt_explicit_encrypt (_mongoc_crypt_t *crypt,
    value_out->value_type = BSON_TYPE_EOD;
 
    /* Create the context for the operation. */
-   state_machine = _state_machine_new (crypt);
+   state_machine = _state_machine_new ();
    state_machine->keyvault_coll = keyvault_coll;
    state_machine->ctx = mongocrypt_ctx_new (crypt->handle);
    if (!state_machine->ctx) {
@@ -1190,7 +939,7 @@ _mongoc_crypt_explicit_decrypt (_mongoc_crypt_t *crypt,
    bool ret = false;
    bson_t result = BSON_INITIALIZER;
 
-   state_machine = _state_machine_new (crypt);
+   state_machine = _state_machine_new ();
    state_machine->keyvault_coll = keyvault_coll;
    state_machine->ctx = mongocrypt_ctx_new (crypt->handle);
    if (!state_machine->ctx) {
@@ -1251,7 +1000,7 @@ _mongoc_crypt_create_datakey (_mongoc_crypt_t *crypt,
    mongocrypt_binary_t *masterkey_w_provider_bin = NULL;
 
    bson_init (doc_out);
-   state_machine = _state_machine_new (crypt);
+   state_machine = _state_machine_new ();
    state_machine->ctx = mongocrypt_ctx_new (crypt->handle);
    if (!state_machine->ctx) {
       _crypt_check_error (crypt->handle, error, true);
