@@ -63,10 +63,10 @@ struct _mongoc_server_monitor_t {
       bool cancel_requested;
    } shared;
 
-   /* Default time to sleep between hello checks (reduced when a scan is
+   /* Default time to sleep between ismaster checks (reduced when a scan is
     * requested) */
    uint64_t heartbeat_frequency_ms;
-   /* The minimum time to sleep between hello checks. */
+   /* The minimum time to sleep between ismaster checks. */
    uint64_t min_heartbeat_frequency_ms;
    int64_t connect_timeout_ms;
    bool use_tls;
@@ -125,13 +125,15 @@ static BSON_GNUC_PRINTF (3, 4) void _server_monitor_log (
 #define MONITOR_LOG_WARNING(sm, ...) \
    _server_monitor_log (sm, MONGOC_LOG_LEVEL_DEBUG, __VA_ARGS__)
 
+/* Called only from server monitor thread.
+ * Caller must hold no locks (user's callback may lock topology mutex).
+ * Locks APM mutex.
+ */
 static void
 _server_monitor_heartbeat_started (mongoc_server_monitor_t *server_monitor,
                                    bool awaited)
 {
    mongoc_apm_server_heartbeat_started_t event;
-   MONGOC_DEBUG_ASSERT (
-      !COMMON_PREFIX (mutex_is_locked) (&server_monitor->topology->apm_mutex));
 
    if (!server_monitor->apm_callbacks.server_heartbeat_started) {
       return;
@@ -148,6 +150,10 @@ _server_monitor_heartbeat_started (mongoc_server_monitor_t *server_monitor,
    bson_mutex_unlock (&server_monitor->topology->apm_mutex);
 }
 
+/* Called only from server monitor thread.
+ * Caller must hold no locks (user's callback may lock topology mutex).
+ * Locks APM mutex.
+ */
 static void
 _server_monitor_heartbeat_succeeded (mongoc_server_monitor_t *server_monitor,
                                      const bson_t *reply,
@@ -173,6 +179,10 @@ _server_monitor_heartbeat_succeeded (mongoc_server_monitor_t *server_monitor,
    bson_mutex_unlock (&server_monitor->topology->apm_mutex);
 }
 
+/* Called only from server monitor thread.
+ * Caller must hold no locks (user's callback may lock topology mutex).
+ * Locks APM mutex.
+ */
 static void
 _server_monitor_heartbeat_failed (mongoc_server_monitor_t *server_monitor,
                                   const bson_error_t *error,
@@ -201,14 +211,16 @@ static void
 _server_monitor_append_cluster_time (mongoc_server_monitor_t *server_monitor,
                                      bson_t *cmd)
 {
-   mc_shared_tpld td =
-      mc_tpld_take_ref (BSON_ASSERT_PTR_INLINE (server_monitor)->topology);
+   mongoc_topology_t *topology;
 
+   topology = server_monitor->topology;
    /* Cluster time is updated on every reply. */
-   if (!bson_empty (&td.ptr->cluster_time)) {
-      bson_append_document (cmd, "$clusterTime", 12, &td.ptr->cluster_time);
+   bson_mutex_lock (&topology->mutex);
+   if (!bson_empty (&topology->description.cluster_time)) {
+      bson_append_document (
+         cmd, "$clusterTime", 12, &topology->description.cluster_time);
    }
-   mc_tpld_drop_ref (&td);
+   bson_mutex_unlock (&topology->mutex);
 }
 
 static bool
@@ -230,7 +242,7 @@ _server_monitor_send_and_recv_opquery (mongoc_server_monitor_t *server_monitor,
    rpc.header.request_id = server_monitor->request_id++;
    rpc.header.response_to = 0;
    rpc.header.opcode = MONGOC_OPCODE_QUERY;
-   rpc.query.flags = MONGOC_QUERY_SECONDARY_OK;
+   rpc.query.flags = MONGOC_QUERY_SLAVE_OK;
    rpc.query.collection = "admin.$cmd";
    rpc.query.skip = 0;
    rpc.query.n_return = -1;
@@ -305,30 +317,25 @@ fail:
 }
 
 static bool
-_server_monitor_polling_hello (mongoc_server_monitor_t *server_monitor,
-                               bool hello_ok,
-                               bson_t *hello_response,
-                               bson_error_t *error)
+_server_monitor_polling_ismaster (mongoc_server_monitor_t *server_monitor,
+                                  bson_t *ismaster_reply,
+                                  bson_error_t *error)
 {
    bson_t cmd;
-   const bson_t *hello;
    bool ret;
 
-   hello = _mongoc_topology_scanner_get_monitoring_cmd (
-      server_monitor->topology->scanner, hello_ok);
-   bson_copy_to (hello, &cmd);
-
+   bson_init (&cmd);
+   bson_append_int32 (&cmd, "isMaster", 8, 1);
    _server_monitor_append_cluster_time (server_monitor, &cmd);
    ret = _server_monitor_send_and_recv_opquery (
-      server_monitor, &cmd, hello_response, error);
+      server_monitor, &cmd, ismaster_reply, error);
    bson_destroy (&cmd);
    return ret;
 }
 
 static bool
-_server_monitor_awaitable_hello_send (mongoc_server_monitor_t *server_monitor,
-                                      bson_t *cmd,
-                                      bson_error_t *error)
+_server_monitor_awaitable_ismaster_send (
+   mongoc_server_monitor_t *server_monitor, bson_t *cmd, bson_error_t *error)
 {
    mongoc_rpc_t rpc = {0};
    mongoc_array_t array_to_write;
@@ -360,8 +367,9 @@ _server_monitor_awaitable_hello_send (mongoc_server_monitor_t *server_monitor,
                                     niovec,
                                     server_monitor->connect_timeout_ms,
                                     error)) {
-      MONITOR_LOG_ERROR (
-         server_monitor, "failed to write awaitable hello: %s", error->message);
+      MONITOR_LOG_ERROR (server_monitor,
+                         "failed to write awaitable ismaster: %s",
+                         error->message);
       _mongoc_array_destroy (&array_to_write);
       return false;
    }
@@ -383,8 +391,8 @@ _server_monitor_poll_with_interrupt (mongoc_server_monitor_t *server_monitor,
                                      bson_error_t *error)
 {
    /* How many milliseconds we should poll for on each tick.
-    * On every tick, check whether the awaitable hello was cancelled. */
-   const int32_t monitor_tick_ms = MONGOC_TOPOLOGY_MIN_HEARTBEAT_FREQUENCY_MS;
+    * On every tick, check whether the awaitable ismaster was cancelled. */
+   const int32_t monitor_tick_ms = 500;
    int64_t timeleft_ms;
 
    while ((timeleft_ms = expire_at_ms - _now_ms ()) > 0) {
@@ -402,7 +410,7 @@ _server_monitor_poll_with_interrupt (mongoc_server_monitor_t *server_monitor,
 
       MONITOR_LOG (
          server_monitor,
-         "polling for awaitable hello reply with timeleft_ms: %" PRId64,
+         "polling for awaitable ismaster reply with timeleft_ms: %" PRId64,
          timeleft_ms);
       ret = mongoc_stream_poll (
          poller, 1, (int32_t) BSON_MIN (timeleft_ms, monitor_tick_ms));
@@ -467,7 +475,7 @@ _get_timeout_ms (int64_t expire_at_ms, bson_error_t *error)
    return timeout_ms;
 }
 
-/* Receive an awaitable hello reply.
+/* Receive an awaitable ismaster reply.
  *
  * May be used to receive additional replies when moreToCome is set.
  * Called only from server monitor thread.
@@ -478,10 +486,11 @@ _get_timeout_ms (int64_t expire_at_ms, bson_error_t *error)
  * On cancellation, no error is set, but cancelled is set to true.
  */
 static bool
-_server_monitor_awaitable_hello_recv (mongoc_server_monitor_t *server_monitor,
-                                      bson_t *hello_response,
-                                      bool *cancelled,
-                                      bson_error_t *error)
+_server_monitor_awaitable_ismaster_recv (
+   mongoc_server_monitor_t *server_monitor,
+   bson_t *ismaster_reply,
+   bool *cancelled,
+   bson_error_t *error)
 {
    bool ret = false;
    mongoc_buffer_t buffer;
@@ -564,61 +573,56 @@ _server_monitor_awaitable_hello_recv (mongoc_server_monitor_t *server_monitor,
       GOTO (fail);
    }
 
-   bson_copy_to (&reply_local, hello_response);
+   bson_copy_to (&reply_local, ismaster_reply);
    server_monitor->more_to_come =
       (rpc.msg.flags & MONGOC_MSG_MORE_TO_COME) != 0;
 
    ret = true;
 fail:
    if (!ret) {
-      bson_init (hello_response);
+      bson_init (ismaster_reply);
    }
    _mongoc_buffer_destroy (&buffer);
    return ret;
 }
 
-/* Send and receive an awaitable hello.
+/* Send and receive an awaitable ismaster.
  *
  * Called only from server monitor thread.
  * May lock server monitor mutex in functions that are called.
  * May block for up to heartbeatFrequencyMS waiting for reply.
  */
 static bool
-_server_monitor_awaitable_hello (mongoc_server_monitor_t *server_monitor,
-                                 const mongoc_server_description_t *description,
-                                 bson_t *hello_response,
-                                 bool *cancelled,
-                                 bson_error_t *error)
+_server_monitor_awaitable_ismaster (mongoc_server_monitor_t *server_monitor,
+                                    const bson_t *topology_version,
+                                    bson_t *ismaster_reply,
+                                    bool *cancelled,
+                                    bson_error_t *error)
 {
-   bson_t cmd;
-   const bson_t *hello;
+   bson_t cmd = BSON_INITIALIZER;
    bool ret = false;
 
-   hello = _mongoc_topology_scanner_get_monitoring_cmd (
-      server_monitor->topology->scanner, description->hello_ok);
-   bson_copy_to (hello, &cmd);
-
+   bson_append_int32 (&cmd, "isMaster", 8, 1);
    _server_monitor_append_cluster_time (server_monitor, &cmd);
-   bson_append_document (
-      &cmd, "topologyVersion", 15, &description->topology_version);
+   bson_append_document (&cmd, "topologyVersion", 15, topology_version);
    bson_append_int32 (
       &cmd, "maxAwaitTimeMS", 14, server_monitor->heartbeat_frequency_ms);
    bson_append_utf8 (&cmd, "$db", 3, "admin", 5);
 
-   if (!_server_monitor_awaitable_hello_send (server_monitor, &cmd, error)) {
+   if (!_server_monitor_awaitable_ismaster_send (server_monitor, &cmd, error)) {
       GOTO (fail);
    }
 
-   if (!_server_monitor_awaitable_hello_recv (
-          server_monitor, hello_response, cancelled, error)) {
-      bson_destroy (hello_response);
+   if (!_server_monitor_awaitable_ismaster_recv (
+          server_monitor, ismaster_reply, cancelled, error)) {
+      bson_destroy (ismaster_reply);
       GOTO (fail);
    }
 
    ret = true;
 fail:
    if (!ret) {
-      bson_init (hello_response);
+      bson_init (ismaster_reply);
    }
    bson_destroy (&cmd);
    return ret;
@@ -628,45 +632,45 @@ fail:
  *
  * Called only from server monitor thread.
  * Caller must hold no locks.
- * Locks server monitor mutex.
+ * Locks topology mutex and server monitor mutex.
  */
 static void
-_update_topology_description (mongoc_server_monitor_t *server_monitor,
-                              mongoc_server_description_t *description)
+_server_monitor_update_topology_description (
+   mongoc_server_monitor_t *server_monitor,
+   mongoc_server_description_t *description)
 {
    mongoc_topology_t *topology;
-   bson_t *hello_response = NULL;
-   mc_tpld_modification tdmod;
+   bson_t *ismaster_reply = NULL;
 
    topology = server_monitor->topology;
-   if (description->has_hello_response) {
-      hello_response = &description->last_hello_response;
+   if (description->has_is_master) {
+      ismaster_reply = &description->last_is_master;
    }
 
-   if (hello_response) {
-      _mongoc_topology_update_cluster_time (topology, hello_response);
+   if (ismaster_reply) {
+      _mongoc_topology_update_cluster_time (topology, ismaster_reply);
    }
 
-   if (bson_atomic_int_fetch (&topology->scanner_state,
-                              bson_memory_order_relaxed) ==
-       MONGOC_TOPOLOGY_SCANNER_SHUTTING_DOWN) {
-      return;
-   }
+   bson_mutex_lock (&topology->mutex);
+   if (topology->scanner_state != MONGOC_TOPOLOGY_SCANNER_SHUTTING_DOWN) {
+      /* This is the another case of holding both locks. topology->mutex is
+       * always locked first, then server monitor mutex after. */
+      bson_mutex_lock (&server_monitor->shared.mutex);
+      server_monitor->shared.scan_requested = false;
+      bson_mutex_unlock (&server_monitor->shared.mutex);
 
-   tdmod = mc_tpld_modify_begin (topology);
-   bson_mutex_lock (&server_monitor->shared.mutex);
-   server_monitor->shared.scan_requested = false;
-   bson_mutex_unlock (&server_monitor->shared.mutex);
-   mongoc_topology_description_handle_hello (tdmod.new_td,
-                                             server_monitor->server_id,
-                                             hello_response,
-                                             description->round_trip_time_msec,
-                                             &description->error);
-   /* Reconcile server monitors. */
-   _mongoc_topology_background_monitoring_reconcile (topology, tdmod.new_td);
+      mongoc_topology_description_handle_ismaster (
+         &server_monitor->topology->description,
+         server_monitor->server_id,
+         ismaster_reply,
+         description->round_trip_time_msec,
+         &description->error);
+      /* Reconcile server monitors. */
+      _mongoc_topology_background_monitoring_reconcile (topology);
+   }
    /* Wake threads performing server selection. */
    mongoc_cond_broadcast (&server_monitor->topology->cond_client);
-   mc_tpld_modify_commit (tdmod);
+   bson_mutex_unlock (&server_monitor->topology->mutex);
 }
 
 /* Create a new server monitor.
@@ -676,16 +680,17 @@ _update_topology_description (mongoc_server_monitor_t *server_monitor,
  */
 mongoc_server_monitor_t *
 mongoc_server_monitor_new (mongoc_topology_t *topology,
-                           mongoc_topology_description_t *td,
                            mongoc_server_description_t *init_description)
 {
-   mongoc_server_monitor_t *server_monitor =
-      bson_malloc0 (sizeof (*server_monitor));
+   mongoc_server_monitor_t *server_monitor;
+
+   server_monitor = bson_malloc0 (sizeof (*server_monitor));
    server_monitor->description =
       mongoc_server_description_new_copy (init_description);
    server_monitor->server_id = init_description->id;
    server_monitor->topology = topology;
-   server_monitor->heartbeat_frequency_ms = td->heartbeat_msec;
+   server_monitor->heartbeat_frequency_ms =
+      topology->description.heartbeat_msec;
    server_monitor->min_heartbeat_frequency_ms =
       topology->min_heartbeat_frequency_msec;
    server_monitor->connect_timeout_ms = topology->connect_timeout_msec;
@@ -701,9 +706,9 @@ mongoc_server_monitor_new (mongoc_topology_t *topology,
    }
 #endif
    memcpy (&server_monitor->apm_callbacks,
-           &td->apm_callbacks,
+           &topology->description.apm_callbacks,
            sizeof (mongoc_apm_callbacks_t));
-   server_monitor->apm_context = td->apm_context;
+   server_monitor->apm_context = topology->description.apm_context;
    server_monitor->initiator = topology->scanner->initiator;
    server_monitor->initiator_context = topology->scanner->initiator_context;
    mongoc_cond_init (&server_monitor->shared.cond);
@@ -711,26 +716,27 @@ mongoc_server_monitor_new (mongoc_topology_t *topology,
    return server_monitor;
 }
 
-/* Creates a stream and performs the initial hello handshake.
+/* Creates a stream and performs the initial ismaster handshake.
  *
  * Called only by server monitor thread.
  * Returns true if both connection and handshake succeeds.
  * Returns false and sets error otherwise.
- * hello_response is always initialized.
+ * ismaster_reply is always initialized.
  */
 static bool
 _server_monitor_setup_connection (mongoc_server_monitor_t *server_monitor,
-                                  bson_t *hello_response,
+                                  bson_t *ismaster_reply,
                                   int64_t *start_us,
                                   bson_error_t *error)
 {
    bson_t cmd = BSON_INITIALIZER;
+   const bson_t *handshake;
    bool ret = false;
 
    ENTRY;
 
    BSON_ASSERT (!server_monitor->stream);
-   bson_init (hello_response);
+   bson_init (ismaster_reply);
 
    server_monitor->more_to_come = false;
 
@@ -764,12 +770,13 @@ _server_monitor_setup_connection (mongoc_server_monitor_t *server_monitor,
    /* Update the start time just before the handshake. */
    *start_us = _now_us ();
    /* Perform handshake. */
+   handshake = _mongoc_topology_get_ismaster (server_monitor->topology);
    bson_destroy (&cmd);
-   _mongoc_topology_dup_handshake_cmd (server_monitor->topology, &cmd);
+   bson_copy_to (handshake, &cmd);
    _server_monitor_append_cluster_time (server_monitor, &cmd);
-   bson_destroy (hello_response);
+   bson_destroy (ismaster_reply);
    if (!_server_monitor_send_and_recv_opquery (
-          server_monitor, &cmd, hello_response, error)) {
+          server_monitor, &cmd, ismaster_reply, error)) {
       GOTO (fail);
    }
 
@@ -779,36 +786,33 @@ fail:
    RETURN (ret);
 }
 
-/**
- * @brief Perform a hello check on a server
+/* Perform an ismaster check of a server.
  *
- * @param server_monitor The server monitor for this server.
- * @param previous_description The most recent view of the description of this
- * server.
- * @param cancelled Output parameter: Whether the monitor check is cancelled.
- * @return mongoc_server_description_t* The newly created updated server
- * description.
- *
- * @note May update the topology description associated with the server monitor.
- *
- * @note In case of error, returns a new server description with the error
- * information, but with no hello reply.
+ * Called only by server monitor thread.
+ * Caller must not hold any locks.
+ * May lock server monitor mutex. May lock topology mutex.
+ * Upon network error, the returned server description will contain the error,
+ * but no ismaster reply.
+ * Upon ismaster cancellation, cancelled will be true, and the returned server
+ * description will not contain an error or ismaster reply.
+ * Upon command error ("ok":0 reply), the returned server description have the
+ * ismaster reply and error set.
+ * Returns a new server description that the caller must destroy.
  */
-static mongoc_server_description_t *
-_server_monitor_check_server (
+mongoc_server_description_t *
+mongoc_server_monitor_check_server (
    mongoc_server_monitor_t *server_monitor,
    const mongoc_server_description_t *previous_description,
    bool *cancelled)
 {
    bool ret = false;
    bson_error_t error;
-   bson_t hello_response;
+   bson_t ismaster_reply;
    int64_t duration_us;
    int64_t start_us;
    bool command_or_network_error = false;
    bool awaited = false;
    mongoc_server_description_t *description;
-   mc_tpld_modification tdmod;
 
    ENTRY;
 
@@ -826,7 +830,7 @@ _server_monitor_check_server (
       awaited = false;
       _server_monitor_heartbeat_started (server_monitor, awaited);
       ret = _server_monitor_setup_connection (
-         server_monitor, &hello_response, &start_us, &error);
+         server_monitor, &ismaster_reply, &start_us, &error);
       GOTO (exit);
    }
 
@@ -835,28 +839,29 @@ _server_monitor_check_server (
       /* Publish a heartbeat started for each additional response read. */
       _server_monitor_heartbeat_started (server_monitor, awaited);
       MONITOR_LOG (server_monitor, "more to come");
-      ret = _server_monitor_awaitable_hello_recv (
-         server_monitor, &hello_response, cancelled, &error);
+      ret = _server_monitor_awaitable_ismaster_recv (
+         server_monitor, &ismaster_reply, cancelled, &error);
       GOTO (exit);
    }
 
    if (!bson_empty (&previous_description->topology_version)) {
       awaited = true;
       _server_monitor_heartbeat_started (server_monitor, awaited);
-      MONITOR_LOG (server_monitor, "awaitable hello");
-      ret = _server_monitor_awaitable_hello (server_monitor,
-                                             previous_description,
-                                             &hello_response,
-                                             cancelled,
-                                             &error);
+      MONITOR_LOG (server_monitor, "awaitable ismaster");
+      ret = _server_monitor_awaitable_ismaster (
+         server_monitor,
+         &previous_description->topology_version,
+         &ismaster_reply,
+         cancelled,
+         &error);
       GOTO (exit);
    }
 
-   MONITOR_LOG (server_monitor, "polling hello");
+   MONITOR_LOG (server_monitor, "polling ismaster");
    awaited = false;
    _server_monitor_heartbeat_started (server_monitor, awaited);
-   ret = _server_monitor_polling_hello (
-      server_monitor, previous_description->hello_ok, &hello_response, &error);
+   ret = _server_monitor_polling_ismaster (
+      server_monitor, &ismaster_reply, &error);
 
 exit:
    duration_us = _now_us () - start_us;
@@ -865,7 +870,7 @@ exit:
 
    /* If ret is true, we have a reply. Check if "ok": 1. */
    if (ret && _mongoc_cmd_check_ok (
-                 &hello_response, MONGOC_ERROR_API_VERSION_2, &error)) {
+                 &ismaster_reply, MONGOC_ERROR_API_VERSION_2, &error)) {
       int64_t rtt_ms = MONGOC_RTT_UNSET;
 
       /* rtt remains MONGOC_RTT_UNSET if awaited. */
@@ -873,9 +878,9 @@ exit:
          rtt_ms = duration_us / 1000;
       }
 
-      mongoc_server_description_handle_hello (
-         description, &hello_response, rtt_ms, NULL);
-      /* If the hello reply could not be parsed, consider this a command
+      mongoc_server_description_handle_ismaster (
+         description, &ismaster_reply, rtt_ms, NULL);
+      /* If the ismaster reply could not be parsed, consider this a command
        * error. */
       if (description->error.code) {
          MONITOR_LOG_ERROR (server_monitor,
@@ -886,7 +891,7 @@ exit:
             server_monitor, &description->error, duration_us, awaited);
       } else {
          _server_monitor_heartbeat_succeeded (
-            server_monitor, &hello_response, duration_us, awaited);
+            server_monitor, &ismaster_reply, duration_us, awaited);
       }
    } else if (*cancelled) {
       MONITOR_LOG (server_monitor, "server monitor cancelled");
@@ -898,12 +903,12 @@ exit:
       _server_monitor_heartbeat_failed (
          server_monitor, &description->error, duration_us, awaited);
    } else {
-      /* The hello reply had "ok":0 or a network error occurred. */
+      /* The ismaster reply had "ok":0 or a network error occurred. */
       MONITOR_LOG_ERROR (server_monitor,
                          "command or network error occurred: %s",
                          error.message);
       command_or_network_error = true;
-      mongoc_server_description_handle_hello (
+      mongoc_server_description_handle_ismaster (
          description, NULL, MONGOC_RTT_UNSET, &error);
       _server_monitor_heartbeat_failed (
          server_monitor, &description->error, duration_us, awaited);
@@ -915,22 +920,19 @@ exit:
       }
       server_monitor->stream = NULL;
       server_monitor->more_to_come = false;
-      tdmod = mc_tpld_modify_begin (server_monitor->topology);
-      /* clear_connection_pool() is a no-op if 'description->id' was already
-       * removed. */
-      _mongoc_topology_description_clear_connection_pool (
-         tdmod.new_td,
-         server_monitor->description->id,
-         &server_monitor->description->service_id);
-      mc_tpld_modify_commit (tdmod);
+      bson_mutex_lock (&server_monitor->topology->mutex);
+      _mongoc_topology_clear_connection_pool (server_monitor->topology,
+                                              server_monitor->description->id);
+      bson_mutex_unlock (&server_monitor->topology->mutex);
    }
 
-   bson_destroy (&hello_response);
+   bson_destroy (&ismaster_reply);
    return description;
 }
 
 /* Request scan of a single server.
  *
+ * Caller does not need to have topology mutex locked.
  * Locks server monitor mutex to deliver scan_requested.
  */
 void
@@ -943,7 +945,7 @@ mongoc_server_monitor_request_scan (mongoc_server_monitor_t *server_monitor)
    bson_mutex_unlock (&server_monitor->shared.mutex);
 }
 
-/* Request cancellation of an in progress awaitable hello.
+/* Request cancellation of an in progress awaitable ismaster.
  *
  * Called from app threads on network errors and during shutdown.
  * Locks server monitor mutex.
@@ -1031,7 +1033,7 @@ static BSON_THREAD_FUN (_server_monitor_thread, server_monitor_void)
       mongoc_server_description_destroy (previous_description);
       previous_description = mongoc_server_description_new_copy (description);
       mongoc_server_description_destroy (description);
-      description = _server_monitor_check_server (
+      description = mongoc_server_monitor_check_server (
          server_monitor, previous_description, &cancelled);
 
       if (cancelled) {
@@ -1039,7 +1041,7 @@ static BSON_THREAD_FUN (_server_monitor_thread, server_monitor_void)
          continue;
       }
 
-      _update_topology_description (server_monitor, description);
+      _server_monitor_update_topology_description (server_monitor, description);
 
       /* Immediately proceed to the next check if the previous response was
        * successful and included the topologyVersion field. */
@@ -1079,12 +1081,11 @@ static BSON_THREAD_FUN (_server_monitor_thread, server_monitor_void)
 
 static bool
 _server_monitor_ping_server (mongoc_server_monitor_t *server_monitor,
-                             bool hello_ok,
                              int64_t *rtt_ms)
 {
    bool ret = false;
    int64_t start_us = _now_us ();
-   bson_t hello_response;
+   bson_t ismaster_reply;
    bson_error_t error;
 
    *rtt_ms = MONGOC_RTT_UNSET;
@@ -1092,18 +1093,18 @@ _server_monitor_ping_server (mongoc_server_monitor_t *server_monitor,
    if (!server_monitor->stream) {
       MONITOR_LOG (server_monitor, "rtt setting up connection");
       ret = _server_monitor_setup_connection (
-         server_monitor, &hello_response, &start_us, &error);
-      bson_destroy (&hello_response);
+         server_monitor, &ismaster_reply, &start_us, &error);
+      bson_destroy (&ismaster_reply);
    }
 
    if (server_monitor->stream) {
-      MONITOR_LOG (server_monitor, "rtt polling hello");
-      ret = _server_monitor_polling_hello (
-         server_monitor, hello_ok, &hello_response, &error);
+      MONITOR_LOG (server_monitor, "rtt polling ismaster");
+      ret = _server_monitor_polling_ismaster (
+         server_monitor, &ismaster_reply, &error);
       if (ret) {
          *rtt_ms = (_now_us () - start_us) / 1000;
       }
-      bson_destroy (&hello_response);
+      bson_destroy (&ismaster_reply);
    }
    return ret;
 }
@@ -1114,12 +1115,13 @@ _server_monitor_ping_server (mongoc_server_monitor_t *server_monitor,
  */
 static BSON_THREAD_FUN (_server_monitor_rtt_thread, server_monitor_void)
 {
-   mongoc_server_monitor_t *server_monitor = server_monitor_void;
+   mongoc_server_monitor_t *server_monitor;
+
+   server_monitor = (mongoc_server_monitor_t *) server_monitor_void;
 
    while (true) {
       int64_t rtt_ms;
       bson_error_t error;
-      bool hello_ok;
 
       bson_mutex_lock (&server_monitor->shared.mutex);
       if (server_monitor->shared.state != MONGOC_THREAD_RUNNING) {
@@ -1128,31 +1130,21 @@ static BSON_THREAD_FUN (_server_monitor_rtt_thread, server_monitor_void)
       }
       bson_mutex_unlock (&server_monitor->shared.mutex);
 
-      {
-         mc_shared_tpld td = mc_tpld_take_ref (server_monitor->topology);
-         const mongoc_server_description_t *sd =
-            mongoc_topology_description_server_by_id_const (
-               td.ptr, server_monitor->description->id, &error);
-         hello_ok = sd ? sd->hello_ok : false;
-         mc_tpld_drop_ref (&td);
-      }
-
-      _server_monitor_ping_server (server_monitor, hello_ok, &rtt_ms);
+      _server_monitor_ping_server (server_monitor, &rtt_ms);
       if (rtt_ms != MONGOC_RTT_UNSET) {
-         mc_tpld_modification tdmod =
-            mc_tpld_modify_begin (server_monitor->topology);
-         mongoc_server_description_t *const mut_sd =
-            mongoc_topology_description_server_by_id (
-               tdmod.new_td, server_monitor->description->id, &error);
-         if (mut_sd) {
-            mongoc_server_description_update_rtt (mut_sd, rtt_ms);
-            mc_tpld_modify_commit (tdmod);
-         } else {
+         mongoc_server_description_t *sd;
+
+         bson_mutex_lock (&server_monitor->topology->mutex);
+         sd = mongoc_topology_description_server_by_id (
+            &server_monitor->topology->description,
+            server_monitor->description->id,
+            &error);
+         if (sd) {
             /* If the server description has been removed, the RTT thread will
-             * be terminated by background monitoring soon, so we have nothing
-             * to do but wait until we are about to be stopped. */
-            mc_tpld_modify_drop (tdmod);
+             * be terminated by background monitoring soon. */
+            mongoc_server_description_update_rtt (sd, rtt_ms);
          }
+         bson_mutex_unlock (&server_monitor->topology->mutex);
       }
       mongoc_server_monitor_wait (server_monitor);
    }
@@ -1194,6 +1186,7 @@ mongoc_server_monitor_run_as_rtt (mongoc_server_monitor_t *server_monitor)
  * Returns true if in state MONGOC_THREAD_OFF and the server monitor can be
  * safely destroyed.
  * Called during topology description reconcile.
+ * Caller may hold topology mutex.
  * Locks server monitor mutex.
  */
 bool
@@ -1214,7 +1207,7 @@ mongoc_server_monitor_request_shutdown (mongoc_server_monitor_t *server_monitor)
    }
    mongoc_cond_signal (&server_monitor->shared.cond);
    bson_mutex_unlock (&server_monitor->shared.mutex);
-   /* Cancel an in-progress hello check. */
+   /* Cancel an in-progress ismaster check. */
    if (!off) {
       mongoc_server_monitor_request_cancel (server_monitor);
    }
@@ -1224,6 +1217,8 @@ mongoc_server_monitor_request_shutdown (mongoc_server_monitor_t *server_monitor)
 /* Request thread shutdown and block until the server monitor thread terminates.
  *
  * Called by one thread.
+ * Caller must not hold topology mutex. Server monitor thread may need to lock
+ * it again in the process of shutting down.
  * Locks the server monitor mutex.
  */
 void
