@@ -123,13 +123,6 @@ count_dots (const char *s)
    return n;
 }
 
-static char *
-lowercase_str_new (const char *key)
-{
-   char *ret = bson_strdup (key);
-   mongoc_lowercase (key, ret);
-   return ret;
-}
 
 /* at least one character, and does not start with dot */
 static bool
@@ -141,10 +134,8 @@ valid_hostname (const char *s)
 }
 
 
-bool
-mongoc_uri_validate_srv_result (const mongoc_uri_t *uri,
-                                const char *host,
-                                bson_error_t *error)
+static bool
+validate_srv_result (mongoc_uri_t *uri, const char *host, bson_error_t *error)
 {
    const char *service;
    const char *service_root;
@@ -173,14 +164,14 @@ mongoc_uri_validate_srv_result (const mongoc_uri_t *uri,
    return true;
 }
 
-/* copy and upsert @host into @uri's host list. */
+/* upsert @host into @uri's host list. Side effect: modifies host->next when
+ * inserting. */
 static bool
 _upsert_into_host_list (mongoc_uri_t *uri,
                         mongoc_host_list_t *host,
                         bson_error_t *error)
 {
-   if (uri->is_srv &&
-       !mongoc_uri_validate_srv_result (uri, host->host, error)) {
+   if (uri->is_srv && !validate_srv_result (uri, host->host, error)) {
       return false;
    }
 
@@ -711,14 +702,6 @@ mongoc_uri_bson_append_or_replace_key (bson_t *options,
 }
 
 
-bool
-mongoc_uri_has_option (const mongoc_uri_t *uri, const char *key)
-{
-   bson_iter_t iter;
-
-   return bson_iter_init_find_case (&iter, &uri->options, key);
-}
-
 bool
 mongoc_uri_option_is_int32 (const char *key)
 {
@@ -748,7 +731,6 @@ bool
 mongoc_uri_option_is_bool (const char *key)
 {
    return !strcasecmp (key, MONGOC_URI_CANONICALIZEHOSTNAME) ||
-          !strcasecmp (key, MONGOC_URI_DIRECTCONNECTION) ||
           !strcasecmp (key, MONGOC_URI_JOURNAL) ||
           !strcasecmp (key, MONGOC_URI_RETRYREADS) ||
           !strcasecmp (key, MONGOC_URI_RETRYWRITES) ||
@@ -759,8 +741,6 @@ mongoc_uri_option_is_bool (const char *key)
           !strcasecmp (key, MONGOC_URI_TLSINSECURE) ||
           !strcasecmp (key, MONGOC_URI_TLSALLOWINVALIDCERTIFICATES) ||
           !strcasecmp (key, MONGOC_URI_TLSALLOWINVALIDHOSTNAMES) ||
-          !strcasecmp (key, MONGOC_URI_TLSDISABLECERTIFICATEREVOCATIONCHECK) ||
-          !strcasecmp (key, MONGOC_URI_TLSDISABLEOCSPENDPOINTCHECK) ||
           /* deprecated options */
           !strcasecmp (key, MONGOC_URI_SSL) ||
           !strcasecmp (key, MONGOC_URI_SSLALLOWINVALIDCERTIFICATES) ||
@@ -1058,30 +1038,22 @@ mongoc_uri_apply_options (mongoc_uri_t *uri,
        * also recognised as 32-bit ints.
        */
       if (mongoc_uri_option_is_int64 (key)) {
-         if (0 < strlen (value)) {
-            if (!_mongoc_uri_parse_int64 (key, value, &v_int64)) {
-               goto UNSUPPORTED_VALUE;
-            }
+         if (!_mongoc_uri_parse_int64 (key, value, &v_int64)) {
+            goto UNSUPPORTED_VALUE;
+         }
 
-            if (!_mongoc_uri_set_option_as_int64_with_error (
-                   uri, canon, v_int64, error)) {
-               return false;
-            }
-         } else {
-            MONGOC_WARNING ("Empty value provided for \"%s\"", key);
+         if (!_mongoc_uri_set_option_as_int64_with_error (
+                uri, canon, v_int64, error)) {
+            return false;
          }
       } else if (mongoc_uri_option_is_int32 (key)) {
-         if (0 < strlen (value)) {
-            if (!mongoc_uri_parse_int32 (key, value, &v_int)) {
-               goto UNSUPPORTED_VALUE;
-            }
+         if (!mongoc_uri_parse_int32 (key, value, &v_int)) {
+            goto UNSUPPORTED_VALUE;
+         }
 
-            if (!_mongoc_uri_set_option_as_int32_with_error (
-                   uri, canon, v_int, error)) {
-               return false;
-            }
-         } else {
-            MONGOC_WARNING ("Empty value provided for \"%s\"", key);
+         if (!_mongoc_uri_set_option_as_int32_with_error (
+                uri, canon, v_int, error)) {
+            return false;
          }
       } else if (!strcmp (key, MONGOC_URI_W)) {
          if (*value == '-' || isdigit (*value)) {
@@ -1096,46 +1068,36 @@ mongoc_uri_apply_options (mongoc_uri_t *uri,
          }
 
       } else if (mongoc_uri_option_is_bool (key)) {
-         if (0 < strlen (value)) {
-            if (0 == strcasecmp (value, "true")) {
-               bval = true;
-            } else if (0 == strcasecmp (value, "false")) {
-               bval = false;
-            } else if ((0 == strcmp (value, "1")) ||
-                       (0 == strcasecmp (value, "yes")) ||
-                       (0 == strcasecmp (value, "y")) ||
-                       (0 == strcasecmp (value, "t"))) {
-               MONGOC_WARNING ("Deprecated boolean value for \"%s\": \"%s\", "
-                               "please update to \"%s=true\"",
-                               key,
-                               value,
-                               key);
-               bval = true;
-            } else if ((0 == strcasecmp (value, "0")) ||
-                       (0 == strcasecmp (value, "-1")) ||
-                       (0 == strcmp (value, "no")) ||
-                       (0 == strcmp (value, "n")) ||
-                       (0 == strcmp (value, "f"))) {
-               MONGOC_WARNING ("Deprecated boolean value for \"%s\": \"%s\", "
-                               "please update to \"%s=false\"",
-                               key,
-                               value,
-                               key);
-               bval = false;
-            } else {
-               goto UNSUPPORTED_VALUE;
-            }
-
-            if (!mongoc_uri_set_option_as_bool (uri, canon, bval)) {
-               bson_set_error (error,
-                               MONGOC_ERROR_COMMAND,
-                               MONGOC_ERROR_COMMAND_INVALID_ARG,
-                               "Failed to set %s to %d",
-                               canon, bval);
-               return false;
-            }
+         if (0 == strcasecmp (value, "true")) {
+            bval = true;
+         } else if (0 == strcasecmp (value, "false")) {
+            bval = false;
+         } else if ((0 == strcmp (value, "1")) ||
+                    (0 == strcasecmp (value, "yes")) ||
+                    (0 == strcasecmp (value, "y")) ||
+                    (0 == strcasecmp (value, "t"))) {
+            MONGOC_WARNING ("Deprecated boolean value for \"%s\": \"%s\", "
+                            "please update to \"%s=true\"",
+                            key,
+                            value,
+                            key);
+            bval = true;
+         } else if ((0 == strcasecmp (value, "0")) ||
+                    (0 == strcasecmp (value, "-1")) ||
+                    (0 == strcmp (value, "no")) || (0 == strcmp (value, "n")) ||
+                    (0 == strcmp (value, "f"))) {
+            MONGOC_WARNING ("Deprecated boolean value for \"%s\": \"%s\", "
+                            "please update to \"%s=false\"",
+                            key,
+                            value,
+                            key);
+            bval = false;
          } else {
-            MONGOC_WARNING ("Empty value provided for \"%s\"", key);
+            goto UNSUPPORTED_VALUE;
+         }
+
+         if (!mongoc_uri_set_option_as_bool (uri, canon, bval)) {
+            return false;
          }
 
       } else if (!strcmp (key, MONGOC_URI_READPREFERENCETAGS)) {
@@ -1273,53 +1235,15 @@ mongoc_uri_finalize_tls (mongoc_uri_t *uri, bson_error_t *error)
       mongoc_uri_set_option_as_bool (uri, MONGOC_URI_TLS, true);
    }
 
-   /* tlsInsecure implies tlsAllowInvalidCertificates, tlsAllowInvalidHostnames,
-    * tlsDisableOCSPEndpointCheck, and tlsDisableCertificateRevocationCheck, so
-    * consider it an error to have both. The user might have the wrong idea. */
    if (bson_has_field (&uri->options, MONGOC_URI_TLSINSECURE) &&
        (bson_has_field (&uri->options,
                         MONGOC_URI_TLSALLOWINVALIDCERTIFICATES) ||
-        bson_has_field (&uri->options, MONGOC_URI_TLSALLOWINVALIDHOSTNAMES) ||
-        bson_has_field (&uri->options,
-                        MONGOC_URI_TLSDISABLEOCSPENDPOINTCHECK) ||
-        bson_has_field (&uri->options,
-                        MONGOC_URI_TLSDISABLECERTIFICATEREVOCATIONCHECK))) {
-      MONGOC_URI_ERROR (error,
-                        "%s may not be specified with %s, %s, %s, or %s",
-                        MONGOC_URI_TLSINSECURE,
-                        MONGOC_URI_TLSALLOWINVALIDCERTIFICATES,
-                        MONGOC_URI_TLSALLOWINVALIDHOSTNAMES,
-                        MONGOC_URI_TLSDISABLEOCSPENDPOINTCHECK,
-                        MONGOC_URI_TLSDISABLECERTIFICATEREVOCATIONCHECK);
-      return false;
-   }
-
-   /* tlsAllowInvalidCertificates implies tlsDisableOCSPEndpointCheck and
-    * tlsDisableCertificateRevocationCheck, so consider it an error to have
-    * both. The user might have the wrong idea. */
-   if (bson_has_field (&uri->options, MONGOC_URI_TLSALLOWINVALIDCERTIFICATES) &&
-       (bson_has_field (&uri->options,
-                        MONGOC_URI_TLSDISABLECERTIFICATEREVOCATIONCHECK) ||
-        bson_has_field (&uri->options,
-                        MONGOC_URI_TLSDISABLEOCSPENDPOINTCHECK))) {
+        bson_has_field (&uri->options, MONGOC_URI_TLSALLOWINVALIDHOSTNAMES))) {
       MONGOC_URI_ERROR (error,
                         "%s may not be specified with %s or %s",
+                        MONGOC_URI_TLSINSECURE,
                         MONGOC_URI_TLSALLOWINVALIDCERTIFICATES,
-                        MONGOC_URI_TLSDISABLEOCSPENDPOINTCHECK,
-                        MONGOC_URI_TLSDISABLECERTIFICATEREVOCATIONCHECK);
-      return false;
-   }
-
-   /*  tlsDisableCertificateRevocationCheck implies tlsDisableOCSPEndpointCheck,
-    * so consider it an error to have both. The user might have the wrong idea.
-    */
-   if (bson_has_field (&uri->options,
-                       MONGOC_URI_TLSDISABLECERTIFICATEREVOCATIONCHECK) &&
-       bson_has_field (&uri->options, MONGOC_URI_TLSDISABLEOCSPENDPOINTCHECK)) {
-      MONGOC_URI_ERROR (error,
-                        "%s may not be specified with %s",
-                        MONGOC_URI_TLSDISABLECERTIFICATEREVOCATIONCHECK,
-                        MONGOC_URI_TLSDISABLEOCSPENDPOINTCHECK);
+                        MONGOC_URI_TLSALLOWINVALIDHOSTNAMES);
       return false;
    }
 
@@ -1358,12 +1282,9 @@ mongoc_uri_finalize_auth (mongoc_uri_t *uri,
                &uri->credentials, MONGOC_URI_AUTHSOURCE, -1, "$external", -1);
          }
       }
-      /* MONGODB-X509 and MONGODB-AWS are the only mechanisms that don't require
-       * username */
-      if (!(strcasecmp (mongoc_uri_get_auth_mechanism (uri), "MONGODB-X509") ==
-               0 ||
-            strcasecmp (mongoc_uri_get_auth_mechanism (uri), "MONGODB-AWS") ==
-               0)) {
+      /* MONGODB-X509 is the only mechanism that doesn't require username */
+      if (strcasecmp (mongoc_uri_get_auth_mechanism (uri), "MONGODB-X509") !=
+          0) {
          if (!mongoc_uri_get_username (uri) ||
              strcmp (mongoc_uri_get_username (uri), "") == 0) {
             MONGOC_URI_ERROR (error,
@@ -1414,40 +1335,6 @@ mongoc_uri_finalize_auth (mongoc_uri_t *uri,
    return true;
 }
 
-static bool
-mongoc_uri_finalize_directconnection (mongoc_uri_t *uri, bson_error_t *error)
-{
-   bool directconnection = false;
-
-   directconnection =
-      mongoc_uri_get_option_as_bool (uri, MONGOC_URI_DIRECTCONNECTION, false);
-   if (!directconnection) {
-      return true;
-   }
-
-   /* URI options spec: "The driver MUST report an error if the
-    * directConnection=true URI option is specified with an SRV URI, because
-    * the URI may resolve to multiple hosts. The driver MUST allow specifying
-    * directConnection=false URI option with an SRV URI." */
-   if (uri->is_srv) {
-      MONGOC_URI_ERROR (
-         error, "%s", "SRV URI not allowed with directConnection option");
-      return false;
-   }
-
-   /* URI options spec: "The driver MUST report an error if the
-    * directConnection=true URI option is specified with multiple seeds." */
-   if (uri->hosts && uri->hosts->next) {
-      MONGOC_URI_ERROR (
-         error,
-         "%s",
-         "Multiple seeds not allowed with directConnection option");
-      return false;
-   }
-
-   return true;
-}
-
 static bool
 mongoc_uri_parse_before_slash (mongoc_uri_t *uri,
                                const char *before_slash,
@@ -1560,10 +1447,6 @@ mongoc_uri_parse (mongoc_uri_t *uri, const char *str, bson_error_t *error)
       goto error;
    }
 
-   if (!mongoc_uri_finalize_directconnection (uri, error)) {
-      goto error;
-   }
-
    bson_free (before_slash);
    return true;
 
@@ -2314,8 +2197,7 @@ mongoc_uri_unescape (const char *escaped_string)
 #else
              (1 != sscanf (&ptr[1], "%02x", &hex))
 #endif
-             ||
-             0 == hex) {
+          || 0 == hex) {
             bson_string_free (str, true);
             MONGOC_WARNING ("Invalid %% escape sequence");
             return NULL;
@@ -2332,8 +2214,7 @@ mongoc_uri_unescape (const char *escaped_string)
 
    /* Check that after unescaping, it is still valid UTF-8 */
    if (unescape_occurred && !bson_utf8_validate (str->str, str->len, false)) {
-      MONGOC_WARNING (
-         "Invalid %% escape sequence: unescaped string contains invalid UTF-8");
+      MONGOC_WARNING ("Invalid %% escape sequence: unescaped string contains invalid UTF-8");
       bson_string_free (str, true);
       return NULL;
    }
@@ -2426,13 +2307,7 @@ mongoc_uri_get_tls (const mongoc_uri_t *uri) /* IN */
        bson_iter_init_find_case (
           &iter, &uri->options, MONGOC_URI_TLSINSECURE) ||
        bson_iter_init_find_case (
-          &iter, &uri->options, MONGOC_URI_TLSCERTIFICATEKEYFILEPASSWORD) ||
-       bson_iter_init_find_case (
-          &iter, &uri->options, MONGOC_URI_TLSDISABLEOCSPENDPOINTCHECK) ||
-       bson_iter_init_find_case (
-          &iter,
-          &uri->options,
-          MONGOC_URI_TLSDISABLECERTIFICATEREVOCATIONCHECK)) {
+          &iter, &uri->options, MONGOC_URI_TLSCERTIFICATEKEYFILEPASSWORD)) {
       return true;
    }
 
@@ -2585,7 +2460,6 @@ _mongoc_uri_set_option_as_int32_with_error (mongoc_uri_t *uri,
    const char *option;
    const bson_t *options;
    bson_iter_t iter;
-   char *option_lowercase = NULL;
 
    option = mongoc_uri_canonicalize_option (option_orig);
    /* Server Discovery and Monitoring Spec: "the driver MUST NOT permit users
@@ -2624,16 +2498,14 @@ _mongoc_uri_set_option_as_int32_with_error (mongoc_uri_t *uri,
          return false;
       }
    }
-   option_lowercase = lowercase_str_new (option);
-   if (!bson_append_int32 (&uri->options, option_lowercase, -1, value)) {
-      bson_free (option_lowercase);
+
+   if (!bson_append_int32 (&uri->options, option, -1, value)) {
       MONGOC_URI_ERROR (
          error, "Failed to set URI option \"%s\" to %d", option_orig, value);
 
       return false;
    }
 
-   bson_free (option_lowercase);
    return true;
 }
 
@@ -2660,7 +2532,6 @@ _mongoc_uri_set_option_as_int32 (mongoc_uri_t *uri,
    const char *option;
    const bson_t *options;
    bson_iter_t iter;
-   char *option_lowercase = NULL;
 
    option = mongoc_uri_canonicalize_option (option_orig);
    if ((options = mongoc_uri_get_options (uri)) &&
@@ -2673,9 +2544,7 @@ _mongoc_uri_set_option_as_int32 (mongoc_uri_t *uri,
       }
    }
 
-   option_lowercase = lowercase_str_new (option);
-   bson_append_int32 (&uri->options, option_lowercase, -1, value);
-   bson_free (option_lowercase);
+   bson_append_int32 (&uri->options, option, -1, value);
    return true;
 }
 
@@ -2813,7 +2682,6 @@ _mongoc_uri_set_option_as_int64_with_error (mongoc_uri_t *uri,
    const char *option;
    const bson_t *options;
    bson_iter_t iter;
-   char *option_lowercase = NULL;
 
    option = mongoc_uri_canonicalize_option (option_orig);
 
@@ -2833,9 +2701,7 @@ _mongoc_uri_set_option_as_int64_with_error (mongoc_uri_t *uri,
       }
    }
 
-   option_lowercase = lowercase_str_new (option);
-   if (!bson_append_int64 (&uri->options, option_lowercase, -1, value)) {
-      bson_free (option_lowercase);
+   if (!bson_append_int64 (&uri->options, option, -1, value)) {
       MONGOC_URI_ERROR (error,
                         "Failed to set URI option \"%s\" to %" PRId64,
                         option_orig,
@@ -2843,7 +2709,7 @@ _mongoc_uri_set_option_as_int64_with_error (mongoc_uri_t *uri,
 
       return false;
    }
-   bson_free (option_lowercase);
+
    return true;
 }
 
@@ -2914,7 +2780,6 @@ mongoc_uri_set_option_as_bool (mongoc_uri_t *uri,
                                bool value)
 {
    const char *option;
-   char *option_lowercase;
    const bson_t *options;
    bson_iter_t iter;
 
@@ -2934,9 +2799,8 @@ mongoc_uri_set_option_as_bool (mongoc_uri_t *uri,
          return false;
       }
    }
-   option_lowercase = lowercase_str_new (option);
-   bson_append_bool (&uri->options, option_lowercase, -1, value);
-   bson_free (option_lowercase);
+
+   bson_append_bool (&uri->options, option, -1, value);
    return true;
 }
 
@@ -3012,7 +2876,6 @@ mongoc_uri_set_option_as_utf8 (mongoc_uri_t *uri,
 {
    const char *option;
    size_t len;
-   char *option_lowercase = NULL;
 
    option = mongoc_uri_canonicalize_option (option_orig);
    BSON_ASSERT (option);
@@ -3029,10 +2892,7 @@ mongoc_uri_set_option_as_utf8 (mongoc_uri_t *uri,
    if (!bson_strcasecmp (option, MONGOC_URI_APPNAME)) {
       return mongoc_uri_set_appname (uri, value);
    } else {
-      option_lowercase = lowercase_str_new (option);
-      mongoc_uri_bson_append_or_replace_key (
-         &uri->options, option_lowercase, value);
-      bson_free (option_lowercase);
+      mongoc_uri_bson_append_or_replace_key (&uri->options, option, value);
    }
 
    return true;
@@ -3072,39 +2932,3 @@ _mongoc_uri_copy_and_replace_host_list (const mongoc_uri_t *original,
    _mongoc_host_list_from_string (uri->hosts, host);
    return uri;
 }
-
-bool
-mongoc_uri_init_with_srv_host_list (mongoc_uri_t *uri,
-                                    mongoc_host_list_t *host_list,
-                                    bson_error_t *error)
-{
-   mongoc_host_list_t *host;
-
-   BSON_ASSERT (uri->is_srv);
-   BSON_ASSERT (!uri->hosts);
-
-   LL_FOREACH (host_list, host)
-   {
-      if (!mongoc_uri_upsert_host_and_port (uri, host->host_and_port, error)) {
-         return false;
-      }
-   }
-
-   return true;
-}
-
-#ifdef MONGOC_ENABLE_CRYPTO
-void
-_mongoc_uri_init_scram (const mongoc_uri_t *uri,
-                        mongoc_scram_t *scram,
-                        mongoc_crypto_hash_algorithm_t algo)
-{
-   BSON_ASSERT (uri);
-   BSON_ASSERT (scram);
-
-   _mongoc_scram_init (scram, algo);
-
-   _mongoc_scram_set_pass (scram, mongoc_uri_get_password (uri));
-   _mongoc_scram_set_user (scram, mongoc_uri_get_username (uri));
-}
-#endif
