@@ -19,11 +19,6 @@
 #include "mongocrypt-ctx-private.h"
 #include "mongocrypt-key-broker-private.h"
 
-#define ALGORITHM_DETERMINISTIC "AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic"
-#define ALGORITHM_DETERMINISTIC_LEN 43
-#define ALGORITHM_RANDOM "AEAD_AES_256_CBC_HMAC_SHA_512-Random"
-#define ALGORITHM_RANDOM_LEN 36
-
 bool
 _mongocrypt_ctx_fail_w_msg (mongocrypt_ctx_t *ctx, const char *msg)
 {
@@ -172,13 +167,85 @@ mongocrypt_ctx_setopt_key_alt_name (mongocrypt_ctx_t *ctx,
 }
 
 
+bool
+mongocrypt_ctx_setopt_key_material (mongocrypt_ctx_t *ctx,
+                                    mongocrypt_binary_t *key_material)
+{
+   bson_t as_bson;
+   bson_iter_t iter;
+   const char *key;
+   _mongocrypt_buffer_t buffer;
+
+   if (!ctx) {
+      return false;
+   }
+
+   if (ctx->initialized) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "cannot set options after init");
+   }
+
+   if (ctx->opts.key_material.owned) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "keyMaterial already set");
+   }
+
+   if (ctx->state == MONGOCRYPT_CTX_ERROR) {
+      return false;
+   }
+
+   if (!key_material || !key_material->data) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "option must be non-NULL");
+   }
+
+   if (!_mongocrypt_binary_to_bson (key_material, &as_bson)) {
+      return _mongocrypt_ctx_fail_w_msg (ctx,
+                                         "invalid keyMaterial bson object");
+   }
+
+   /* TODO: use _mongocrypt_parse_required_binary once MONGOCRYPT-380 is
+    * resolved.*/
+   if (!bson_iter_init (&iter, &as_bson) || !bson_iter_next (&iter)) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid bson");
+   }
+
+   key = bson_iter_key (&iter);
+   BSON_ASSERT (key);
+   if (0 != strcmp (key, "keyMaterial")) {
+      return _mongocrypt_ctx_fail_w_msg (
+         ctx, "keyMaterial must have field 'keyMaterial'");
+   }
+
+   if (!_mongocrypt_buffer_from_binary_iter (&buffer, &iter)) {
+      return _mongocrypt_ctx_fail_w_msg (ctx,
+                                         "keyMaterial must be binary data");
+   }
+
+   if (buffer.len != MONGOCRYPT_KEY_LEN) {
+      _mongocrypt_set_error (
+         ctx->status,
+         MONGOCRYPT_STATUS_ERROR_CLIENT,
+         MONGOCRYPT_GENERIC_ERROR_CODE,
+         "keyMaterial should have length %d, but has length %" PRIu32,
+         MONGOCRYPT_KEY_LEN,
+         buffer.len);
+      return _mongocrypt_ctx_fail (ctx);
+   }
+
+   _mongocrypt_buffer_steal (&ctx->opts.key_material, &buffer);
+
+   if (bson_iter_next (&iter)) {
+      return _mongocrypt_ctx_fail_w_msg (
+         ctx, "unrecognized field, only keyMaterial expected");
+   }
+
+   return true;
+}
+
+
 bool
 mongocrypt_ctx_setopt_algorithm (mongocrypt_ctx_t *ctx,
                                  const char *algorithm,
                                  int len)
 {
-   size_t calculated_len;
-
    if (!ctx) {
       return false;
    }
@@ -191,7 +258,8 @@ mongocrypt_ctx_setopt_algorithm (mongocrypt_ctx_t *ctx,
       return false;
    }
 
-   if (ctx->opts.algorithm != MONGOCRYPT_ENCRYPTION_ALGORITHM_NONE) {
+   if (ctx->opts.algorithm != MONGOCRYPT_ENCRYPTION_ALGORITHM_NONE ||
+       ctx->opts.index_type.set) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "already set algorithm");
    }
 
@@ -203,7 +271,7 @@ mongocrypt_ctx_setopt_algorithm (mongocrypt_ctx_t *ctx,
       return _mongocrypt_ctx_fail_w_msg (ctx, "passed null algorithm");
    }
 
-   calculated_len = len == -1 ? strlen (algorithm) : (size_t) len;
+   const size_t calculated_len = len == -1 ? strlen (algorithm) : (size_t) len;
    if (ctx->crypt->log.trace_enabled) {
       _mongocrypt_log (&ctx->crypt->log,
                        MONGOCRYPT_LOG_LEVEL_TRACE,
@@ -214,21 +282,29 @@ mongocrypt_ctx_setopt_algorithm (mongocrypt_ctx_t *ctx,
                        algorithm);
    }
 
-   if (calculated_len == ALGORITHM_DETERMINISTIC_LEN &&
-       strncmp (algorithm,
-                ALGORITHM_DETERMINISTIC,
-                ALGORITHM_DETERMINISTIC_LEN) == 0) {
+   mstr_view algo_str = mstrv_view_data (algorithm, calculated_len);
+   if (mstr_eq (algo_str, mstrv_lit (MONGOCRYPT_ALGORITHM_DETERMINISTIC_STR))) {
       ctx->opts.algorithm = MONGOCRYPT_ENCRYPTION_ALGORITHM_DETERMINISTIC;
-      return true;
-   }
-
-   if (calculated_len == ALGORITHM_RANDOM_LEN &&
-       strncmp (algorithm, ALGORITHM_RANDOM, ALGORITHM_RANDOM_LEN) == 0) {
+   } else if (mstr_eq (algo_str, mstrv_lit (MONGOCRYPT_ALGORITHM_RANDOM_STR))) {
       ctx->opts.algorithm = MONGOCRYPT_ENCRYPTION_ALGORITHM_RANDOM;
-      return true;
+   } else if (mstr_eq (algo_str,
+                       mstrv_lit (MONGOCRYPT_ALGORITHM_INDEXED_STR))) {
+      ctx->opts.index_type.value = MONGOCRYPT_INDEX_TYPE_EQUALITY;
+      ctx->opts.index_type.set = true;
+   } else if (mstr_eq (algo_str,
+                       mstrv_lit (MONGOCRYPT_ALGORITHM_UNINDEXED_STR))) {
+      ctx->opts.index_type.value = MONGOCRYPT_INDEX_TYPE_NONE;
+      ctx->opts.index_type.set = true;
+   } else {
+      char *error = bson_strdup_printf ("unsupported algorithm string \"%.*s\"",
+                                        (int) algo_str.len,
+                                        algo_str.data);
+      _mongocrypt_ctx_fail_w_msg (ctx, error);
+      bson_free (error);
+      return false;
    }
 
-   return _mongocrypt_ctx_fail_w_msg (ctx, "unsupported algorithm");
+   return true;
 }
 
 
@@ -236,7 +312,7 @@ mongocrypt_ctx_t *
 mongocrypt_ctx_new (mongocrypt_t *crypt)
 {
    mongocrypt_ctx_t *ctx;
-   int ctx_size;
+   size_t ctx_size;
 
    if (!crypt) {
       return NULL;
@@ -292,7 +368,8 @@ _mongo_feed_keys (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *in)
    _mongocrypt_buffer_t buf;
 
    _mongocrypt_buffer_from_binary (&buf, in);
-   if (!_mongocrypt_key_broker_add_doc (&ctx->kb, &buf)) {
+   if (!_mongocrypt_key_broker_add_doc (
+          &ctx->kb, _mongocrypt_ctx_kms_providers (ctx), &buf)) {
       BSON_ASSERT (!_mongocrypt_key_broker_status (&ctx->kb, ctx->status));
       return _mongocrypt_ctx_fail (ctx);
    }
@@ -317,12 +394,13 @@ _next_kms_ctx (mongocrypt_ctx_t *ctx)
 static bool
 _kms_done (mongocrypt_ctx_t *ctx)
 {
-   if (!_mongocrypt_key_broker_kms_done (&ctx->kb)) {
+   _mongocrypt_opts_kms_providers_t *kms_providers =
+      _mongocrypt_ctx_kms_providers (ctx);
+   if (!_mongocrypt_key_broker_kms_done (&ctx->kb, kms_providers)) {
       BSON_ASSERT (!_mongocrypt_key_broker_status (&ctx->kb, ctx->status));
       return _mongocrypt_ctx_fail (ctx);
    }
-   ctx->state = MONGOCRYPT_CTX_READY;
-   return true;
+   return _mongocrypt_ctx_state_from_key_broker (ctx);
 }
 
 
@@ -349,6 +427,10 @@ mongocrypt_ctx_mongo_op (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *out)
       CHECK_AND_CALL (mongo_op_keys, ctx, out);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_KMS:
+   case MONGOCRYPT_CTX_READY:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -391,6 +473,10 @@ mongocrypt_ctx_mongo_feed (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *in)
       CHECK_AND_CALL (mongo_feed_keys, ctx, in);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_KMS:
+   case MONGOCRYPT_CTX_READY:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -416,6 +502,10 @@ mongocrypt_ctx_mongo_done (mongocrypt_ctx_t *ctx)
       CHECK_AND_CALL (mongo_done_keys, ctx);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_KMS:
+   case MONGOCRYPT_CTX_READY:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -458,6 +548,12 @@ mongocrypt_ctx_next_kms_ctx (mongocrypt_ctx_t *ctx)
       return ctx->vtable.next_kms_ctx (ctx);
    case MONGOCRYPT_CTX_ERROR:
       return NULL;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_MONGO_COLLINFO:
+   case MONGOCRYPT_CTX_NEED_MONGO_KEYS:
+   case MONGOCRYPT_CTX_NEED_MONGO_MARKINGS:
+   case MONGOCRYPT_CTX_READY:
    default:
       _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
       return NULL;
@@ -465,6 +561,55 @@ mongocrypt_ctx_next_kms_ctx (mongocrypt_ctx_t *ctx)
 }
 
 
+bool
+mongocrypt_ctx_provide_kms_providers (
+   mongocrypt_ctx_t *ctx, mongocrypt_binary_t *kms_providers_definition)
+{
+   if (!ctx) {
+      return false;
+   }
+
+   if (!ctx->initialized) {
+      _mongocrypt_ctx_fail_w_msg (ctx, "ctx NULL or uninitialized");
+      return false;
+   }
+
+   if (ctx->state != MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS) {
+      _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
+      return false;
+   }
+
+   if (!_mongocrypt_parse_kms_providers (kms_providers_definition,
+                                         &ctx->per_ctx_kms_providers,
+                                         ctx->status,
+                                         &ctx->crypt->log)) {
+      return false;
+   }
+
+   if (!_mongocrypt_opts_kms_providers_validate (&ctx->per_ctx_kms_providers,
+                                                 ctx->status)) {
+      /* Remove the parsed KMS providers if they are invalid */
+      _mongocrypt_opts_kms_providers_cleanup (&ctx->per_ctx_kms_providers);
+      memset (
+         &ctx->per_ctx_kms_providers, 0, sizeof (ctx->per_ctx_kms_providers));
+      return false;
+   }
+
+   memcpy (&ctx->kms_providers,
+           &ctx->crypt->opts.kms_providers,
+           sizeof (_mongocrypt_opts_kms_providers_t));
+   _mongocrypt_opts_merge_kms_providers (&ctx->kms_providers,
+                                         &ctx->per_ctx_kms_providers);
+
+   ctx->state = ctx->kb.state == KB_ADDING_DOCS ? MONGOCRYPT_CTX_NEED_MONGO_KEYS
+                                                : MONGOCRYPT_CTX_NEED_KMS;
+   if (ctx->vtable.after_kms_credentials_provided) {
+      return ctx->vtable.after_kms_credentials_provided (ctx);
+   }
+   return true;
+}
+
+
 bool
 mongocrypt_ctx_kms_done (mongocrypt_ctx_t *ctx)
 {
@@ -484,6 +629,12 @@ mongocrypt_ctx_kms_done (mongocrypt_ctx_t *ctx)
       return ctx->vtable.kms_done (ctx);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_MONGO_COLLINFO:
+   case MONGOCRYPT_CTX_NEED_MONGO_KEYS:
+   case MONGOCRYPT_CTX_NEED_MONGO_MARKINGS:
+   case MONGOCRYPT_CTX_READY:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -513,6 +664,12 @@ mongocrypt_ctx_finalize (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *out)
       return ctx->vtable.finalize (ctx, out);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_KMS:
+   case MONGOCRYPT_CTX_NEED_MONGO_COLLINFO:
+   case MONGOCRYPT_CTX_NEED_MONGO_KEYS:
+   case MONGOCRYPT_CTX_NEED_MONGO_MARKINGS:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -545,13 +702,14 @@ mongocrypt_ctx_destroy (mongocrypt_ctx_t *ctx)
       ctx->vtable.cleanup (ctx);
    }
 
-   bson_free (ctx->opts.masterkey_aws_region);
-   bson_free (ctx->opts.masterkey_aws_cmk);
-   bson_free (ctx->opts.masterkey_aws_endpoint);
+   _mongocrypt_opts_kms_providers_cleanup (&ctx->per_ctx_kms_providers);
+   _mongocrypt_kek_cleanup (&ctx->opts.kek);
    mongocrypt_status_destroy (ctx->status);
    _mongocrypt_key_broker_cleanup (&ctx->kb);
+   _mongocrypt_buffer_cleanup (&ctx->opts.key_material);
    _mongocrypt_key_alt_name_destroy_all (ctx->opts.key_alt_names);
    _mongocrypt_buffer_cleanup (&ctx->opts.key_id);
+   _mongocrypt_buffer_cleanup (&ctx->opts.index_key_id);
    bson_free (ctx);
    return;
 }
@@ -564,6 +722,11 @@ mongocrypt_ctx_setopt_masterkey_aws (mongocrypt_ctx_t *ctx,
                                      const char *cmk,
                                      int32_t cmk_len)
 {
+   mongocrypt_binary_t *bin;
+   bson_t as_bson;
+   bool ret;
+   char *temp = NULL;
+
    if (!ctx) {
       return false;
    }
@@ -575,21 +738,44 @@ mongocrypt_ctx_setopt_masterkey_aws (mongocrypt_ctx_t *ctx,
       return false;
    }
 
-   if (ctx->opts.masterkey_kms_provider) {
+   if (ctx->opts.kek.kms_provider != MONGOCRYPT_KMS_PROVIDER_AWS &&
+       ctx->opts.kek.kms_provider != MONGOCRYPT_KMS_PROVIDER_NONE) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "master key already set");
    }
 
-   if (!_mongocrypt_validate_and_copy_string (
-          region, region_len, &ctx->opts.masterkey_aws_region) ||
-       region_len == 0) {
-      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid region");
+   if (ctx->opts.kek.kms_provider == MONGOCRYPT_KMS_PROVIDER_AWS &&
+       ctx->opts.kek.provider.aws.region) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "master key already set");
    }
 
-   if (!_mongocrypt_validate_and_copy_string (
-          cmk, cmk_len, &ctx->opts.masterkey_aws_cmk) ||
-       cmk_len == 0) {
-      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid cmk passed");
+   if (!_mongocrypt_validate_and_copy_string (region, region_len, &temp) ||
+       region_len == 0) {
+      bson_free (temp);
+      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid region");
    }
+   bson_free (temp);
+
+   temp = NULL;
+   if (!_mongocrypt_validate_and_copy_string (cmk, cmk_len, &temp) ||
+       cmk_len == 0) {
+      bson_free (temp);
+      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid cmk");
+   }
+   bson_free (temp);
+
+   bson_init (&as_bson);
+   bson_append_utf8 (&as_bson,
+                     MONGOCRYPT_STR_AND_LEN ("provider"),
+                     MONGOCRYPT_STR_AND_LEN ("aws"));
+   bson_append_utf8 (
+      &as_bson, MONGOCRYPT_STR_AND_LEN ("region"), region, region_len);
+   bson_append_utf8 (&as_bson, MONGOCRYPT_STR_AND_LEN ("key"), cmk, cmk_len);
+   bin = mongocrypt_binary_new_from_data ((uint8_t *) bson_get_data (&as_bson),
+                                          as_bson.len);
+
+   ret = mongocrypt_ctx_setopt_key_encryption_key (ctx, bin);
+   mongocrypt_binary_destroy (bin);
+   bson_destroy (&as_bson);
 
    if (ctx->crypt->log.trace_enabled) {
       _mongocrypt_log (&ctx->crypt->log,
@@ -597,19 +783,16 @@ mongocrypt_ctx_setopt_masterkey_aws (mongocrypt_ctx_t *ctx,
                        "%s (%s=\"%s\", %s=%d, %s=\"%s\", %s=%d)",
                        BSON_FUNC,
                        "region",
-                       ctx->opts.masterkey_aws_region,
+                       ctx->opts.kek.provider.aws.region,
                        "region_len",
                        region_len,
                        "cmk",
-                       ctx->opts.masterkey_aws_cmk,
+                       ctx->opts.kek.provider.aws.cmk,
                        "cmk_len",
                        cmk_len);
    }
 
-   ctx->opts.masterkey_kms_provider = MONGOCRYPT_KMS_PROVIDER_AWS;
-   ctx->opts.masterkey_aws_region_len = region_len;
-   ctx->opts.masterkey_aws_cmk_len = cmk_len;
-   return true;
+   return ret;
 }
 
 
@@ -627,11 +810,11 @@ mongocrypt_ctx_setopt_masterkey_local (mongocrypt_ctx_t *ctx)
       return false;
    }
 
-   if (ctx->opts.masterkey_kms_provider) {
+   if (ctx->opts.kek.kms_provider) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "master key already set");
    }
 
-   ctx->opts.masterkey_kms_provider = MONGOCRYPT_KMS_PROVIDER_LOCAL;
+   ctx->opts.kek.kms_provider = MONGOCRYPT_KMS_PROVIDER_LOCAL;
    return true;
 }
 
@@ -642,6 +825,12 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
 {
    bool has_id = false, has_alt_name = false, has_multiple_alt_names = false;
 
+   // This condition is guarded in setopt_algorithm:
+   BSON_ASSERT (
+      !(ctx->opts.index_type.set &&
+        ctx->opts.algorithm != MONGOCRYPT_ENCRYPTION_ALGORITHM_NONE) &&
+      "Both an encryption algorithm and an index_type were set.");
+
    if (ctx->initialized) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "cannot double initialize");
    }
@@ -661,19 +850,19 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
     * not.
     */
 
-   if (opts_spec->masterkey == OPT_REQUIRED) {
-      if (!ctx->opts.masterkey_kms_provider) {
+   if (opts_spec->kek == OPT_REQUIRED) {
+      if (!ctx->opts.kek.kms_provider) {
          return _mongocrypt_ctx_fail_w_msg (ctx, "master key required");
       }
-      if (!(ctx->opts.masterkey_kms_provider &
-            ctx->crypt->opts.kms_providers)) {
+      if (!ctx->crypt->opts.use_need_kms_credentials_state &&
+          !(ctx->opts.kek.kms_provider &
+            _mongocrypt_ctx_kms_providers (ctx)->configured_providers)) {
          return _mongocrypt_ctx_fail_w_msg (
             ctx, "requested kms provider not configured");
       }
    }
 
-   if (opts_spec->masterkey == OPT_PROHIBITED &&
-       ctx->opts.masterkey_kms_provider) {
+   if (opts_spec->kek == OPT_PROHIBITED && ctx->opts.kek.kms_provider) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "master key prohibited");
    }
 
@@ -713,6 +902,11 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
       }
    }
 
+   if (opts_spec->key_material == OPT_PROHIBITED &&
+       ctx->opts.key_material.owned) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "key material prohibited");
+   }
+
    if (opts_spec->algorithm == OPT_REQUIRED &&
        ctx->opts.algorithm == MONGOCRYPT_ENCRYPTION_ALGORITHM_NONE) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "algorithm required");
@@ -723,16 +917,6 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
       return _mongocrypt_ctx_fail_w_msg (ctx, "algorithm prohibited");
    }
 
-   if (opts_spec->endpoint == OPT_REQUIRED &&
-       !ctx->opts.masterkey_aws_endpoint) {
-      return _mongocrypt_ctx_fail_w_msg (ctx, "endpoint required");
-   }
-
-   if (opts_spec->endpoint == OPT_PROHIBITED &&
-       ctx->opts.masterkey_aws_endpoint) {
-      return _mongocrypt_ctx_fail_w_msg (ctx, "endpoint prohibited");
-   }
-
    _mongocrypt_key_broker_init (&ctx->kb, ctx->crypt);
    return true;
 }
@@ -760,12 +944,23 @@ _mongocrypt_ctx_state_from_key_broker (mongocrypt_ctx_t *ctx)
       ret = false;
       break;
    case KB_ADDING_DOCS:
-      /* Require key documents from driver. */
+      /* Encrypted keys need KMS, which need to be provided before
+       * adding docs. */
+      if (_mongocrypt_needs_credentials (ctx->crypt)) {
+         new_state = MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS;
+      } else {
+         /* Require key documents from driver. */
+         new_state = MONGOCRYPT_CTX_NEED_MONGO_KEYS;
+      }
+      ret = true;
+      break;
+   case KB_ADDING_DOCS_ANY:
+      /* Assume KMS credentials have been provided. */
       new_state = MONGOCRYPT_CTX_NEED_MONGO_KEYS;
       ret = true;
       break;
+   case KB_AUTHENTICATING:
    case KB_DECRYPTING_KEY_MATERIAL:
-      /* Encrypted keys need KMS. */
       new_state = MONGOCRYPT_CTX_NEED_KMS;
       ret = true;
       break;
@@ -778,8 +973,9 @@ _mongocrypt_ctx_state_from_key_broker (mongocrypt_ctx_t *ctx)
       ret = true;
       break;
    /* As currently implemented, we do not expect to ever be in KB_REQUESTING
-    * state when calling this function. */
+    * or KB_REQUESTING_ANY state when calling this function. */
    case KB_REQUESTING:
+   default:
       CLIENT_ERR ("key broker in unexpected state");
       new_state = MONGOCRYPT_CTX_ERROR;
       ret = false;
@@ -811,14 +1007,136 @@ mongocrypt_ctx_setopt_masterkey_aws_endpoint (mongocrypt_ctx_t *ctx,
       return false;
    }
 
-   if (ctx->opts.masterkey_aws_endpoint) {
+   if (ctx->opts.kek.kms_provider != MONGOCRYPT_KMS_PROVIDER_AWS &&
+       ctx->opts.kek.kms_provider != MONGOCRYPT_KMS_PROVIDER_NONE) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "endpoint prohibited");
+   }
+
+   ctx->opts.kek.kms_provider = MONGOCRYPT_KMS_PROVIDER_AWS;
+
+   if (ctx->opts.kek.provider.aws.endpoint) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "already set masterkey endpoint");
    }
 
-   if (!_mongocrypt_validate_and_copy_string (
-          endpoint, endpoint_len, &ctx->opts.masterkey_aws_endpoint)) {
-      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid masterkey endpoint");
+   ctx->opts.kek.provider.aws.endpoint = _mongocrypt_endpoint_new (
+      endpoint, endpoint_len, NULL /* opts */, ctx->status);
+   if (!ctx->opts.kek.provider.aws.endpoint) {
+      return _mongocrypt_ctx_fail (ctx);
    }
-   ctx->opts.masterkey_aws_endpoint_len = endpoint_len;
+
    return true;
-}
\ No newline at end of file
+}
+
+bool
+mongocrypt_ctx_setopt_key_encryption_key (mongocrypt_ctx_t *ctx,
+                                          mongocrypt_binary_t *bin)
+{
+   bson_t as_bson;
+
+   if (!ctx) {
+      return false;
+   }
+
+   if (ctx->initialized) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "cannot set options after init");
+   }
+
+   if (ctx->state == MONGOCRYPT_CTX_ERROR) {
+      return false;
+   }
+
+   if (ctx->opts.kek.kms_provider) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "key encryption key already set");
+   }
+
+   if (!_mongocrypt_binary_to_bson (bin, &as_bson)) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid BSON");
+   }
+
+   if (!_mongocrypt_kek_parse_owned (&as_bson, &ctx->opts.kek, ctx->status)) {
+      return _mongocrypt_ctx_fail (ctx);
+   }
+
+   if (ctx->crypt->log.trace_enabled) {
+      char *bin_str = bson_as_canonical_extended_json (&as_bson, NULL);
+      _mongocrypt_log (&ctx->crypt->log,
+                       MONGOCRYPT_LOG_LEVEL_TRACE,
+                       "%s (%s=\"%s\")",
+                       BSON_FUNC,
+                       "bin",
+                       bin_str);
+      bson_free (bin_str);
+   }
+
+   return true;
+}
+
+_mongocrypt_opts_kms_providers_t *
+_mongocrypt_ctx_kms_providers (mongocrypt_ctx_t *ctx)
+{
+   return ctx->kms_providers.configured_providers
+             ? &ctx->kms_providers
+             : &ctx->crypt->opts.kms_providers;
+}
+
+bool
+mongocrypt_ctx_setopt_contention_factor (mongocrypt_ctx_t *ctx,
+                                         int64_t contention_factor)
+{
+   if (!ctx) {
+      return false;
+   }
+   ctx->opts.contention_factor.value = contention_factor;
+   ctx->opts.contention_factor.set = true;
+   return true;
+}
+
+bool
+mongocrypt_ctx_setopt_index_key_id (mongocrypt_ctx_t *ctx,
+                                    mongocrypt_binary_t *key_id)
+{
+   if (!ctx) {
+      return false;
+   }
+
+   return _set_binary_opt (
+      ctx, key_id, &ctx->opts.index_key_id, BSON_SUBTYPE_UUID);
+}
+
+bool
+mongocrypt_ctx_setopt_query_type (mongocrypt_ctx_t *ctx,
+                                  const char *query_type,
+                                  int len)
+{
+   if (!ctx) {
+      return false;
+   }
+
+   if (ctx->initialized) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "Cannot set options after init");
+   }
+   if (ctx->state == MONGOCRYPT_CTX_ERROR) {
+      return false;
+   }
+   if (len < -1) {
+      return _mongocrypt_ctx_fail_w_msg (ctx,
+                                         "Invalid query_type string length");
+   }
+   if (!query_type) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "Invalid null query_type string");
+   }
+
+   const size_t calc_len = len == -1 ? strlen (query_type) : (size_t) len;
+   mstr_view qt_str = mstrv_view_data (query_type, calc_len);
+   if (mstr_eq (qt_str, mstrv_lit (MONGOCRYPT_QUERY_TYPE_EQUALITY_STR))) {
+      ctx->opts.query_type.value = MONGOCRYPT_QUERY_TYPE_EQUALITY;
+      ctx->opts.query_type.set = true;
+   } else {
+      char *error = bson_strdup_printf (
+         "Unsupported query_type \"%.*s\"", (int) qt_str.len, qt_str.data);
+      _mongocrypt_ctx_fail_w_msg (ctx, error);
+      bson_free (error);
+      return false;
+   }
+   return true;
+}
