@@ -48,23 +48,171 @@ _mongocrypt_opts_kms_provider_gcp_cleanup (
    _mongocrypt_buffer_cleanup (&kms_provider_gcp->private_key);
 }
 
+void
+_mongocrypt_opts_kms_providers_cleanup (
+   _mongocrypt_opts_kms_providers_t *kms_providers)
+{
+   bson_free (kms_providers->aws.secret_access_key);
+   bson_free (kms_providers->aws.access_key_id);
+   bson_free (kms_providers->aws.session_token);
+   _mongocrypt_buffer_cleanup (&kms_providers->local.key);
+   _mongocrypt_opts_kms_provider_azure_cleanup (&kms_providers->azure);
+   _mongocrypt_opts_kms_provider_gcp_cleanup (&kms_providers->gcp);
+   _mongocrypt_endpoint_destroy (kms_providers->kmip.endpoint);
+}
+
+
+void
+_mongocrypt_opts_merge_kms_providers (
+   _mongocrypt_opts_kms_providers_t *dest,
+   const _mongocrypt_opts_kms_providers_t *source)
+{
+   if (source->configured_providers & MONGOCRYPT_KMS_PROVIDER_AWS) {
+      memcpy (&dest->aws, &source->aws, sizeof (source->aws));
+      dest->configured_providers |= MONGOCRYPT_KMS_PROVIDER_AWS;
+   }
+   if (source->configured_providers & MONGOCRYPT_KMS_PROVIDER_LOCAL) {
+      memcpy (&dest->local, &source->local, sizeof (source->local));
+      dest->configured_providers |= MONGOCRYPT_KMS_PROVIDER_LOCAL;
+   }
+   if (source->configured_providers & MONGOCRYPT_KMS_PROVIDER_AZURE) {
+      memcpy (&dest->azure, &source->azure, sizeof (source->azure));
+      dest->configured_providers |= MONGOCRYPT_KMS_PROVIDER_AZURE;
+   }
+   if (source->configured_providers & MONGOCRYPT_KMS_PROVIDER_GCP) {
+      memcpy (&dest->gcp, &source->gcp, sizeof (source->gcp));
+      dest->configured_providers |= MONGOCRYPT_KMS_PROVIDER_GCP;
+   }
+   if (source->configured_providers & MONGOCRYPT_KMS_PROVIDER_KMIP) {
+      memcpy (&dest->kmip, &source->kmip, sizeof (source->kmip));
+      dest->configured_providers |= MONGOCRYPT_KMS_PROVIDER_KMIP;
+   }
+   /* ensure all providers were copied */
+   BSON_ASSERT (!(source->configured_providers & ~dest->configured_providers));
+}
+
+
 void
 _mongocrypt_opts_cleanup (_mongocrypt_opts_t *opts)
 {
-   bson_free (opts->kms_provider_aws.secret_access_key);
-   bson_free (opts->kms_provider_aws.access_key_id);
-   bson_free (opts->kms_provider_aws.session_token);
-   _mongocrypt_buffer_cleanup (&opts->kms_provider_local.key);
+   _mongocrypt_opts_kms_providers_cleanup (&opts->kms_providers);
    _mongocrypt_buffer_cleanup (&opts->schema_map);
-   _mongocrypt_opts_kms_provider_azure_cleanup (&opts->kms_provider_azure);
-   _mongocrypt_opts_kms_provider_gcp_cleanup (&opts->kms_provider_gcp);
-   _mongocrypt_endpoint_destroy (opts->kms_provider_kmip.endpoint);
+   _mongocrypt_buffer_cleanup (&opts->encrypted_field_config_map);
    // Free any lib search paths added by the caller
-   for (int i = 0; i < opts->n_cselib_search_paths; ++i) {
-      mstr_free (opts->cselib_search_paths[i]);
+   for (int i = 0; i < opts->n_crypt_shared_lib_search_paths; ++i) {
+      mstr_free (opts->crypt_shared_lib_search_paths[i]);
    }
-   bson_free (opts->cselib_search_paths);
-   mstr_free (opts->csfle_lib_override_path);
+   bson_free (opts->crypt_shared_lib_search_paths);
+   mstr_free (opts->crypt_shared_lib_override_path);
+}
+
+
+bool
+_mongocrypt_opts_kms_providers_validate (
+   _mongocrypt_opts_kms_providers_t *kms_providers, mongocrypt_status_t *status)
+{
+   if (!kms_providers->configured_providers &&
+       !kms_providers->need_credentials) {
+      CLIENT_ERR ("no kms provider set");
+      return false;
+   }
+
+   if (kms_providers->configured_providers & MONGOCRYPT_KMS_PROVIDER_AWS) {
+      if (!kms_providers->aws.access_key_id ||
+          !kms_providers->aws.secret_access_key) {
+         CLIENT_ERR ("aws credentials unset");
+         return false;
+      }
+   }
+
+   if (kms_providers->configured_providers & MONGOCRYPT_KMS_PROVIDER_LOCAL) {
+      if (_mongocrypt_buffer_empty (&kms_providers->local.key)) {
+         CLIENT_ERR ("local data key unset");
+         return false;
+      }
+   }
+
+   return true;
+}
+
+/* _shares_bson_fields checks if @one or @two share any top-level field names.
+ * Returns false on error and sets @status. Returns true if no error
+ * occurred. Sets @found to the first shared field name found.
+ * If no shared field names are found, @found is set to NULL.
+ */
+static bool
+_shares_bson_fields (bson_t *one,
+                     bson_t *two,
+                     const char **found,
+                     mongocrypt_status_t *status)
+{
+   bson_iter_t iter1;
+   bson_iter_t iter2;
+
+   *found = NULL;
+   if (!bson_iter_init (&iter1, one)) {
+      CLIENT_ERR ("error iterating one BSON in _shares_bson_fields");
+      return false;
+   }
+   while (bson_iter_next (&iter1)) {
+      const char *key1 = bson_iter_key (&iter1);
+
+      if (!bson_iter_init (&iter2, two)) {
+         CLIENT_ERR ("error iterating two BSON in _shares_bson_fields");
+         return false;
+      }
+      while (bson_iter_next (&iter2)) {
+         const char *key2 = bson_iter_key (&iter2);
+         if (0 == strcmp (key1, key2)) {
+            *found = key1;
+            return true;
+         }
+      }
+   }
+   return true;
+}
+
+/* _validate_encrypted_field_config_map_and_schema_map validates that the same
+ * namespace is not both in encrypted_field_config_map and schema_map. */
+bool
+_validate_encrypted_field_config_map_and_schema_map (
+   _mongocrypt_buffer_t *encrypted_field_config_map,
+   _mongocrypt_buffer_t *schema_map,
+   mongocrypt_status_t *status)
+{
+   const char *found;
+   bson_t schema_map_bson;
+   bson_t encrypted_field_config_map_bson;
+
+   /* If either map is unset, there is nothing to validate. Return true to
+    * signal no error. */
+   if (_mongocrypt_buffer_empty (encrypted_field_config_map)) {
+      return true;
+   }
+   if (_mongocrypt_buffer_empty (schema_map)) {
+      return true;
+   }
+
+   if (!_mongocrypt_buffer_to_bson (schema_map, &schema_map_bson)) {
+      CLIENT_ERR ("error converting schema_map to BSON");
+      return false;
+   }
+   if (!_mongocrypt_buffer_to_bson (encrypted_field_config_map,
+                                    &encrypted_field_config_map_bson)) {
+      CLIENT_ERR ("error converting encrypted_field_config_map to BSON");
+      return false;
+   }
+   if (!_shares_bson_fields (
+          &schema_map_bson, &encrypted_field_config_map_bson, &found, status)) {
+      return false;
+   }
+   if (found != NULL) {
+      CLIENT_ERR (
+         "%s is present in both schema_map and encrypted_field_config_map",
+         found);
+      return false;
+   }
+   return true;
 }
 
 
@@ -72,27 +220,12 @@ bool
 _mongocrypt_opts_validate (_mongocrypt_opts_t *opts,
                            mongocrypt_status_t *status)
 {
-   if (!opts->kms_providers) {
-      CLIENT_ERR ("no kms provider set");
+   if (!_validate_encrypted_field_config_map_and_schema_map (
+          &opts->encrypted_field_config_map, &opts->schema_map, status)) {
       return false;
    }
-
-   if (opts->kms_providers & MONGOCRYPT_KMS_PROVIDER_AWS) {
-      if (!opts->kms_provider_aws.access_key_id ||
-          !opts->kms_provider_aws.secret_access_key) {
-         CLIENT_ERR ("aws credentials unset");
-         return false;
-      }
-   }
-
-   if (opts->kms_providers & MONGOCRYPT_KMS_PROVIDER_LOCAL) {
-      if (_mongocrypt_buffer_empty (&opts->kms_provider_local.key)) {
-         CLIENT_ERR ("local data key unset");
-         return false;
-      }
-   }
-
-   return true;
+   return _mongocrypt_opts_kms_providers_validate (&opts->kms_providers,
+                                                   status);
 }
 
 bool
@@ -293,4 +426,4 @@ _mongocrypt_check_allowed_fields_va (const bson_t *bson,
       }
    }
    return true;
-}
\ No newline at end of file
+}
