@@ -26,4 +26,14 @@ _mongocrypt_status_copy_to (mongocrypt_status_t *src, mongocrypt_status_t *dst);
 void
 _mongocrypt_status_reset (mongocrypt_status_t *status);
 
+/* Append the message of @to_append in @status.
+ * Example:
+ * - @status has the message "status error"
+ * - @to_append has the message "append error"
+ * After calling, @status will have the message: "status error: append error"
+ */
+void
+_mongocrypt_status_append (mongocrypt_status_t *status,
+                           mongocrypt_status_t *to_append);
+
 #endif /* MONGOCRYPT_STATUS_PRIVATE_H */
