@@ -17,6 +17,9 @@
 #include "bson/bson.h"
 
 #include <php.h>
+#if PHP_VERSION_ID >= 80100
+#include <Zend/zend_enum.h>
+#endif
 #include <Zend/zend_interfaces.h>
 
 #include "php_phongo.h"
@@ -43,12 +46,13 @@
 #endif
 
 /* Forwards declarations */
+static void php_phongo_bson_append(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* entry);
 static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out);
 
 /* Determines whether the argument should be serialized as a BSON array or
  * document. IS_ARRAY is returned if the argument's keys are a sequence of
  * integers starting at zero; otherwise, IS_OBJECT is returned. */
-static int php_phongo_is_array_or_document(zval* val) /* {{{ */
+static int php_phongo_is_array_or_document(zval* val)
 {
 	HashTable* ht_data = HASH_OF(val);
 	int        count;
@@ -80,14 +84,14 @@ static int php_phongo_is_array_or_document(zval* val) /* {{{ */
 	}
 
 	return IS_ARRAY;
-} /* }}} */
+}
 
 /* Appends the array or object argument to the BSON document. If the object is
  * an instance of MongoDB\BSON\Serializable, the return value of bsonSerialize()
  * will be appended as an embedded document. Other MongoDB\BSON\Type instances
  * will be appended as the appropriate BSON type. Other array or object values
  * will be appended as an embedded document. */
-static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* object) /* {{{ */
+static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* object)
 {
 	if (Z_TYPE_P(object) == IS_OBJECT && instanceof_function(Z_OBJCE_P(object), php_phongo_cursorid_ce)) {
 		bson_append_int64(bson, key, key_len, Z_CURSORID_OBJ_P(object)->id);
@@ -236,7 +240,23 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 
 		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Unexpected %s instance: %s", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(object)->name));
 		return;
-	} else {
+	}
+
+#if PHP_VERSION_ID >= 80100
+	if (Z_TYPE_P(object) == IS_OBJECT && Z_OBJCE_P(object)->ce_flags & ZEND_ACC_ENUM) {
+		if (Z_OBJCE_P(object)->enum_backing_type == IS_UNDEF) {
+			char* path_string = php_phongo_field_path_as_string(field_path);
+			phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Non-backed enum %s cannot be serialized for field path \"%s\"", ZSTR_VAL(Z_OBJCE_P(object)->name), path_string);
+			efree(path_string);
+			return;
+		}
+
+		php_phongo_bson_append(bson, field_path, flags, key, key_len, zend_enum_fetch_case_value(Z_OBJ_P(object)));
+		return;
+	}
+#endif /* PHP_VERSION_ID >= 80100 */
+
+	{
 		bson_t child;
 
 		mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding document");
@@ -244,12 +264,12 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 		php_phongo_zval_to_bson_internal(object, field_path, flags, &child, NULL);
 		bson_append_document_end(bson, &child);
 	}
-} /* }}} */
+}
 
 /* Appends the zval argument to the BSON document. If the argument is an object,
  * or an array that should be serialized as an embedded document, this function
  * will defer to php_phongo_bson_append_object(). */
-static void php_phongo_bson_append(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* entry) /* {{{ */
+static void php_phongo_bson_append(bson_t* bson, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, const char* key, long key_len, zval* entry)
 {
 	php_phongo_field_path_write_item_at_current_level(field_path, key);
 
@@ -337,9 +357,9 @@ try_again:
 			efree(path_string);
 		}
 	}
-} /* }}} */
+}
 
-static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out) /* {{{ */
+static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path* field_path, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out)
 {
 	HashTable* ht_data = NULL;
 	zval       obj_data;
@@ -389,6 +409,16 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 				break;
 			}
 
+			/* For the error handling that follows, we can safely assume that we
+			 * are at the root level, since php_phongo_bson_append_object would
+			 * have already been called for a non-root level. */
+#if PHP_VERSION_ID >= 80100
+			if (Z_OBJCE_P(data)->ce_flags & ZEND_ACC_ENUM) {
+				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Enum %s cannot be serialized as a root element", ZSTR_VAL(Z_OBJCE_P(data)->name));
+				return;
+			}
+#endif /* PHP_VERSION_ID >= 80100 */
+
 			if (instanceof_function(Z_OBJCE_P(data), php_phongo_type_ce)) {
 				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "%s instance %s cannot be serialized as a root element", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(data)->name));
 				return;
@@ -478,41 +508,40 @@ cleanup:
 	if (!Z_ISUNDEF(obj_data)) {
 		zval_ptr_dtor(&obj_data);
 	}
-} /* }}} */
+}
 
 /* Converts the array or object argument to a BSON document. If the object is an
  * instance of MongoDB\BSON\Serializable, the return value of bsonSerialize()
  * will be used. */
-void php_phongo_zval_to_bson(zval* data, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out) /* {{{ */
+void php_phongo_zval_to_bson(zval* data, php_phongo_bson_flags_t flags, bson_t* bson, bson_t** bson_out)
 {
 	php_phongo_field_path* field_path = php_phongo_field_path_alloc(false);
 
 	php_phongo_zval_to_bson_internal(data, field_path, flags, bson, bson_out);
 
 	php_phongo_field_path_free(field_path);
-} /* }}} */
+}
 
 /* Converts the argument to a bson_value_t. If the object is an instance of
  * MongoDB\BSON\Serializable, the return value of bsonSerialize() will be
- * used. */
-void php_phongo_zval_to_bson_value(zval* data, php_phongo_bson_flags_t flags, bson_value_t* value) /* {{{ */
+ * used. It is the caller's responsibility to call bson_value_destroy. */
+void php_phongo_zval_to_bson_value(zval* data, php_phongo_bson_flags_t flags, bson_value_t* value)
 {
 	bson_iter_t iter;
 	bson_t      bson = BSON_INITIALIZER;
+	zval        data_object;
 
-	zval* data_object = ecalloc(1, sizeof(zval));
-
-	array_init_size(data_object, 1);
-	add_assoc_zval(data_object, "data", data);
+	array_init_size(&data_object, 1);
+	add_assoc_zval(&data_object, "data", data);
 
 	Z_TRY_ADDREF_P(data);
 
-	php_phongo_zval_to_bson(data_object, flags, &bson, NULL);
+	php_phongo_zval_to_bson(&data_object, flags, &bson, NULL);
 
 	if (bson_iter_init_find(&iter, &bson, "data")) {
 		bson_value_copy(bson_iter_value(&iter), value);
 	}
 
-	zval_ptr_dtor(data_object);
-	efree(data_object);
-} /* }}} */
+	bson_destroy(&bson);
+	zval_ptr_dtor(&data_object);
+}
