@@ -51,7 +51,48 @@ typedef struct {
 	bool             is_persistent;
 } php_phongo_pclient_t;
 
-static mongoc_uri_t* php_phongo_make_uri(const char* uri_string) /* {{{ */
+static const mongoc_client_t* get_first_pclient_client(HashTable* ht)
+{
+	if (ht) {
+		php_phongo_pclient_t* pclient = NULL;
+
+		zend_hash_internal_pointer_reset(ht);
+		pclient = zend_hash_get_current_data_ptr(ht);
+
+		if (pclient) {
+			return pclient->client;
+		}
+	}
+
+	return NULL;
+}
+
+/* Returns the version of the crypt_shared library, or NULL if it's unavailable.
+ * Querying the version requires a mongoc_client_t pointer. Since the shared
+ * library can only be loaded once, any client will return the same result so we
+ * consult the first persistent or request-scoped client we can find.
+ *
+ * Note: this may incorrectly return NULL if crypt_shared was loaded through a
+ * mongoc_client_t since destroyed (e.g. single requested-scoped client);
+ * however, that's the best can do with libmongoc's API. */
+const char* php_phongo_crypt_shared_version()
+{
+	const mongoc_client_t* client = NULL;
+
+	client = get_first_pclient_client(&MONGODB_G(persistent_clients));
+
+	if (!client) {
+		client = get_first_pclient_client(MONGODB_G(request_clients));
+	}
+
+	if (client) {
+		return mongoc_client_get_crypt_shared_version(client);
+	}
+
+	return NULL;
+}
+
+static mongoc_uri_t* php_phongo_make_uri(const char* uri_string)
 {
 	mongoc_uri_t* uri;
 	bson_error_t  error = { 0 };
@@ -65,59 +106,7 @@ static mongoc_uri_t* php_phongo_make_uri(const char* uri_string) /* {{{ */
 	}
 
 	return uri;
-} /* }}} */
-
-static const char* php_phongo_bson_type_to_string(bson_type_t type) /* {{{ */
-{
-	switch (type) {
-		case BSON_TYPE_EOD:
-			return "EOD";
-		case BSON_TYPE_DOUBLE:
-			return "double";
-		case BSON_TYPE_UTF8:
-			return "string";
-		case BSON_TYPE_DOCUMENT:
-			return "document";
-		case BSON_TYPE_ARRAY:
-			return "array";
-		case BSON_TYPE_BINARY:
-			return "Binary";
-		case BSON_TYPE_UNDEFINED:
-			return "undefined";
-		case BSON_TYPE_OID:
-			return "ObjectId";
-		case BSON_TYPE_BOOL:
-			return "boolean";
-		case BSON_TYPE_DATE_TIME:
-			return "UTCDateTime";
-		case BSON_TYPE_NULL:
-			return "null";
-		case BSON_TYPE_REGEX:
-			return "Regex";
-		case BSON_TYPE_DBPOINTER:
-			return "DBPointer";
-		case BSON_TYPE_CODE:
-			return "Javascript";
-		case BSON_TYPE_SYMBOL:
-			return "symbol";
-		case BSON_TYPE_CODEWSCOPE:
-			return "Javascript with scope";
-		case BSON_TYPE_INT32:
-			return "32-bit integer";
-		case BSON_TYPE_TIMESTAMP:
-			return "Timestamp";
-		case BSON_TYPE_INT64:
-			return "64-bit integer";
-		case BSON_TYPE_DECIMAL128:
-			return "Decimal128";
-		case BSON_TYPE_MAXKEY:
-			return "MaxKey";
-		case BSON_TYPE_MINKEY:
-			return "MinKey";
-		default:
-			return "unknown";
-	}
-} /* }}} */
+}
 
 #define PHONGO_URI_INVALID_TYPE(iter, expected)        \
 	phongo_throw_exception(                            \
@@ -127,7 +116,7 @@ static const char* php_phongo_bson_type_to_string(bson_type_t type) /* {{{ */
 		bson_iter_key(&(iter)),                        \
 		php_phongo_bson_type_to_string(bson_iter_type(&(iter))))
 
-static bool php_phongo_apply_options_to_uri(mongoc_uri_t* uri, bson_t* options) /* {{{ */
+static bool php_phongo_apply_options_to_uri(mongoc_uri_t* uri, bson_t* options)
 {
 	bson_iter_t iter;
 
@@ -328,9 +317,9 @@ static bool php_phongo_apply_options_to_uri(mongoc_uri_t* uri, bson_t* options)
 	}
 
 	return true;
-} /* }}} */
+}
 
-static bool php_phongo_apply_rc_options_to_uri(mongoc_uri_t* uri, bson_t* options) /* {{{ */
+static bool php_phongo_apply_rc_options_to_uri(mongoc_uri_t* uri, bson_t* options)
 {
 	bson_iter_t                  iter;
 	mongoc_read_concern_t*       new_rc;
@@ -368,9 +357,9 @@ static bool php_phongo_apply_rc_options_to_uri(mongoc_uri_t* uri, bson_t* option
 	mongoc_read_concern_destroy(new_rc);
 
 	return true;
-} /* }}} */
+}
 
-static bool php_phongo_apply_rp_options_to_uri(mongoc_uri_t* uri, bson_t* options) /* {{{ */
+static bool php_phongo_apply_rp_options_to_uri(mongoc_uri_t* uri, bson_t* options)
 {
 	bson_iter_t                iter;
 	mongoc_read_prefs_t*       new_rp;
@@ -524,9 +513,9 @@ static bool php_phongo_apply_rp_options_to_uri(mongoc_uri_t* uri, bson_t* option
 	mongoc_read_prefs_destroy(new_rp);
 
 	return true;
-} /* }}} */
+}
 
-static bool php_phongo_apply_wc_options_to_uri(mongoc_uri_t* uri, bson_t* options) /* {{{ */
+static bool php_phongo_apply_wc_options_to_uri(mongoc_uri_t* uri, bson_t* options)
 {
 	bson_iter_t                   iter;
 	mongoc_write_concern_t*       new_wc;
@@ -657,7 +646,7 @@ static bool php_phongo_apply_wc_options_to_uri(mongoc_uri_t* uri, bson_t* option
 	mongoc_write_concern_destroy(new_wc);
 
 	return true;
-} /* }}} */
+}
 
 #ifdef MONGOC_ENABLE_SSL
 static void php_phongo_mongoc_ssl_opts_from_uri(mongoc_ssl_opt_t* ssl_opt, mongoc_uri_t* uri, bool* any_ssl_option_set)
@@ -1124,7 +1113,7 @@ cleanup:
 	}
 }
 
-static mongoc_client_t* php_phongo_make_mongo_client(const mongoc_uri_t* uri, zval* driverOptions) /* {{{ */
+static mongoc_client_t* php_phongo_make_mongo_client(const mongoc_uri_t* uri, zval* driverOptions)
 {
 	const char *     mongoc_version, *bson_version;
 	mongoc_client_t* client;
@@ -1159,7 +1148,7 @@ static mongoc_client_t* php_phongo_make_mongo_client(const mongoc_uri_t* uri, zv
 	}
 
 	return client;
-} /* }}} */
+}
 
 /* Adds a client to the appropriate registry. Persistent and request-scoped
  * clients each have their own registries (i.e. HashTables), which use different
@@ -1230,7 +1219,7 @@ static mongoc_client_t* php_phongo_find_persistent_client(const char* hash, size
 	return NULL;
 }
 
-static bool phongo_manager_set_serverapi_opts(php_phongo_manager_t* manager, zval* driverOptions) /* {{{ */
+static bool phongo_manager_set_serverapi_opts(php_phongo_manager_t* manager, zval* driverOptions)
 {
 	zval*                   zServerApi;
 	php_phongo_serverapi_t* server_api;
@@ -1255,10 +1244,10 @@ static bool phongo_manager_set_serverapi_opts(php_phongo_manager_t* manager, zva
 	}
 
 	return true;
-} /* }}} */
+}
 
 #ifdef MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION
-static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manager, zval* driverOptions) /* {{{ */
+static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manager, zval* driverOptions)
 {
 	zval*                          zAutoEncryptionOpts;
 	bson_error_t                   error                = { 0 };
@@ -1278,11 +1267,42 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 
 	auto_encryption_opts = mongoc_auto_encryption_opts_new();
 
+	if (php_array_existsc(zAutoEncryptionOpts, "bypassAutoEncryption")) {
+		mongoc_auto_encryption_opts_set_bypass_auto_encryption(auto_encryption_opts, php_array_fetch_bool(zAutoEncryptionOpts, "bypassAutoEncryption"));
+	}
+
+	if (php_array_existsc(zAutoEncryptionOpts, "bypassQueryAnalysis")) {
+		mongoc_auto_encryption_opts_set_bypass_query_analysis(auto_encryption_opts, php_array_fetch_bool(zAutoEncryptionOpts, "bypassQueryAnalysis"));
+	}
+
+	if (php_array_existsc(zAutoEncryptionOpts, "encryptedFieldsMap")) {
+		zval*  enc_fields_map = php_array_fetch(zAutoEncryptionOpts, "encryptedFieldsMap");
+		bson_t bson_map       = BSON_INITIALIZER;
+
+		if (Z_TYPE_P(enc_fields_map) != IS_OBJECT && Z_TYPE_P(enc_fields_map) != IS_ARRAY) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"encryptedFieldsMap\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(enc_fields_map));
+			goto cleanup;
+		}
+
+		php_phongo_zval_to_bson(enc_fields_map, PHONGO_BSON_NONE, &bson_map, NULL);
+		if (EG(exception)) {
+			goto cleanup;
+		}
+
+		mongoc_auto_encryption_opts_set_encrypted_fields_map(auto_encryption_opts, &bson_map);
+
+		bson_destroy(&bson_map);
+
+		/* Copy the encryptedFieldsMap to the Manager since PHPLIB may need to
+		 * access it for createCollection and dropCollection helpers. */
+		ZVAL_ZVAL(&manager->enc_fields_map, enc_fields_map, 1, 0);
+	}
+
 	if (php_array_existsc(zAutoEncryptionOpts, "keyVaultClient")) {
 		zval* key_vault_client = php_array_fetch(zAutoEncryptionOpts, "keyVaultClient");
 
 		if (Z_TYPE_P(key_vault_client) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(key_vault_client), php_phongo_manager_ce)) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"keyVaultClient\" encryption option to be %s, %s given", ZSTR_VAL(php_phongo_manager_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(key_vault_client));
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"keyVaultClient\" autoEncryption option to be %s, %s given", ZSTR_VAL(php_phongo_manager_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(key_vault_client));
 			goto cleanup;
 		}
 
@@ -1312,7 +1332,7 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 		key_vault_ns = php_array_fetch_string(zAutoEncryptionOpts, "keyVaultNamespace", &plen, &pfree);
 
 		if (!phongo_split_namespace(key_vault_ns, &db_name, &coll_name)) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"keyVaultNamespace\" encryption option to contain a full collection name");
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"keyVaultNamespace\" autoEncryption option to contain a full collection namespace");
 
 			if (pfree) {
 				efree(key_vault_ns);
@@ -1336,7 +1356,7 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 		bson_t bson_providers = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(kms_providers) != IS_OBJECT && Z_TYPE_P(kms_providers) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"kmsProviders\" encryption option to be an array or object");
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"kmsProviders\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(kms_providers));
 			goto cleanup;
 		}
 
@@ -1355,7 +1375,7 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 		bson_t bson_map   = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(schema_map) != IS_OBJECT && Z_TYPE_P(schema_map) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"schemaMap\" encryption option to be an array or object");
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"schemaMap\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(schema_map));
 			goto cleanup;
 		}
 
@@ -1374,7 +1394,7 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 		bson_t bson_options = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(tls_options) != IS_OBJECT && Z_TYPE_P(tls_options) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"tlsOptions\" encryption option to be an array or object");
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"tlsOptions\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(tls_options));
 			goto cleanup;
 		}
 
@@ -1388,18 +1408,12 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 		bson_destroy(&bson_options);
 	}
 
-	if (php_array_existsc(zAutoEncryptionOpts, "bypassAutoEncryption")) {
-		zend_bool bypass_auto_encryption = php_array_fetch_bool(zAutoEncryptionOpts, "bypassAutoEncryption");
-
-		mongoc_auto_encryption_opts_set_bypass_auto_encryption(auto_encryption_opts, bypass_auto_encryption);
-	}
-
 	if (php_array_existsc(zAutoEncryptionOpts, "extraOptions")) {
 		zval*  extra_options = php_array_fetch(zAutoEncryptionOpts, "extraOptions");
 		bson_t bson_options  = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(extra_options) != IS_OBJECT && Z_TYPE_P(extra_options) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"extraOptions\" encryption option to be an array or object");
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"extraOptions\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(extra_options));
 			goto cleanup;
 		}
 
@@ -1424,9 +1438,8 @@ cleanup:
 	mongoc_auto_encryption_opts_destroy(auto_encryption_opts);
 	return retval;
 }
-/* }}} */
 #else  /* MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION */
-static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manager, zval* driverOptions) /* {{{ */
+static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manager, zval* driverOptions)
 {
 	if (!driverOptions || !php_array_existsc(driverOptions, "autoEncryption")) {
 		return true;
@@ -1436,10 +1449,9 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 
 	return false;
 }
-/* }}} */
 #endif /* MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION */
 
-void phongo_manager_init(php_phongo_manager_t* manager, const char* uri_string, zval* options, zval* driverOptions) /* {{{ */
+void phongo_manager_init(php_phongo_manager_t* manager, const char* uri_string, zval* options, zval* driverOptions)
 {
 	bson_t        bson_options = BSON_INITIALIZER;
 	mongoc_uri_t* uri          = NULL;
@@ -1550,7 +1562,7 @@ cleanup:
 		php_phongo_free_ssl_opt(ssl_opt);
 	}
 #endif
-} /* }}} */
+}
 
 static void phongo_pclient_reset_once(php_phongo_pclient_t* pclient, int pid)
 {
