@@ -5,7 +5,6 @@ use MongoDB\Driver\Command;
 use MongoDB\Driver\Manager;
 use MongoDB\Driver\ReadPreference;
 use MongoDB\Driver\Server;
-use MongoDB\Driver\ServerApi;
 use MongoDB\Driver\WriteConcern;
 use MongoDB\Driver\WriteConcernError;
 use MongoDB\Driver\WriteError;
@@ -51,18 +50,15 @@ function drop_collection($uri, $databaseName, $collectionName)
     $command = new Command(['drop' => $collectionName]);
 
     try {
-        /* Unless we are dropping a collection within the "local" database,
-         * which does not support a write concern, we need to use w:majority due
-         * to the issue explained in SERVER-35613: "drop" uses a two phase
-         * commit, and due to that, it is possible that a lock can't be acquired
-         * for a transaction that gets quickly started as the "drop" reaper
-         * hasn't completed yet. */
-        $wc = $databaseName === 'local' ? new WriteConcern(1) : new WriteConcern(WriteConcern::MAJORITY);
-
+        /* We need to use WriteConcern::MAJORITY here due to the issue
+         * explained in SERVER-35613: "drop" uses a two phase commit, and due
+         * to that, it is possible that a lock can't be acquired for a
+         * transaction that gets quickly started as the "drop" reaper hasn't
+         * completed yet. */
         $server->executeCommand(
             $databaseName,
             $command,
-            ['writeConcern' => $wc]
+            ['writeConcern' => new WriteConcern(WriteConcern::MAJORITY)]
         );
     } catch (RuntimeException $e) {
         if ($e->getMessage() !== 'ns not found') {
@@ -92,15 +88,6 @@ function get_module_info($row)
     return $matches[1];
 }
 
-function create_test_manager(string $uri = null, array $options = [], array $driverOptions = [])
-{
-    if (getenv('API_VERSION') && ! isset($driverOptions['serverApi'])) {
-        $driverOptions['serverApi'] = new ServerApi(getenv('API_VERSION'));
-    }
-
-    return new Manager($uri ?? URI, $options, $driverOptions);
-}
-
 /**
  * Returns the primary server.
  *
@@ -110,7 +97,7 @@ function create_test_manager(string $uri = null, array $options = [], array $dri
  */
 function get_primary_server($uri)
 {
-    return create_test_manager($uri)->selectServer(new ReadPreference('primary'));
+    return (new Manager($uri))->selectServer(new ReadPreference('primary'));
 }
 
 /**
@@ -122,7 +109,7 @@ function get_primary_server($uri)
  */
 function get_secondary_server($uri)
 {
-    return create_test_manager($uri)->selectServer(new ReadPreference('secondary'));
+    return (new Manager($uri))->selectServer(new ReadPreference('secondary'));
 }
 
 /**
@@ -222,17 +209,6 @@ function get_uri_option($uri, $option)
     return $matches[1];
 }
 
-/**
- * Checks that the topology is load balanced.
- *
- * @param string $uri
- * @return boolean
- */
-function is_load_balanced($uri)
-{
-    return get_primary_server($uri)->getType() === Server::TYPE_LOAD_BALANCER;
-}
-
 /**
  * Checks that the topology is a sharded cluster.
  *
@@ -245,19 +221,19 @@ function is_mongos($uri)
 }
 
 /**
- * Checks that the topology is a sharded cluster using a replica set.
- *
- * Note: only the first shard is checked.
+ * Checks that the topology is a sharded cluster using a replica set
  */
-function is_sharded_cluster_with_replica_set($uri)
+function is_mongos_with_replica_set($uri)
 {
-    $server = get_primary_server($uri);
-
-    if ($server->getType() !== Server::TYPE_MONGOS && $server->getType() !== Server::TYPE_LOAD_BALANCER) {
+    if (! is_mongos($uri)) {
         return false;
     }
 
-    $cursor = $server->executeQuery('config.shards', new \MongoDB\Driver\Query([], ['limit' => 1]));
+    $cursor = get_primary_server($uri)->executeQuery(
+        'config.shards',
+        new \MongoDB\Driver\Query([], ['limit' => 1])
+    );
+
     $cursor->setTypeMap(['root' => 'array', 'document' => 'array']);
     $document = current($cursor->toArray());
 
@@ -285,8 +261,6 @@ function is_replica_set($uri)
         return false;
     }
 
-    /* Note: this may return a false negative if replicaSet is specified through
-     * a TXT record for a mongodb+srv connection string. */
     if (get_uri_option($uri, 'replicaSet') === NULL) {
         return false;
     }
@@ -449,22 +423,7 @@ function json_canonicalize($json)
      * decoding to a stdClass (see: https://bugs.php.net/bug.php?id=46600). Work
      * around this by replacing "_empty_" keys before returning.
      */
-    $json = str_replace('"_empty_":', '"":', $json);
-
-    /* Canonicalize string values for $numberDouble to ensure they are converted
-     * the same as number literals in legacy and relaxed output. This is needed
-     * because the printf format in _bson_as_json_visit_double uses a high level
-     * of precision and may not produce the exponent notation expected by the
-     * BSON corpus tests. */
-    $json = preg_replace_callback(
-        '/{"\$numberDouble":"(-?\d+(\.\d+([eE]\+\d+)?)?)"}/',
-        function ($matches) {
-            return '{"$numberDouble":"' . json_encode(json_decode($matches[1])) . '"}';
-        },
-        $json
-    );
-
-    return $json;
+    return str_replace('"_empty_":', '"":', $json);
 }
 
 /**
@@ -584,120 +543,84 @@ function destroyTemporaryMongoInstance($id = NULL)
     $json = file_get_contents(MONGO_ORCHESTRATION_URI . "/servers/$id", false, $ctx);
 }
 
-/**
- * Converts an error level (constant or bitmask) to a string description.
- */
-function severityToString(int $severity): string {
-    static $constants = [
-        'E_ERROR' => E_ERROR,
-        'E_WARNING' => E_WARNING,
-        'E_PARSE' => E_PARSE,
-        'E_NOTICE' => E_NOTICE,
-        'E_CORE_ERROR' => E_CORE_ERROR,
-        'E_CORE_WARNING' => E_CORE_WARNING,
-        'E_COMPILE_ERROR' => E_COMPILE_ERROR,
-        'E_COMPILE_WARNING' => E_COMPILE_WARNING,
-        'E_USER_ERROR' => E_USER_ERROR,
-        'E_USER_WARNING' => E_USER_WARNING,
-        'E_USER_NOTICE' => E_USER_NOTICE,
-        'E_STRICT' => E_STRICT,
-        'E_RECOVERABLE_ERROR' => E_RECOVERABLE_ERROR,
-        'E_DEPRECATED' => E_DEPRECATED,
-        'E_USER_DEPRECATED' => E_USER_DEPRECATED,
-        // E_ALL is handled separately
-    ];
-
-    if ($severity === E_ALL) {
-        return 'E_ALL';
+function severityToString($type) {
+    switch($type) {
+    case E_DEPRECATED:
+        return "E_DEPRECATED";
+    case E_RECOVERABLE_ERROR:
+        return "E_RECOVERABLE_ERROR";
+    case E_WARNING:
+        return "E_WARNING";
+    case E_NOTICE:
+        return "E_NOTICE";
+    default:
+        return "Some other #_$type";
     }
-
-    foreach ($constants as $constant => $value) {
-        if ($severity & $value) {
-            $matches[] = $constant;
-        }
-    }
-
-    return empty($matches) ? 'UNKNOWN' : implode('|', $matches);
 }
-
-/**
- * Expects the callable to raise an error matching the expected severity, which
- * may be a constant or bitmask. May optionally expect the error to be raised
- * from a particular function. Returns the message from the raised error or
- * exception, or an empty string if neither was thrown.
- */
-function raises(callable $callable, int $expectedSeverity, string $expectedFromFunction = null): string
-{
-    set_error_handler(function(int $severity, string $message, string $file, int $line) {
+function raises($function, $type, $infunction = null) {
+    $errhandler = function($severity, $message, $file, $line, $errcontext) {
         throw new ErrorException($message, 0, $severity, $file, $line);
-    });
+    };
 
+    set_error_handler($errhandler, $type);
     try {
-        call_user_func($callable);
-    } catch (ErrorException $e) {
-        if (!($e->getSeverity() & $expectedSeverity)) {
-            printf("ALMOST: Got %s - expected %s\n", severityToString($e->getSeverity()), severityToString($expectedSeverity));
-            return $e->getMessage();
+        $function();
+    } catch(Exception $e) {
+        $exceptionname = get_class($e);
+
+        if ($e instanceof ErrorException && $e->getSeverity() & $type) {
+            if ($infunction) {
+                $trace = $e->getTrace();
+                $function = $trace[0]["function"];
+                if (strcasecmp($function, $infunction) == 0) {
+                    printf("OK: Got %s thrown from %s\n", $exceptionname, $infunction);
+                } else {
+                    printf("ALMOST: Got %s - but was thrown in %s, not %s\n", $exceptionname, $function, $infunction);
+                }
+                restore_error_handler();
+                return $e->getMessage();
+            }
+            printf("OK: Got %s\n", severityToString($type));
+        } else {
+            printf("ALMOST: Got %s - expected %s\n", get_class($e), $exceptionname);
         }
-
-        if ($expectedFromFunction === null) {
-            printf("OK: Got %s\n", severityToString($e->getSeverity()));
-            return $e->getMessage();
-        }
-
-        $fromFunction = $e->getTrace()[0]['function'];
-
-        if (strcasecmp($fromFunction, $expectedFromFunction) !== 0) {
-            printf("ALMOST: Got %s - but was raised from %s, not %s\n", errorLevelToString($e->getSeverity()), $fromFunction, $expectedFromFunction);
-            return $e->getMessage();
-        }
-
-        printf("OK: Got %s raised from %s\n", severityToString($e->getSeverity()), $fromFunction);
-        return $e->getMessage();
-    } catch (Throwable $e) {
-        printf("ALMOST: Got %s - expected %s\n", get_class($e), ErrorException::class);
-        return $e->getMessage();
-    } finally {
         restore_error_handler();
-    }
-
-    printf("FAILED: Expected %s, but no error raised!\n", ErrorException::class);
-    return '';
-}
-
-/**
- * Expects the callable to throw an expected exception. May optionally expect
- * the exception to be thrown from a particular function. Returns the message
- * from the thrown exception, or an empty string if one was not thrown.
- */
-function throws(callable $callable, string $expectedException, string $expectedFromFunction = null): string
-{
-    try {
-        call_user_func($callable);
-    } catch (Throwable $e) {
-        if (!($e instanceof $expectedException)) {
-            printf("ALMOST: Got %s - expected %s\n", get_class($e), $expectedException);
-            return $e->getMessage();
-        }
-
-        if ($expectedFromFunction === null) {
-            printf("OK: Got %s\n", $expectedException);
-            return $e->getMessage();
-        }
-
-        $fromFunction = $e->getTrace()[0]['function'];
-
-        if (strcasecmp($fromFunction, $expectedFromFunction) !== 0) {
-            printf("ALMOST: Got %s - but was thrown from %s, not %s\n", $expectedException, $fromFunction, $expectedFromFunction);
-            return $e->getMessage();
-        }
-
-        printf("OK: Got %s thrown from %s\n", $expectedException, $fromFunction);
         return $e->getMessage();
     }
 
-    printf("FAILED: Expected %s, but no exception thrown!\n", $expectedException);
-    return '';
+    printf("FAILED: Expected %s thrown!\n", ErrorException::class);
+    restore_error_handler();
+}
+function throws($function, $exceptionname, $infunction = null) {
+    try {
+        $function();
+    } catch (Throwable $e) {
+    } catch (Exception $e) {
+    }
+
+    if (!isset($e)) {
+        echo "FAILED: Expected $exceptionname thrown, but no exception thrown!\n";
+        return;
+    }
+
+    $message = str_replace(array("\n", "\r"), ' ', $e->getMessage());
+    if ($e instanceof $exceptionname) {
+        if ($infunction) {
+            $trace = $e->getTrace();
+            $function = $trace[0]["function"];
+            if (strcasecmp($function, $infunction) == 0) {
+                printf("OK: Got %s thrown from %s\n", $exceptionname, $infunction);
+            } else {
+                printf("ALMOST: Got %s - but was thrown in %s, not %s (%s)\n", $exceptionname, $function, $infunction, $message);
+            }
+            return $e->getMessage();
+        }
+        printf("OK: Got %s\n", $exceptionname);
+    } else {
+        printf("ALMOST: Got %s (%s) - expected %s\n", get_class($e), $message, $exceptionname);
+    }
+
+    return $e->getMessage();
 }
 
 function printServer(Server $server)
@@ -806,6 +729,14 @@ function failMaxTimeMS(Server $server)
     configureTargetedFailPoint($server, 'maxTimeAlwaysTimeOut', [ 'times' => 1 ]);
 }
 
+function getMOPresetBase() {
+    if (!($BASE = getenv("mongodb_orchestration_base"))) {
+        $BASE = "/phongo/";
+    }
+
+    return $BASE;
+}
+
 function toPHP($var, $typemap = array()) {
     return MongoDB\BSON\toPHP($var, $typemap);
 }
@@ -833,6 +764,11 @@ function failGetMore(Manager $manager)
     $primary = $manager->selectServer(new ReadPreference('primary'));
     $version = get_server_version_from_server($primary);
 
+    if (version_compare($version, "3.2", "<")) {
+        configureFailPoint($manager, 'failReceivedGetmore', 'alwaysOn');
+        return;
+    }
+
     if (version_compare($version, "4.0", ">=")) {
         /* We use 237 here, as that's the same original code that MongoD would
          * throw if a cursor had already gone by the time we call getMore. This
